/* A recursive-descent parser generated by greg 0.4.3 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
struct _GREG;
#define YYRULECOUNT 228

/* PEG Markdown Highlight
 * Copyright 2011-2012 Ali Rantakari -- http://hasseg.org
 * Licensed under the GPL2+ and MIT licenses (see LICENSE for more info).
 * 
 * pmh_grammar.leg
 * 
 * This is a slightly adapted version of the PEG grammar from John MacFarlane's
 * peg-markdown compiler.
 */

/* PEG Markdown Highlight
 * Copyright 2011-2012 Ali Rantakari -- http://hasseg.org
 * Licensed under the GPL2+ and MIT licenses (see LICENSE for more info).
 * 
 * pmh_parser_head.c
 * 
 * Code to be inserted into the beginning of the parser code generated
 * from the PEG grammar.
 */

#include "pmh_parser.h"

#ifndef pmh_DEBUG_OUTPUT
#define pmh_DEBUG_OUTPUT 0
#endif

#if pmh_DEBUG_OUTPUT
#define pmh_IF(x)           if (x)
#define pmh_PRINTF(x, ...)  fprintf(stderr, x, ##__VA_ARGS__)
#define pmh_PUTCHAR(x)      putchar(x)
#else
#define pmh_IF(x)
#define pmh_PRINTF(x, ...)
#define pmh_PUTCHAR(x)
#endif



// Alias strdup to _strdup on MSVC:
#ifdef _MSC_VER
#define strdup _strdup
#endif



// Internal language element occurrence structure, containing
// both public and private members:
struct pmh_RealElement
{
    // "Public" members:
    // (these must match what's defined in pmh_definitions.h)
    // -----------------------------------------------
    pmh_element_type type;
    unsigned long pos;
    unsigned long end;
    struct pmh_RealElement *next;
    char *label;
    char *address;
    
    // "Private" members for use by the parser itself:
    // -----------------------------------------------
    
    // next element in list of all elements:
    struct pmh_RealElement *all_elems_next;
    
    // offset to text (for elements of type pmh_EXTRA_TEXT, used when the
    // parser reads the value of 'text'):
    int text_offset;
    
    // text content (for elements of type pmh_EXTRA_TEXT):
    char *text;
    
    // children of element (for elements of type pmh_RAW_LIST)
    struct pmh_RealElement *children;
};
typedef struct pmh_RealElement pmh_realelement;




// Parser state data:
typedef struct
{
    /* The original, unmodified UTF-8 input: */
    char *original_input;
    
    /* The offsets of the bytes we have stripped from original_input: */
    unsigned long *strip_positions;
    size_t strip_positions_len;
    
    /* Buffer of characters to be parsed: */
    char *charbuf;
    
    /* Linked list of {start, end} offset pairs determining which parts */
    /* of charbuf to actually parse: */
    pmh_realelement *current_elem;
    pmh_realelement *elem_head;
    
    /* Current parsing offset within charbuf: */
    unsigned long offset;
    
    /* The extensions to use for parsing (bitfield */
    /* of enum pmh_extensions): */
    int extensions;
    
    /* Array of parsing result elements, indexed by type: */
    pmh_realelement **head_elems;
    
    /* Whether we are parsing only references: */
    bool parsing_only_references;
    
    /* List of reference elements: */
    pmh_realelement *references;
} parser_data;

static parser_data *mk_parser_data(char *original_input,
                                   unsigned long *strip_positions,
                                   size_t strip_positions_len,
                                   char *charbuf,
                                   pmh_realelement *parsing_elems,
                                   unsigned long offset,
                                   int extensions,
                                   pmh_realelement **head_elems,
                                   pmh_realelement *references)
{
    parser_data *p_data = (parser_data *)malloc(sizeof(parser_data));
    p_data->extensions = extensions;
    p_data->original_input = original_input;
    p_data->strip_positions = strip_positions;
    p_data->strip_positions_len = strip_positions_len;
    p_data->charbuf = charbuf;
    p_data->offset = offset;
    p_data->elem_head = p_data->current_elem = parsing_elems;
    p_data->references = references;
    p_data->parsing_only_references = false;
    if (head_elems != NULL)
        p_data->head_elems = head_elems;
    else {
        p_data->head_elems = (pmh_realelement **)
                             malloc(sizeof(pmh_realelement *) * pmh_NUM_TYPES);
        int i;
        for (i = 0; i < pmh_NUM_TYPES; i++)
            p_data->head_elems[i] = NULL;
    }
    return p_data;
}


// Forward declarations
static void parse_markdown(parser_data *p_data);
static void parse_references(parser_data *p_data);





static char **get_element_type_names()
{
    static char **elem_type_names = NULL;
    if (elem_type_names == NULL)
    {
        elem_type_names = (char **)malloc(sizeof(char*) * pmh_NUM_LANG_TYPES);
        int i;
        for (i = 0; i < pmh_NUM_LANG_TYPES; i++)
            elem_type_names[i] = NULL;
        elem_type_names[pmh_LINK] = "LINK";
        elem_type_names[pmh_AUTO_LINK_URL] = "AUTO_LINK_URL";
        elem_type_names[pmh_AUTO_LINK_EMAIL] = "AUTO_LINK_EMAIL";
        elem_type_names[pmh_IMAGE] = "IMAGE";
        elem_type_names[pmh_CODE] = "CODE";
        elem_type_names[pmh_HTML] = "HTML";
        elem_type_names[pmh_HTML_ENTITY] = "HTML_ENTITY";
        elem_type_names[pmh_EMPH] = "EMPH";
        elem_type_names[pmh_STRONG] = "STRONG";
        elem_type_names[pmh_LIST_BULLET] = "LIST_BULLET";
        elem_type_names[pmh_LIST_ENUMERATOR] = "LIST_ENUMERATOR";
        elem_type_names[pmh_COMMENT] = "COMMENT";
        elem_type_names[pmh_H1] = "H1";
        elem_type_names[pmh_H2] = "H2";
        elem_type_names[pmh_H3] = "H3";
        elem_type_names[pmh_H4] = "H4";
        elem_type_names[pmh_H5] = "H5";
        elem_type_names[pmh_H6] = "H6";
        elem_type_names[pmh_BLOCKQUOTE] = "BLOCKQUOTE";
        elem_type_names[pmh_VERBATIM] = "VERBATIM";
        elem_type_names[pmh_HTMLBLOCK] = "HTMLBLOCK";
        elem_type_names[pmh_HRULE] = "HRULE";
        elem_type_names[pmh_REFERENCE] = "REFERENCE";
        elem_type_names[pmh_NOTE] = "NOTE";
    }
    return elem_type_names;
}

pmh_element_type pmh_element_type_from_name(char *name)
{
    char **elem_type_names = get_element_type_names();
    
    int i;
    for (i = 0; i < pmh_NUM_LANG_TYPES; i++)
    {
        char *i_name = elem_type_names[i];
        if (i_name == NULL)
            continue;
        if (strcmp(i_name, name) == 0)
            return (pmh_element_type)i;
    }
    
    return pmh_NO_TYPE;
}

char *pmh_element_name_from_type(pmh_element_type type)
{
    char **elem_type_names = get_element_type_names();
    char* ret = elem_type_names[type];
    if (ret == NULL)
        return "unknown type";
    return ret;
}




/*
Remove pmh_RAW elements with zero length; return pointer
to new head.
*/
static pmh_realelement *remove_zero_length_raw_spans(pmh_realelement *elem)
{
    pmh_realelement *head = elem;
    pmh_realelement *parent = NULL;
    pmh_realelement *c = head;
    while (c != NULL)
    {
        if (c->type == pmh_RAW && c->pos >= c->end)
        {
            if (parent != NULL)
                parent->next = c->next;
            else
                head = c->next;
            parent = c;
            c = c->next;
            continue;
        }
        parent = c;
        c = c->next;
    }
    return head;
}

#if pmh_DEBUG_OUTPUT
/*
Print null-terminated string s.t. some characters are
represented by their corresponding espace sequences
*/
static void print_str_literal_escapes(char *str)
{
    char *c = str;
    pmh_PRINTF("'");
    while (*c != '\0')
    {
        if (*c == '\n')         pmh_PRINTF("\\n");
        else if (*c == '\t')    pmh_PRINTF("\\t");
        else putchar(*c);
        c++;
    }
    pmh_PRINTF("'");
}
#endif

#if pmh_DEBUG_OUTPUT
/*
Print elements in a linked list of
pmh_RAW, pmh_SEPARATOR, pmh_EXTRA_TEXT elements
*/
static void print_raw_spans_inline(pmh_realelement *elem)
{
    pmh_realelement *cur = elem;
    while (cur != NULL)
    {
        if (cur->type == pmh_SEPARATOR)
            pmh_PRINTF("<pmh_SEP %ld> ", cur->pos);
        else if (cur->type == pmh_EXTRA_TEXT) {
            pmh_PRINTF("{pmh_ETEXT ");
            print_str_literal_escapes(cur->text);
            pmh_PRINTF("}");
        }
        else
            pmh_PRINTF("(%ld-%ld) ", cur->pos, cur->end);
        cur = cur->next;
    }
}
#endif

/*
Perform postprocessing parsing runs for pmh_RAW_LIST elements in `elem`,
iteratively until no such elements exist.
*/
static void process_raw_blocks(parser_data *p_data)
{
    pmh_PRINTF("--------process_raw_blocks---------\n");
    while (p_data->head_elems[pmh_RAW_LIST] != NULL)
    {
        pmh_PRINTF("new iteration.\n");
        pmh_realelement *cursor = p_data->head_elems[pmh_RAW_LIST];
        p_data->head_elems[pmh_RAW_LIST] = NULL;
        while (cursor != NULL)
        {
            pmh_realelement *span_list = (pmh_realelement*)cursor->children;
            
            span_list = remove_zero_length_raw_spans(span_list);
            
            #if pmh_DEBUG_OUTPUT
            pmh_PRINTF("  process: ");
            print_raw_spans_inline(span_list);
            pmh_PRINTF("\n");
            #endif
            
            while (span_list != NULL)
            {
                pmh_PRINTF("next: span_list: %ld-%ld\n",
                           span_list->pos, span_list->end);
                
                // Skip separators in the beginning, as well as
                // separators after another separator:
                if (span_list->type == pmh_SEPARATOR) {
                    span_list = span_list->next;
                    continue;
                }
                
                // Store list of spans until next separator in subspan_list:
                pmh_realelement *subspan_list = span_list;
                pmh_realelement *previous = NULL;
                while (span_list != NULL && span_list->type != pmh_SEPARATOR) {
                    previous = span_list;
                    span_list = span_list->next;
                }
                if (span_list != NULL && span_list->type == pmh_SEPARATOR) {
                    span_list = span_list->next;
                    previous->next = NULL;
                }
                
                #if pmh_DEBUG_OUTPUT
                pmh_PRINTF("    subspan process: ");
                print_raw_spans_inline(subspan_list);
                pmh_PRINTF("\n");
                #endif
                
                // Process subspan_list:
                parser_data *raw_p_data = mk_parser_data(
                    p_data->original_input,
                    p_data->strip_positions,
                    p_data->strip_positions_len,
                    p_data->charbuf,
                    subspan_list,
                    subspan_list->pos,
                    p_data->extensions,
                    p_data->head_elems,
                    p_data->references
                );
                parse_markdown(raw_p_data);
                free(raw_p_data);
                
                pmh_PRINTF("parse over\n");
            }
            
            cursor = cursor->next;
        }
    }
}


#if pmh_DEBUG_OUTPUT
static void print_raw_blocks(char *text, pmh_realelement *elem[])
{
    pmh_PRINTF("--------print_raw_blocks---------\n");
    pmh_PRINTF("block:\n");
    pmh_realelement *cursor = elem[pmh_RAW_LIST];
    while (cursor != NULL)
    {
        print_raw_spans_inline(cursor->children);
        cursor = cursor->next;
    }
}
#endif




/* Free all elements created while parsing */
void pmh_free_elements(pmh_element **elems)
{
    pmh_realelement *cursor = (pmh_realelement*)elems[pmh_ALL];
    while (cursor != NULL) {
        pmh_realelement *tofree = cursor;
        cursor = cursor->all_elems_next;
        if (tofree->text != NULL)
            free(tofree->text);
        if (tofree->label != NULL)
            free(tofree->label);
        if (tofree->address != NULL)
            free(tofree->address);
        free(tofree);
    }
    elems[pmh_ALL] = NULL;
    
    free(elems);
}






#define IS_CONTINUATION_BYTE(x) ((x & 0xC0) == 0x80)
#define HAS_UTF8_BOM(x)         ( ((*x & 0xFF) == 0xEF)\
                                  && ((*(x+1) & 0xFF) == 0xBB)\
                                  && ((*(x+2) & 0xFF) == 0xBF) )
#define ADD_STRIP_POS(x) \
    /* reallocate more space for the array, if needed: */ \
    if (strip_positions_size <= strip_positions_pos) { \
        size_t new_size = strip_positions_size * 2; \
        unsigned long *new_arr = (unsigned long *) \
                                 calloc(new_size, \
                                        sizeof(unsigned long)); \
        memcpy(new_arr, strip_positions, \
               (sizeof(unsigned long) * strip_positions_size)); \
        strip_positions_size = new_size; \
        free(strip_positions); \
        strip_positions = new_arr; \
    } \
    strip_positions[strip_positions_pos] = x; \
    strip_positions_pos++;

/*
Copy `str` to `out`, while doing the following:
  - remove UTF-8 continuation bytes
  - remove possible UTF-8 BOM (byte order mark)
  - append two newlines to the end (like peg-markdown does)
  - keep track of which bytes we have stripped (in strip_positions)
*/
static int strcpy_preformat(char *str, char **out,
                            unsigned long **out_strip_positions,
                            size_t *out_strip_positions_len)
{
    size_t strip_positions_size = 1024;
    size_t strip_positions_pos = 0;
    unsigned long *strip_positions = (unsigned long *)
                                     calloc(strip_positions_size,
                                            sizeof(unsigned long));
    
    
    // +2 in the following is due to the "\n\n" suffix:
    char *new_str = (char *)malloc(sizeof(char) * strlen(str) + 1 + 2);
    char *c = str;
    int i = 0;
    
    if (HAS_UTF8_BOM(c)) {
        c += 3;
        ADD_STRIP_POS(0);
        ADD_STRIP_POS(1);
        ADD_STRIP_POS(2);
    }
    
    while (*c != '\0')
    {
        if (!IS_CONTINUATION_BYTE(*c)) {
            *(new_str+i) = *c, i++;
        } else {
            ADD_STRIP_POS((int)(c-str));
        }
        c++;
    }
    
    *(new_str+(i++)) = '\n';
    *(new_str+(i++)) = '\n';
    *(new_str+i) = '\0';
    
    *out = new_str;
    *out_strip_positions = strip_positions;
    *out_strip_positions_len = strip_positions_pos;
    return i;
}



void pmh_markdown_to_elements(char *text, int extensions,
                              pmh_element **out_result[])
{
    char *text_copy = NULL;
    unsigned long *strip_positions = NULL;
    size_t strip_positions_len = 0;
    int text_copy_len = strcpy_preformat(text, &text_copy, &strip_positions,
                                         &strip_positions_len);
    
    pmh_realelement *parsing_elem = (pmh_realelement *)
                                    malloc(sizeof(pmh_realelement));
    parsing_elem->type = pmh_RAW;
    parsing_elem->pos = 0;
    parsing_elem->end = text_copy_len;
    parsing_elem->next = NULL;
    
    parser_data *p_data = mk_parser_data(
        text,
        strip_positions,
        strip_positions_len,
        text_copy,
        parsing_elem,
        0,
        extensions,
        NULL,
        NULL
    );
    pmh_realelement **result = p_data->head_elems;
    
    if (*text_copy != '\0')
    {
        // Get reference definitions into p_data->references
        parse_references(p_data);
        
        // Reset parser state to beginning of input
        p_data->offset = 0;
        p_data->current_elem = p_data->elem_head;
        
        // Parse whole document
        parse_markdown(p_data);
        
        #if pmh_DEBUG_OUTPUT
        print_raw_blocks(text_copy, result);
        #endif
        
        process_raw_blocks(p_data);
    }
    
    free(strip_positions);
    free(p_data);
    free(parsing_elem);
    free(text_copy);
    
    *out_result = (pmh_element**)result;
}



/*
Mergesort linked list of elements (using comparison function `compare`),
return new head. (Adapted slightly from Simon Tatham's algorithm.)
*/
static pmh_element *ll_mergesort(pmh_element *list,
                                 int (*compare)(const pmh_element*,
                                                const pmh_element*))
{
    if (!list)
        return NULL;
    
    pmh_element *out_head = list;
    
    /* Merge widths of doubling size until done */
    int merge_width = 1;
    while (1)
    {
        pmh_element *l, *r; /* left & right segment pointers */
        pmh_element *tail = NULL; /* tail of sorted section */
        
        l = out_head;
        out_head = NULL;
        
        int merge_count = 0;
        
        while (l)
        {
            merge_count++;
            
            /* Position r, determine lsize & rsize */
            r = l;
            int lsize = 0;
            int i;
            for (i = 0; i < merge_width; i++) {
                lsize++;
                r = r->next;
                if (!r)
                    break;
            }
            int rsize = merge_width;
            
            /* Merge l & r */
            while (lsize > 0 || (rsize > 0 && r))
            {
                bool get_from_left = false;
                if (lsize == 0)             get_from_left = false;
                else if (rsize == 0 || !r)  get_from_left = true;
                else if (compare(l,r) <= 0) get_from_left = true;
                
                pmh_element *e;
                if (get_from_left) {
                    e = l; l = l->next; lsize--;
                } else {
                    e = r; r = r->next; rsize--;
                }
                
                /* add the next pmh_element to the merged list */
                if (tail)
                    tail->next = e;
                else
                    out_head = e;
                tail = e;
            }
            
            l = r;
        }
        tail->next = NULL;
        
        if (merge_count <= 1)
            return out_head;
        
        merge_width *= 2;
    }
}

static int elem_compare_by_pos(const pmh_element *a, const pmh_element *b)
{
    return a->pos - b->pos;
}

void pmh_sort_elements_by_pos(pmh_element *element_lists[])
{
    int i;
    for (i = 0; i < pmh_NUM_LANG_TYPES; i++)
        element_lists[i] = ll_mergesort(element_lists[i], &elem_compare_by_pos);
}








/* return true if extension is selected */
static bool extension(parser_data *p_data, int ext)
{
    return ((p_data->extensions & ext) != 0);
}

/* return reference pmh_realelement for a given label */
static pmh_realelement *get_reference(parser_data *p_data, char *label)
{
    if (!label)
        return NULL;
    
    pmh_realelement *cursor = p_data->references;
    while (cursor != NULL)
    {
        if (cursor->label && strcmp(label, cursor->label) == 0)
            return cursor;
        cursor = cursor->next;
    }
    return NULL;
}


/* cons an element/list onto a list, returning pointer to new head */
static pmh_realelement *cons(pmh_realelement *elem, pmh_realelement *list)
{
    assert(elem != NULL);
    
    pmh_realelement *cur = elem;
    while (cur->next != NULL) {
        cur = cur->next;
    }
    cur->next = list;
    
    return elem;
}


/* reverse a list, returning pointer to new list */
static pmh_realelement *reverse(pmh_realelement *list)
{
    pmh_realelement *new_head = NULL;
    pmh_realelement *next = NULL;
    while (list != NULL) {
        next = list->next;
        list->next = new_head;
        new_head = list;
        list = next;
    }
    return new_head;
}



/* construct pmh_realelement */
static pmh_realelement *mk_element(parser_data *p_data, pmh_element_type type,
                                   long pos, long end)
{
    pmh_realelement *result = (pmh_realelement *)malloc(sizeof(pmh_realelement));
    result->type = type;
    result->pos = pos;
    result->end = end;
    result->next = NULL;
    result->text_offset = 0;
    result->label = result->address = result->text = NULL;
    
    pmh_realelement *old_all_elements_head = p_data->head_elems[pmh_ALL];
    p_data->head_elems[pmh_ALL] = result;
    result->all_elems_next = old_all_elements_head;
    
    //pmh_PRINTF("  mk_element: %s [%ld - %ld]\n", pmh_element_name_from_type(type), pos, end);
    
    return result;
}

static pmh_realelement *copy_element(parser_data *p_data, pmh_realelement *elem)
{
    pmh_realelement *result = mk_element(p_data, elem->type, elem->pos, elem->end);
    result->label = (elem->label == NULL) ? NULL : strdup(elem->label);
    result->text = (elem->text == NULL) ? NULL : strdup(elem->text);
    result->address = (elem->address == NULL) ? NULL : strdup(elem->address);
    return result;
}

/* construct pmh_EXTRA_TEXT pmh_realelement */
static pmh_realelement *mk_etext(parser_data *p_data, char *string)
{
    pmh_realelement *result;
    assert(string != NULL);
    result = mk_element(p_data, pmh_EXTRA_TEXT, 0,0);
    result->text = strdup(string);
    return result;
}


/*
Given an element where the offsets {pos, end} represent
locations in the *parsed text* (defined by the linked list of pmh_RAW and
pmh_EXTRA_TEXT elements in p_data->current_elem), fix these offsets to represent
corresponding offsets in the parse buffer (p_data->charbuf). Also split
the given pmh_realelement into multiple parts if its offsets span multiple
p_data->current_elem elements. Return the (list of) elements with real offsets.
*/
static pmh_realelement *fix_offsets(parser_data *p_data, pmh_realelement *elem)
{
    if (elem->type == pmh_EXTRA_TEXT)
        return mk_etext(p_data, elem->text);
    
    pmh_realelement *new_head = copy_element(p_data, elem);
    
    pmh_realelement *tail = new_head;
    pmh_realelement *prev = NULL;
    
    bool found_start = false;
    bool found_end = false;
    bool tail_needs_pos = false;
    unsigned long previous_end = 0;
    unsigned long c = 0;
    
    pmh_realelement *cursor = p_data->elem_head;
    while (cursor != NULL)
    {
        int thislen = (cursor->type == pmh_EXTRA_TEXT)
                        ? strlen(cursor->text)
                        : cursor->end - cursor->pos;
        
        if (tail_needs_pos && cursor->type != pmh_EXTRA_TEXT) {
            tail->pos = cursor->pos;
            tail_needs_pos = false;
        }
        
        unsigned int this_pos = cursor->pos;
        
        if (!found_start && (c <= elem->pos && elem->pos <= c+thislen)) {
            tail->pos = (cursor->type == pmh_EXTRA_TEXT)
                        ? previous_end
                        : cursor->pos + (elem->pos - c);
            this_pos = tail->pos;
            found_start = true;
        }
        
        if (!found_end && (c <= elem->end && elem->end <= c+thislen)) {
            tail->end = (cursor->type == pmh_EXTRA_TEXT)
                        ? previous_end
                        : cursor->pos + (elem->end - c);
            found_end = true;
        }
        
        if (found_start && found_end)
            break;
        
        if (cursor->type != pmh_EXTRA_TEXT)
            previous_end = cursor->end;
        
        if (found_start) {
            pmh_realelement *new_elem = copy_element(p_data, tail);
            new_elem->pos = this_pos;
            new_elem->end = cursor->end;
            new_elem->next = tail;
            if (prev != NULL)
                prev->next = new_elem;
            if (new_head == tail)
                new_head = new_elem;
            prev = new_elem;
            tail_needs_pos = true;
        }
        
        c += thislen;
        cursor = cursor->next;
    }
    
    return new_head;
}



/* Add an element to p_data->head_elems. */
static void add(parser_data *p_data, pmh_realelement *elem)
{
    if (elem->type != pmh_RAW_LIST)
    {
        pmh_PRINTF("  add: %s [%ld - %ld]\n",
                   pmh_element_name_from_type(elem->type), elem->pos, elem->end);
        elem = fix_offsets(p_data, elem);
        pmh_PRINTF("     > %s [%ld - %ld]\n",
                   pmh_element_name_from_type(elem->type), elem->pos, elem->end);
    }
    else
    {
        pmh_PRINTF("  add: pmh_RAW_LIST ");
        pmh_realelement *cursor = elem->children;
        pmh_realelement *previous = NULL;
        while (cursor != NULL)
        {
            pmh_realelement *next = cursor->next;
            pmh_PRINTF("(%ld-%ld)>", cursor->pos, cursor->end);
            pmh_realelement *new_cursor = fix_offsets(p_data, cursor);
            if (previous != NULL)
                previous->next = new_cursor;
            else
                elem->children = new_cursor;
            pmh_PRINTF("(%ld-%ld)", new_cursor->pos, new_cursor->end);
            while (new_cursor->next != NULL) {
                new_cursor = new_cursor->next;
                pmh_PRINTF("(%ld-%ld)", new_cursor->pos, new_cursor->end);
            }
            pmh_PRINTF(" ");
            if (next != NULL)
                new_cursor->next = next;
            previous = new_cursor;
            cursor = next;
        }
        pmh_PRINTF("\n");
    }
    
    if (p_data->head_elems[elem->type] == NULL)
        p_data->head_elems[elem->type] = elem;
    else
    {
        pmh_realelement *last = elem;
        while (last->next != NULL)
            last = last->next;
        last->next = p_data->head_elems[elem->type];
        p_data->head_elems[elem->type] = elem;
    }
}


// Given a range in the list of spans we use for parsing (pos, end), return
// a copy of the corresponding section in the original input, with all of
// the UTF-8 bytes intact:
static char *copy_input_span(parser_data *p_data,
                             unsigned long pos, unsigned long end)
{
    if (end <= pos)
        return NULL;
    
    char *ret = NULL;
    
    // Adjust (pos,end) to match actual indexes in charbuf:
    pmh_realelement *dummy = mk_element(p_data, pmh_NO_TYPE, pos, end);
    pmh_realelement *fixed_dummies = fix_offsets(p_data, dummy);
    pmh_realelement *cursor = fixed_dummies;
    while (cursor != NULL)
    {
        if (cursor->end <= cursor->pos)
        {
            cursor = cursor->next;
            continue;
        }
        
        // Adjust cursor's span to take bytes stripped from the original
        // input into account (i.e. match the corresponding span in
        // p_data->original_input):
        unsigned long adjusted_pos = cursor->pos;
        unsigned long adjusted_end = cursor->end;
        size_t i;
        for (i = 0; i < p_data->strip_positions_len; i++)
        {
            unsigned long strip_position = p_data->strip_positions[i];
            if (strip_position <= adjusted_pos)
                adjusted_pos++;
            if (strip_position <= adjusted_end)
                adjusted_end++;
            else
                break;
        }
        
        // Copy span from original input:
        size_t adjusted_len = adjusted_end - adjusted_pos;
        char *str = (char *)malloc(sizeof(char)*adjusted_len + 1);
        *str = '\0';
        strncat(str, (p_data->original_input + adjusted_pos), adjusted_len);
        
        if (ret == NULL)
            ret = str;
        else
        {
            // append str to ret:
            char *new_ret = (char *)malloc(sizeof(char)
                                           *(strlen(str) + strlen(ret)) + 1);
            *new_ret = '\0';
            strcat(new_ret, ret);
            strcat(new_ret, str);
            free(ret);
            free(str);
            ret = new_ret;
        }
        
        cursor = cursor->next;
    }
    
    return ret;
}





# define YYSTYPE pmh_realelement *
#ifdef __DEBUG__
# define YY_DEBUG 1
#endif

#define YY_INPUT(buf, result, max_size)\
        yy_input_func(buf, &result, max_size, (parser_data *)G->data)

static void yy_input_func(char *buf, int *result, int max_size,
                          parser_data *p_data)
{
    if (p_data->current_elem == NULL)
    {
        (*result) = 0;
        return;
    }
    
    if (p_data->current_elem->type == pmh_EXTRA_TEXT)
    {
        int yyc;
        bool moreToRead = (p_data->current_elem->text
                           && *(p_data->current_elem->text
                                + p_data->current_elem->text_offset) != '\0');
        if (moreToRead)
        {
            yyc = *(p_data->current_elem->text + p_data->current_elem->text_offset++);
            pmh_PRINTF("\e[47;30m"); pmh_PUTCHAR(yyc); pmh_PRINTF("\e[0m");
            pmh_IF(yyc == '\n') pmh_PRINTF("\e[47m \e[0m");
        }
        else
        {
            yyc = EOF;
            p_data->current_elem = p_data->current_elem->next;
            pmh_PRINTF("\e[41m \e[0m");
            if (p_data->current_elem != NULL)
                p_data->offset = p_data->current_elem->pos;
        }
        (*result) = (EOF == yyc) ? 0 : (*(buf) = yyc, 1);
        return;
    }
    
    *(buf) = *(p_data->charbuf + p_data->offset);
    (*result) = (*buf != '\0');
    p_data->offset++;
    
    pmh_PRINTF("\e[43;30m"); pmh_PUTCHAR(*buf); pmh_PRINTF("\e[0m");
    pmh_IF(*buf == '\n') pmh_PRINTF("\e[42m \e[0m");
    
    if (p_data->offset >= p_data->current_elem->end)
    {
        p_data->current_elem = p_data->current_elem->next;
        pmh_PRINTF("\e[41m \e[0m");
        if (p_data->current_elem != NULL)
            p_data->offset = p_data->current_elem->pos;
    }
}



#define elem(x)     mk_element((parser_data *)G->data, x, thunk->begin, thunk->end)
#define elem_s(x)   mk_element((parser_data *)G->data, x, s->pos, thunk->end)
#define mk_sep      mk_element((parser_data *)G->data, pmh_SEPARATOR, 0,0)
#define mk_notype   mk_element((parser_data *)G->data, pmh_NO_TYPE, 0,0)
#define etext(x)    mk_etext((parser_data *)G->data, x)
#define ADD(x)      add((parser_data *)G->data, x)
#define EXT(x)      extension((parser_data *)G->data, x)
#define REF_EXISTS(x) reference_exists((parser_data *)G->data, x)
#define GET_REF(x)  get_reference((parser_data *)G->data, x)
#define PARSING_REFERENCES ((parser_data *)G->data)->parsing_only_references
#define FREE_LABEL(l) { free(l->label); l->label = NULL; }
#define FREE_ADDRESS(l) { free(l->address); l->address = NULL; }

// This gives us the text matched with < > as it appears in the original input:
#define COPY_YYTEXT_ORIG() copy_input_span((parser_data *)G->data, thunk->begin, thunk->end)


#ifndef YY_ALLOC
#define YY_ALLOC(N, D) malloc(N)
#endif
#ifndef YY_CALLOC
#define YY_CALLOC(N, S, D) calloc(N, S)
#endif
#ifndef YY_REALLOC
#define YY_REALLOC(B, N, D) realloc(B, N)
#endif
#ifndef YY_FREE
#define YY_FREE free
#endif
#ifndef YY_LOCAL
#define YY_LOCAL(T)     static T
#endif
#ifndef YY_ACTION
#define YY_ACTION(T)    static T
#endif
#ifndef YY_RULE
#define YY_RULE(T)      static T
#endif
#ifndef YY_PARSE
#define YY_PARSE(T)     T
#endif
#ifndef YY_NAME
#define YY_NAME(N) yy##N
#endif
#ifndef YY_INPUT
#define YY_INPUT(buf, result, max_size)                 \
  {                                                     \
    int yyc= getchar();                                 \
    result= (EOF == yyc) ? 0 : (*(buf)= yyc, 1);        \
    yyprintf((stderr, "<%c>", yyc));                  \
  }
#endif
#ifndef YY_BEGIN
#define YY_BEGIN        ( G->begin= G->pos, 1)
#endif
#ifndef YY_END
#define YY_END          ( G->end= G->pos, 1)
#endif
#ifdef YY_DEBUG
# define yyprintf(args) fprintf args
#else
# define yyprintf(args)
#endif
#ifndef YYSTYPE
#define YYSTYPE int
#endif
#ifndef YY_XTYPE
#define YY_XTYPE void *
#endif
#ifndef YY_XVAR
#define YY_XVAR yyxvar
#endif

#ifndef YY_STACK_SIZE
#define YY_STACK_SIZE 128
#endif

#ifndef YY_BUFFER_START_SIZE
#define YY_BUFFER_START_SIZE 1024
#endif

#ifndef YY_PART
#define yydata G->data
#define yy G->ss

struct _yythunk; // forward declaration
typedef void (*yyaction)(struct _GREG *G, char *yytext, int yyleng, struct _yythunk *thunkpos, YY_XTYPE YY_XVAR);
typedef struct _yythunk { int begin, end;  yyaction  action;  struct _yythunk *next; } yythunk;

typedef struct _GREG {
  char *buf;
  int buflen;
  int   offset;
  int   pos;
  int   limit;
  char *text;
  int   textlen;
  int   begin;
  int   end;
  yythunk *thunks;
  int   thunkslen;
  int thunkpos;
  YYSTYPE ss;
  YYSTYPE *val;
  YYSTYPE *vals;
  int valslen;
  YY_XTYPE data;
} GREG;

YY_LOCAL(int) yyrefill(GREG *G)
{
  int yyn;
  while (G->buflen - G->pos < 512)
    {
      G->buflen *= 2;
      G->buf= (char *)YY_REALLOC(G->buf, G->buflen, G->data);
    }
  YY_INPUT((G->buf + G->pos), yyn, (G->buflen - G->pos));
  if (!yyn) return 0;
  G->limit += yyn;
  return 1;
}

YY_LOCAL(int) yymatchDot(GREG *G)
{
  if (G->pos >= G->limit && !yyrefill(G)) return 0;
  ++G->pos;
  return 1;
}

YY_LOCAL(int) yymatchChar(GREG *G, int c)
{
  if (G->pos >= G->limit && !yyrefill(G)) return 0;
  if ((unsigned char)G->buf[G->pos] == c)
    {
      ++G->pos;
      yyprintf((stderr, "  ok   yymatchChar(%c) @ %s\n", c, G->buf+G->pos));
      return 1;
    }
  yyprintf((stderr, "  fail yymatchChar(%c) @ %s\n", c, G->buf+G->pos));
  return 0;
}

YY_LOCAL(int) yymatchString(GREG *G, char *s)
{
  int yysav= G->pos;
  while (*s)
    {
      if (G->pos >= G->limit && !yyrefill(G)) return 0;
      if (G->buf[G->pos] != *s)
        {
          G->pos= yysav;
          return 0;
        }
      ++s;
      ++G->pos;
    }
  return 1;
}

YY_LOCAL(int) yymatchClass(GREG *G, unsigned char *bits)
{
  int c;
  if (G->pos >= G->limit && !yyrefill(G)) return 0;
  c= (unsigned char)G->buf[G->pos];
  if (bits[c >> 3] & (1 << (c & 7)))
    {
      ++G->pos;
      yyprintf((stderr, "  ok   yymatchClass @ %s\n", G->buf+G->pos));
      return 1;
    }
  yyprintf((stderr, "  fail yymatchClass @ %s\n", G->buf+G->pos));
  return 0;
}

YY_LOCAL(void) yyDo(GREG *G, yyaction action, int begin, int end)
{
  while (G->thunkpos >= G->thunkslen)
    {
      G->thunkslen *= 2;
      G->thunks= (yythunk *)YY_REALLOC(G->thunks, sizeof(yythunk) * G->thunkslen, G->data);
    }
  G->thunks[G->thunkpos].begin=  begin;
  G->thunks[G->thunkpos].end=    end;
  G->thunks[G->thunkpos].action= action;
  ++G->thunkpos;
}

YY_LOCAL(int) yyText(GREG *G, int begin, int end)
{
  int yyleng= end - begin;
  if (yyleng <= 0)
    yyleng= 0;
  else
    {
      while (G->textlen < (yyleng + 1))
        {
          G->textlen *= 2;
          G->text= (char *)YY_REALLOC(G->text, G->textlen, G->data);
        }
      memcpy(G->text, G->buf + begin, yyleng);
    }
  G->text[yyleng]= '\0';
  return yyleng;
}

YY_LOCAL(void) yyDone(GREG *G)
{
  int pos;
  for (pos= 0; pos < G->thunkpos; ++pos)
    {
      yythunk *thunk= &G->thunks[pos];
      int yyleng= thunk->end ? yyText(G, thunk->begin, thunk->end) : thunk->begin;
      yyprintf((stderr, "DO [%d] %p %s\n", pos, thunk->action, G->text));
      thunk->action(G, G->text, yyleng, thunk, G->data);
    }
  G->thunkpos= 0;
}

YY_LOCAL(void) yyCommit(GREG *G)
{
  if ((G->limit -= G->pos))
    {
      memmove(G->buf, G->buf + G->pos, G->limit);
    }
  G->offset += G->pos;
  G->begin -= G->pos;
  G->end -= G->pos;
  G->pos= G->thunkpos= 0;
}

YY_LOCAL(int) yyAccept(GREG *G, int tp0)
{
  if (tp0)
    {
      fprintf(stderr, "accept denied at %d\n", tp0);
      return 0;
    }
  else
    {
      yyDone(G);
      yyCommit(G);
    }
  return 1;
}

YY_LOCAL(void) yyPush(GREG *G, char *text, int count, yythunk *thunk, YY_XTYPE YY_XVAR) { G->val += count; }
YY_LOCAL(void) yyPop(GREG *G, char *text, int count, yythunk *thunk, YY_XTYPE YY_XVAR)  { G->val -= count; }
YY_LOCAL(void) yySet(GREG *G, char *text, int count, yythunk *thunk, YY_XTYPE YY_XVAR)  { G->val[count]= G->ss; }

#endif /* YY_PART */

#define YYACCEPT        yyAccept(G, yythunkpos0)

YY_RULE(int) yy_RawNoteBlock(GREG *G); /* 228 */
YY_RULE(int) yy_RawNoteReference(GREG *G); /* 227 */
YY_RULE(int) yy_ExtendedSpecialChar(GREG *G); /* 226 */
YY_RULE(int) yy_AlphanumericAscii(GREG *G); /* 225 */
YY_RULE(int) yy_Quoted(GREG *G); /* 224 */
YY_RULE(int) yy_HtmlTag(GREG *G); /* 223 */
YY_RULE(int) yy_Ticks5(GREG *G); /* 222 */
YY_RULE(int) yy_Ticks4(GREG *G); /* 221 */
YY_RULE(int) yy_Ticks3(GREG *G); /* 220 */
YY_RULE(int) yy_Ticks2(GREG *G); /* 219 */
YY_RULE(int) yy_Ticks1(GREG *G); /* 218 */
YY_RULE(int) yy_SkipBlock(GREG *G); /* 217 */
YY_RULE(int) yy_References(GREG *G); /* 216 */
YY_RULE(int) yy_EmptyTitle(GREG *G); /* 215 */
YY_RULE(int) yy_RefTitleParens(GREG *G); /* 214 */
YY_RULE(int) yy_RefTitleDouble(GREG *G); /* 213 */
YY_RULE(int) yy_RefTitleSingle(GREG *G); /* 212 */
YY_RULE(int) yy_RefTitle(GREG *G); /* 211 */
YY_RULE(int) yy_RefSrc(GREG *G); /* 210 */
YY_RULE(int) yy_AutoLinkEmail(GREG *G); /* 209 */
YY_RULE(int) yy_AutoLinkUrl(GREG *G); /* 208 */
YY_RULE(int) yy_TitleDouble(GREG *G); /* 207 */
YY_RULE(int) yy_TitleSingle(GREG *G); /* 206 */
YY_RULE(int) yy_Nonspacechar(GREG *G); /* 205 */
YY_RULE(int) yy_SourceContents(GREG *G); /* 204 */
YY_RULE(int) yy_Title(GREG *G); /* 203 */
YY_RULE(int) yy_Source(GREG *G); /* 202 */
YY_RULE(int) yy_Label(GREG *G); /* 201 */
YY_RULE(int) yy_ReferenceLinkSingle(GREG *G); /* 200 */
YY_RULE(int) yy_ReferenceLinkDouble(GREG *G); /* 199 */
YY_RULE(int) yy_AutoLink(GREG *G); /* 198 */
YY_RULE(int) yy_ReferenceLink(GREG *G); /* 197 */
YY_RULE(int) yy_ExplicitLink(GREG *G); /* 196 */
YY_RULE(int) yy_TwoUlClose(GREG *G); /* 195 */
YY_RULE(int) yy_TwoUlOpen(GREG *G); /* 194 */
YY_RULE(int) yy_TwoStarClose(GREG *G); /* 193 */
YY_RULE(int) yy_TwoStarOpen(GREG *G); /* 192 */
YY_RULE(int) yy_StrongUl(GREG *G); /* 191 */
YY_RULE(int) yy_OneUlClose(GREG *G); /* 190 */
YY_RULE(int) yy_OneUlOpen(GREG *G); /* 189 */
YY_RULE(int) yy_StrongStar(GREG *G); /* 188 */
YY_RULE(int) yy_OneStarClose(GREG *G); /* 187 */
YY_RULE(int) yy_OneStarOpen(GREG *G); /* 186 */
YY_RULE(int) yy_EmphUl(GREG *G); /* 185 */
YY_RULE(int) yy_EmphStar(GREG *G); /* 184 */
YY_RULE(int) yy_StarLine(GREG *G); /* 183 */
YY_RULE(int) yy_UlLine(GREG *G); /* 182 */
YY_RULE(int) yy_SpecialChar(GREG *G); /* 181 */
YY_RULE(int) yy_Eof(GREG *G); /* 180 */
YY_RULE(int) yy_NormalEndline(GREG *G); /* 179 */
YY_RULE(int) yy_TerminalEndline(GREG *G); /* 178 */
YY_RULE(int) yy_LineBreak(GREG *G); /* 177 */
YY_RULE(int) yy_CharEntity(GREG *G); /* 176 */
YY_RULE(int) yy_DecEntity(GREG *G); /* 175 */
YY_RULE(int) yy_HexEntity(GREG *G); /* 174 */
YY_RULE(int) yy_Alphanumeric(GREG *G); /* 173 */
YY_RULE(int) yy_NormalChar(GREG *G); /* 172 */
YY_RULE(int) yy_Symbol(GREG *G); /* 171 */
YY_RULE(int) yy_EscapedChar(GREG *G); /* 170 */
YY_RULE(int) yy_Entity(GREG *G); /* 169 */
YY_RULE(int) yy_RawHtml(GREG *G); /* 168 */
YY_RULE(int) yy_Code(GREG *G); /* 167 */
YY_RULE(int) yy_InlineNote(GREG *G); /* 166 */
YY_RULE(int) yy_NoteReference(GREG *G); /* 165 */
YY_RULE(int) yy_Link(GREG *G); /* 164 */
YY_RULE(int) yy_Image(GREG *G); /* 163 */
YY_RULE(int) yy_Emph(GREG *G); /* 162 */
YY_RULE(int) yy_Strong(GREG *G); /* 161 */
YY_RULE(int) yy_Space(GREG *G); /* 160 */
YY_RULE(int) yy_UlOrStarLine(GREG *G); /* 159 */
YY_RULE(int) yy_Str(GREG *G); /* 158 */
YY_RULE(int) yy_InStyleTags(GREG *G); /* 157 */
YY_RULE(int) yy_StyleClose(GREG *G); /* 156 */
YY_RULE(int) yy_StyleOpen(GREG *G); /* 155 */
YY_RULE(int) yy_HtmlBlockType(GREG *G); /* 154 */
YY_RULE(int) yy_HtmlBlockSelfClosing(GREG *G); /* 153 */
YY_RULE(int) yy_HtmlComment(GREG *G); /* 152 */
YY_RULE(int) yy_HtmlBlockInTags(GREG *G); /* 151 */
YY_RULE(int) yy_HtmlBlockScript(GREG *G); /* 150 */
YY_RULE(int) yy_HtmlBlockCloseScript(GREG *G); /* 149 */
YY_RULE(int) yy_HtmlBlockOpenScript(GREG *G); /* 148 */
YY_RULE(int) yy_HtmlBlockTr(GREG *G); /* 147 */
YY_RULE(int) yy_HtmlBlockCloseTr(GREG *G); /* 146 */
YY_RULE(int) yy_HtmlBlockOpenTr(GREG *G); /* 145 */
YY_RULE(int) yy_HtmlBlockThead(GREG *G); /* 144 */
YY_RULE(int) yy_HtmlBlockCloseThead(GREG *G); /* 143 */
YY_RULE(int) yy_HtmlBlockOpenThead(GREG *G); /* 142 */
YY_RULE(int) yy_HtmlBlockTh(GREG *G); /* 141 */
YY_RULE(int) yy_HtmlBlockCloseTh(GREG *G); /* 140 */
YY_RULE(int) yy_HtmlBlockOpenTh(GREG *G); /* 139 */
YY_RULE(int) yy_HtmlBlockTfoot(GREG *G); /* 138 */
YY_RULE(int) yy_HtmlBlockCloseTfoot(GREG *G); /* 137 */
YY_RULE(int) yy_HtmlBlockOpenTfoot(GREG *G); /* 136 */
YY_RULE(int) yy_HtmlBlockTd(GREG *G); /* 135 */
YY_RULE(int) yy_HtmlBlockCloseTd(GREG *G); /* 134 */
YY_RULE(int) yy_HtmlBlockOpenTd(GREG *G); /* 133 */
YY_RULE(int) yy_HtmlBlockTbody(GREG *G); /* 132 */
YY_RULE(int) yy_HtmlBlockCloseTbody(GREG *G); /* 131 */
YY_RULE(int) yy_HtmlBlockOpenTbody(GREG *G); /* 130 */
YY_RULE(int) yy_HtmlBlockLi(GREG *G); /* 129 */
YY_RULE(int) yy_HtmlBlockCloseLi(GREG *G); /* 128 */
YY_RULE(int) yy_HtmlBlockOpenLi(GREG *G); /* 127 */
YY_RULE(int) yy_HtmlBlockFrameset(GREG *G); /* 126 */
YY_RULE(int) yy_HtmlBlockCloseFrameset(GREG *G); /* 125 */
YY_RULE(int) yy_HtmlBlockOpenFrameset(GREG *G); /* 124 */
YY_RULE(int) yy_HtmlBlockDt(GREG *G); /* 123 */
YY_RULE(int) yy_HtmlBlockCloseDt(GREG *G); /* 122 */
YY_RULE(int) yy_HtmlBlockOpenDt(GREG *G); /* 121 */
YY_RULE(int) yy_HtmlBlockDd(GREG *G); /* 120 */
YY_RULE(int) yy_HtmlBlockCloseDd(GREG *G); /* 119 */
YY_RULE(int) yy_HtmlBlockOpenDd(GREG *G); /* 118 */
YY_RULE(int) yy_HtmlBlockUl(GREG *G); /* 117 */
YY_RULE(int) yy_HtmlBlockCloseUl(GREG *G); /* 116 */
YY_RULE(int) yy_HtmlBlockOpenUl(GREG *G); /* 115 */
YY_RULE(int) yy_HtmlBlockTable(GREG *G); /* 114 */
YY_RULE(int) yy_HtmlBlockCloseTable(GREG *G); /* 113 */
YY_RULE(int) yy_HtmlBlockOpenTable(GREG *G); /* 112 */
YY_RULE(int) yy_HtmlBlockPre(GREG *G); /* 111 */
YY_RULE(int) yy_HtmlBlockClosePre(GREG *G); /* 110 */
YY_RULE(int) yy_HtmlBlockOpenPre(GREG *G); /* 109 */
YY_RULE(int) yy_HtmlBlockP(GREG *G); /* 108 */
YY_RULE(int) yy_HtmlBlockCloseP(GREG *G); /* 107 */
YY_RULE(int) yy_HtmlBlockOpenP(GREG *G); /* 106 */
YY_RULE(int) yy_HtmlBlockOl(GREG *G); /* 105 */
YY_RULE(int) yy_HtmlBlockCloseOl(GREG *G); /* 104 */
YY_RULE(int) yy_HtmlBlockOpenOl(GREG *G); /* 103 */
YY_RULE(int) yy_HtmlBlockNoscript(GREG *G); /* 102 */
YY_RULE(int) yy_HtmlBlockCloseNoscript(GREG *G); /* 101 */
YY_RULE(int) yy_HtmlBlockOpenNoscript(GREG *G); /* 100 */
YY_RULE(int) yy_HtmlBlockNoframes(GREG *G); /* 99 */
YY_RULE(int) yy_HtmlBlockCloseNoframes(GREG *G); /* 98 */
YY_RULE(int) yy_HtmlBlockOpenNoframes(GREG *G); /* 97 */
YY_RULE(int) yy_HtmlBlockMenu(GREG *G); /* 96 */
YY_RULE(int) yy_HtmlBlockCloseMenu(GREG *G); /* 95 */
YY_RULE(int) yy_HtmlBlockOpenMenu(GREG *G); /* 94 */
YY_RULE(int) yy_HtmlBlockH6(GREG *G); /* 93 */
YY_RULE(int) yy_HtmlBlockCloseH6(GREG *G); /* 92 */
YY_RULE(int) yy_HtmlBlockOpenH6(GREG *G); /* 91 */
YY_RULE(int) yy_HtmlBlockH5(GREG *G); /* 90 */
YY_RULE(int) yy_HtmlBlockCloseH5(GREG *G); /* 89 */
YY_RULE(int) yy_HtmlBlockOpenH5(GREG *G); /* 88 */
YY_RULE(int) yy_HtmlBlockH4(GREG *G); /* 87 */
YY_RULE(int) yy_HtmlBlockCloseH4(GREG *G); /* 86 */
YY_RULE(int) yy_HtmlBlockOpenH4(GREG *G); /* 85 */
YY_RULE(int) yy_HtmlBlockH3(GREG *G); /* 84 */
YY_RULE(int) yy_HtmlBlockCloseH3(GREG *G); /* 83 */
YY_RULE(int) yy_HtmlBlockOpenH3(GREG *G); /* 82 */
YY_RULE(int) yy_HtmlBlockH2(GREG *G); /* 81 */
YY_RULE(int) yy_HtmlBlockCloseH2(GREG *G); /* 80 */
YY_RULE(int) yy_HtmlBlockOpenH2(GREG *G); /* 79 */
YY_RULE(int) yy_HtmlBlockH1(GREG *G); /* 78 */
YY_RULE(int) yy_HtmlBlockCloseH1(GREG *G); /* 77 */
YY_RULE(int) yy_HtmlBlockOpenH1(GREG *G); /* 76 */
YY_RULE(int) yy_HtmlBlockForm(GREG *G); /* 75 */
YY_RULE(int) yy_HtmlBlockCloseForm(GREG *G); /* 74 */
YY_RULE(int) yy_HtmlBlockOpenForm(GREG *G); /* 73 */
YY_RULE(int) yy_HtmlBlockFieldset(GREG *G); /* 72 */
YY_RULE(int) yy_HtmlBlockCloseFieldset(GREG *G); /* 71 */
YY_RULE(int) yy_HtmlBlockOpenFieldset(GREG *G); /* 70 */
YY_RULE(int) yy_HtmlBlockDl(GREG *G); /* 69 */
YY_RULE(int) yy_HtmlBlockCloseDl(GREG *G); /* 68 */
YY_RULE(int) yy_HtmlBlockOpenDl(GREG *G); /* 67 */
YY_RULE(int) yy_HtmlBlockDiv(GREG *G); /* 66 */
YY_RULE(int) yy_HtmlBlockCloseDiv(GREG *G); /* 65 */
YY_RULE(int) yy_HtmlBlockOpenDiv(GREG *G); /* 64 */
YY_RULE(int) yy_HtmlBlockDir(GREG *G); /* 63 */
YY_RULE(int) yy_HtmlBlockCloseDir(GREG *G); /* 62 */
YY_RULE(int) yy_HtmlBlockOpenDir(GREG *G); /* 61 */
YY_RULE(int) yy_HtmlBlockCenter(GREG *G); /* 60 */
YY_RULE(int) yy_HtmlBlockCloseCenter(GREG *G); /* 59 */
YY_RULE(int) yy_HtmlBlockOpenCenter(GREG *G); /* 58 */
YY_RULE(int) yy_HtmlBlockBlockquote(GREG *G); /* 57 */
YY_RULE(int) yy_HtmlBlockCloseBlockquote(GREG *G); /* 56 */
YY_RULE(int) yy_HtmlBlockOpenBlockquote(GREG *G); /* 55 */
YY_RULE(int) yy_HtmlBlockAddress(GREG *G); /* 54 */
YY_RULE(int) yy_HtmlBlockCloseAddress(GREG *G); /* 53 */
YY_RULE(int) yy_HtmlAttribute(GREG *G); /* 52 */
YY_RULE(int) yy_Spnl(GREG *G); /* 51 */
YY_RULE(int) yy_HtmlBlockOpenAddress(GREG *G); /* 50 */
YY_RULE(int) yy_OptionallyIndentedLine(GREG *G); /* 49 */
YY_RULE(int) yy_Indent(GREG *G); /* 48 */
YY_RULE(int) yy_ListBlockLine(GREG *G); /* 47 */
YY_RULE(int) yy_ListContinuationBlock(GREG *G); /* 46 */
YY_RULE(int) yy_ListBlock(GREG *G); /* 45 */
YY_RULE(int) yy_ListItem(GREG *G); /* 44 */
YY_RULE(int) yy_Enumerator(GREG *G); /* 43 */
YY_RULE(int) yy_ListItemTight(GREG *G); /* 42 */
YY_RULE(int) yy_ListLoose(GREG *G); /* 41 */
YY_RULE(int) yy_ListTight(GREG *G); /* 40 */
YY_RULE(int) yy_Spacechar(GREG *G); /* 39 */
YY_RULE(int) yy_Bullet(GREG *G); /* 38 */
YY_RULE(int) yy_VerbatimChunk(GREG *G); /* 37 */
YY_RULE(int) yy_IndentedLine(GREG *G); /* 36 */
YY_RULE(int) yy_NonblankIndentedLine(GREG *G); /* 35 */
YY_RULE(int) yy_Line(GREG *G); /* 34 */
YY_RULE(int) yy_StartList(GREG *G); /* 33 */
YY_RULE(int) yy_BlockQuoteRaw(GREG *G); /* 32 */
YY_RULE(int) yy_Endline(GREG *G); /* 31 */
YY_RULE(int) yy_RawLine(GREG *G); /* 30 */
YY_RULE(int) yy_SetextBottom2(GREG *G); /* 29 */
YY_RULE(int) yy_SetextBottom1(GREG *G); /* 28 */
YY_RULE(int) yy_SetextHeading2(GREG *G); /* 27 */
YY_RULE(int) yy_SetextHeading1(GREG *G); /* 26 */
YY_RULE(int) yy_SetextHeading(GREG *G); /* 25 */
YY_RULE(int) yy_AtxHeading(GREG *G); /* 24 */
YY_RULE(int) yy_AtxStart(GREG *G); /* 23 */
YY_RULE(int) yy_Inline(GREG *G); /* 22 */
YY_RULE(int) yy_Sp(GREG *G); /* 21 */
YY_RULE(int) yy_Newline(GREG *G); /* 20 */
YY_RULE(int) yy_AtxInline(GREG *G); /* 19 */
YY_RULE(int) yy_Inlines(GREG *G); /* 18 */
YY_RULE(int) yy_NonindentSpace(GREG *G); /* 17 */
YY_RULE(int) yy_Plain(GREG *G); /* 16 */
YY_RULE(int) yy_Para(GREG *G); /* 15 */
YY_RULE(int) yy_StyleBlock(GREG *G); /* 14 */
YY_RULE(int) yy_HtmlBlock(GREG *G); /* 13 */
YY_RULE(int) yy_BulletList(GREG *G); /* 12 */
YY_RULE(int) yy_OrderedList(GREG *G); /* 11 */
YY_RULE(int) yy_Heading(GREG *G); /* 10 */
YY_RULE(int) yy_HorizontalRule(GREG *G); /* 9 */
YY_RULE(int) yy_Reference(GREG *G); /* 8 */
YY_RULE(int) yy_Note(GREG *G); /* 7 */
YY_RULE(int) yy_Verbatim(GREG *G); /* 6 */
YY_RULE(int) yy_BlockQuote(GREG *G); /* 5 */
YY_RULE(int) yy_BlankLine(GREG *G); /* 4 */
YY_RULE(int) yy_LocMarker(GREG *G); /* 3 */
YY_RULE(int) yy_Block(GREG *G); /* 2 */
YY_RULE(int) yy_Doc(GREG *G); /* 1 */

YY_ACTION(void) yy_1_RawLine(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
  yyprintf((stderr, "do yy_1_RawLine\n"));
   yy = elem(pmh_RAW); ;
}
YY_ACTION(void) yy_1_Line(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
  yyprintf((stderr, "do yy_1_Line\n"));
   yy = mk_element((parser_data *)G->data, pmh_RAW, yy->pos, yy->end); ;
}
YY_ACTION(void) yy_1_StartList(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
  yyprintf((stderr, "do yy_1_StartList\n"));
   yy = NULL; ;
}
YY_ACTION(void) yy_1_HtmlComment(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_1_HtmlComment\n"));
   ADD(elem_s(pmh_COMMENT)); ;
#undef s
}
YY_ACTION(void) yy_1_RawHtml(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_1_RawHtml\n"));
   ADD(elem_s(pmh_HTML)); ;
#undef s
}
YY_ACTION(void) yy_1_Code(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_1_Code\n"));
   ADD(elem_s(pmh_CODE)); ;
#undef s
}
YY_ACTION(void) yy_1_Ticks5(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
  yyprintf((stderr, "do yy_1_Ticks5\n"));
   yy = elem(pmh_NO_TYPE); ;
}
YY_ACTION(void) yy_1_Ticks4(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
  yyprintf((stderr, "do yy_1_Ticks4\n"));
   yy = elem(pmh_NO_TYPE); ;
}
YY_ACTION(void) yy_1_Ticks3(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
  yyprintf((stderr, "do yy_1_Ticks3\n"));
   yy = elem(pmh_NO_TYPE); ;
}
YY_ACTION(void) yy_1_Ticks2(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
  yyprintf((stderr, "do yy_1_Ticks2\n"));
   yy = elem(pmh_NO_TYPE); ;
}
YY_ACTION(void) yy_1_Ticks1(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
  yyprintf((stderr, "do yy_1_Ticks1\n"));
   yy = elem(pmh_NO_TYPE); ;
}
YY_ACTION(void) yy_1_RefSrc(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
  yyprintf((stderr, "do yy_1_RefSrc\n"));
   yy = mk_notype; yy->address = COPY_YYTEXT_ORIG(); ;
}
YY_ACTION(void) yy_2_Label(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_2_Label\n"));
  
            s->pos = s->pos;
            s->end = thunk->end;
            yy = s;
        ;
#undef s
}
YY_ACTION(void) yy_1_Label(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_1_Label\n"));
   s->label = COPY_YYTEXT_ORIG(); ;
#undef s
}
YY_ACTION(void) yy_1_Reference(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define r G->val[-1]
#define l G->val[-2]
#define s G->val[-3]
  yyprintf((stderr, "do yy_1_Reference\n"));
  
                pmh_realelement *el = elem_s(pmh_REFERENCE);
                el->label = strdup(l->label);
                el->address = strdup(r->address);
                ADD(el);
                FREE_LABEL(l);
                FREE_ADDRESS(r);
              ;
#undef r
#undef l
#undef s
}
YY_ACTION(void) yy_3_AutoLinkEmail(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_3_AutoLinkEmail\n"));
  
                s->end = thunk->end;
                ADD(s);
               ;
#undef s
}
YY_ACTION(void) yy_2_AutoLinkEmail(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_2_AutoLinkEmail\n"));
   s->address = COPY_YYTEXT_ORIG(); ;
#undef s
}
YY_ACTION(void) yy_1_AutoLinkEmail(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_1_AutoLinkEmail\n"));
   s->type = pmh_AUTO_LINK_EMAIL; ;
#undef s
}
YY_ACTION(void) yy_3_AutoLinkUrl(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_3_AutoLinkUrl\n"));
  
                s->end = thunk->end;
                ADD(s);
               ;
#undef s
}
YY_ACTION(void) yy_2_AutoLinkUrl(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_2_AutoLinkUrl\n"));
   s->address = COPY_YYTEXT_ORIG(); ;
#undef s
}
YY_ACTION(void) yy_1_AutoLinkUrl(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_1_AutoLinkUrl\n"));
   s->type = pmh_AUTO_LINK_URL; ;
#undef s
}
YY_ACTION(void) yy_3_Source(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
  yyprintf((stderr, "do yy_3_Source\n"));
   yy->address = COPY_YYTEXT_ORIG(); ;
}
YY_ACTION(void) yy_2_Source(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
  yyprintf((stderr, "do yy_2_Source\n"));
   yy->address = COPY_YYTEXT_ORIG(); ;
}
YY_ACTION(void) yy_1_Source(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
  yyprintf((stderr, "do yy_1_Source\n"));
   yy = mk_notype; ;
}
YY_ACTION(void) yy_1_ExplicitLink(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define l G->val[-1]
#define s G->val[-2]
  yyprintf((stderr, "do yy_1_ExplicitLink\n"));
  
                    yy = elem_s(pmh_LINK);
                    if (l->address != NULL)
                        yy->address = strdup(l->address);
                    FREE_LABEL(s);
                    FREE_ADDRESS(l);
                ;
#undef l
#undef s
}
YY_ACTION(void) yy_1_ReferenceLinkSingle(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_1_ReferenceLinkSingle\n"));
  
                        	pmh_realelement *reference = GET_REF(s->label);
                            if (reference) {
                                yy = elem_s(pmh_LINK);
                                yy->label = strdup(s->label);
                                yy->address = strdup(reference->address);
                            } else
                                yy = NULL;
                            FREE_LABEL(s);
                        ;
#undef s
}
YY_ACTION(void) yy_1_ReferenceLinkDouble(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define l G->val[-1]
#define s G->val[-2]
  yyprintf((stderr, "do yy_1_ReferenceLinkDouble\n"));
  
                        	pmh_realelement *reference = GET_REF(l->label);
                            if (reference) {
                                yy = elem_s(pmh_LINK);
                                yy->label = strdup(l->label);
                                yy->address = strdup(reference->address);
                            } else
                                yy = NULL;
                            FREE_LABEL(s);
                            FREE_LABEL(l);
                        ;
#undef l
#undef s
}
YY_ACTION(void) yy_1_Link(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
  yyprintf((stderr, "do yy_1_Link\n"));
   if (yy) ADD(yy); ;
}
YY_ACTION(void) yy_1_Image(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
  yyprintf((stderr, "do yy_1_Image\n"));
  
            if (yy != NULL) {
                yy->type = pmh_IMAGE;
                yy->pos -= 1;
                ADD(yy);
            }
        ;
}
YY_ACTION(void) yy_1_StrongUl(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_1_StrongUl\n"));
   ADD(elem_s(pmh_STRONG)); ;
#undef s
}
YY_ACTION(void) yy_1_TwoUlClose(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
  yyprintf((stderr, "do yy_1_TwoUlClose\n"));
   yy = elem(pmh_NO_TYPE); ;
}
YY_ACTION(void) yy_1_TwoUlOpen(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
  yyprintf((stderr, "do yy_1_TwoUlOpen\n"));
   yy = elem(pmh_NO_TYPE); ;
}
YY_ACTION(void) yy_1_StrongStar(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_1_StrongStar\n"));
   ADD(elem_s(pmh_STRONG)); ;
#undef s
}
YY_ACTION(void) yy_1_TwoStarClose(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
  yyprintf((stderr, "do yy_1_TwoStarClose\n"));
   yy = elem(pmh_NO_TYPE); ;
}
YY_ACTION(void) yy_1_TwoStarOpen(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
  yyprintf((stderr, "do yy_1_TwoStarOpen\n"));
   yy = elem(pmh_NO_TYPE); ;
}
YY_ACTION(void) yy_1_EmphUl(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_1_EmphUl\n"));
   ADD(elem_s(pmh_EMPH)); ;
#undef s
}
YY_ACTION(void) yy_1_OneUlClose(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
  yyprintf((stderr, "do yy_1_OneUlClose\n"));
   yy = elem(pmh_NO_TYPE); ;
}
YY_ACTION(void) yy_1_OneUlOpen(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
  yyprintf((stderr, "do yy_1_OneUlOpen\n"));
   yy = elem(pmh_NO_TYPE); ;
}
YY_ACTION(void) yy_1_EmphStar(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_1_EmphStar\n"));
   ADD(elem_s(pmh_EMPH)); ;
#undef s
}
YY_ACTION(void) yy_1_OneStarClose(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
  yyprintf((stderr, "do yy_1_OneStarClose\n"));
   yy = elem(pmh_NO_TYPE); ;
}
YY_ACTION(void) yy_1_OneStarOpen(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
  yyprintf((stderr, "do yy_1_OneStarOpen\n"));
   yy = elem(pmh_NO_TYPE); ;
}
YY_ACTION(void) yy_1_Entity(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_1_Entity\n"));
   ADD(elem_s(pmh_HTML_ENTITY)); ;
#undef s
}
YY_ACTION(void) yy_1_StyleBlock(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_1_StyleBlock\n"));
   ADD(elem_s(pmh_HTMLBLOCK)); ;
#undef s
}
YY_ACTION(void) yy_1_HtmlBlock(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_1_HtmlBlock\n"));
   ADD(elem_s(pmh_HTMLBLOCK)); ;
#undef s
}
YY_ACTION(void) yy_1_HtmlBlockH6(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_1_HtmlBlockH6\n"));
   ADD(elem_s(pmh_H6)); ;
#undef s
}
YY_ACTION(void) yy_1_HtmlBlockH5(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_1_HtmlBlockH5\n"));
   ADD(elem_s(pmh_H5)); ;
#undef s
}
YY_ACTION(void) yy_1_HtmlBlockH4(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_1_HtmlBlockH4\n"));
   ADD(elem_s(pmh_H4)); ;
#undef s
}
YY_ACTION(void) yy_1_HtmlBlockH3(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_1_HtmlBlockH3\n"));
   ADD(elem_s(pmh_H3)); ;
#undef s
}
YY_ACTION(void) yy_1_HtmlBlockH2(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_1_HtmlBlockH2\n"));
   ADD(elem_s(pmh_H2)); ;
#undef s
}
YY_ACTION(void) yy_1_HtmlBlockH1(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_1_HtmlBlockH1\n"));
   ADD(elem_s(pmh_H1)); ;
#undef s
}
YY_ACTION(void) yy_1_Enumerator(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
  yyprintf((stderr, "do yy_1_Enumerator\n"));
   ADD(elem(pmh_LIST_ENUMERATOR)); ;
}
YY_ACTION(void) yy_3_ListContinuationBlock(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define a G->val[-1]
  yyprintf((stderr, "do yy_3_ListContinuationBlock\n"));
   yy = a; ;
#undef a
}
YY_ACTION(void) yy_2_ListContinuationBlock(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define a G->val[-1]
  yyprintf((stderr, "do yy_2_ListContinuationBlock\n"));
   a = cons(yy, a); ;
#undef a
}
YY_ACTION(void) yy_1_ListContinuationBlock(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define a G->val[-1]
  yyprintf((stderr, "do yy_1_ListContinuationBlock\n"));
   if (*yytext == '\0') /* if strlen(yytext) == 0 */
                                a = cons(elem(pmh_SEPARATOR), a);
                            else
                                a = cons(elem(pmh_RAW), a);
                          ;
#undef a
}
YY_ACTION(void) yy_3_ListBlock(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define a G->val[-1]
  yyprintf((stderr, "do yy_3_ListBlock\n"));
   yy = a; ;
#undef a
}
YY_ACTION(void) yy_2_ListBlock(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define a G->val[-1]
  yyprintf((stderr, "do yy_2_ListBlock\n"));
   a = cons(elem(pmh_RAW), a); ;
#undef a
}
YY_ACTION(void) yy_1_ListBlock(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define a G->val[-1]
  yyprintf((stderr, "do yy_1_ListBlock\n"));
   a = cons(yy, a); ;
#undef a
}
YY_ACTION(void) yy_3_ListItemTight(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define a G->val[-1]
  yyprintf((stderr, "do yy_3_ListItemTight\n"));
   yy = a; ;
#undef a
}
YY_ACTION(void) yy_2_ListItemTight(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define a G->val[-1]
  yyprintf((stderr, "do yy_2_ListItemTight\n"));
   a = cons(yy, a); ;
#undef a
}
YY_ACTION(void) yy_1_ListItemTight(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define a G->val[-1]
  yyprintf((stderr, "do yy_1_ListItemTight\n"));
   a = cons(yy, a); ;
#undef a
}
YY_ACTION(void) yy_3_ListItem(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define a G->val[-1]
  yyprintf((stderr, "do yy_3_ListItem\n"));
   yy = a; ;
#undef a
}
YY_ACTION(void) yy_2_ListItem(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define a G->val[-1]
  yyprintf((stderr, "do yy_2_ListItem\n"));
   a = cons(yy, a); ;
#undef a
}
YY_ACTION(void) yy_1_ListItem(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define a G->val[-1]
  yyprintf((stderr, "do yy_1_ListItem\n"));
   a = cons(yy, a); ;
#undef a
}
YY_ACTION(void) yy_2_ListLoose(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define b G->val[-1]
#define a G->val[-2]
  yyprintf((stderr, "do yy_2_ListLoose\n"));
   pmh_realelement *cur = a;
              while (cur != NULL) {
                  pmh_realelement *rawlist = mk_element((parser_data *)G->data, pmh_RAW_LIST, 0,0);
                  rawlist->children = reverse(cur->children);
                  ADD(rawlist);
                  cur = cur->next;
              }
            ;
#undef b
#undef a
}
YY_ACTION(void) yy_1_ListLoose(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define b G->val[-1]
#define a G->val[-2]
  yyprintf((stderr, "do yy_1_ListLoose\n"));
   b = cons(etext("\n\n"), b); /* In loose list, \n\n added to end of each element */
                pmh_realelement *el = mk_notype;
                el->children = b;
                a = cons(el, a);
              ;
#undef b
#undef a
}
YY_ACTION(void) yy_2_ListTight(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define a G->val[-1]
  yyprintf((stderr, "do yy_2_ListTight\n"));
   pmh_realelement *cur = a;
              while (cur != NULL) {
                  pmh_realelement *rawlist = mk_element((parser_data *)G->data, pmh_RAW_LIST, 0,0);
                  rawlist->children = reverse(cur->children);
                  ADD(rawlist);
                  cur = cur->next;
              }
            ;
#undef a
}
YY_ACTION(void) yy_1_ListTight(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define a G->val[-1]
  yyprintf((stderr, "do yy_1_ListTight\n"));
   pmh_realelement *el = mk_notype;
                el->children = yy;
                a = cons(el, a);
              ;
#undef a
}
YY_ACTION(void) yy_1_Bullet(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
  yyprintf((stderr, "do yy_1_Bullet\n"));
   ADD(elem(pmh_LIST_BULLET)); ;
}
YY_ACTION(void) yy_1_HorizontalRule(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
  yyprintf((stderr, "do yy_1_HorizontalRule\n"));
   ADD(elem(pmh_HRULE)); ;
}
YY_ACTION(void) yy_1_Verbatim(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_1_Verbatim\n"));
   ADD(elem_s(pmh_VERBATIM)); ;
#undef s
}
YY_ACTION(void) yy_5_BlockQuoteRaw(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define a G->val[-1]
  yyprintf((stderr, "do yy_5_BlockQuoteRaw\n"));
   yy = a; ;
#undef a
}
YY_ACTION(void) yy_4_BlockQuoteRaw(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define a G->val[-1]
  yyprintf((stderr, "do yy_4_BlockQuoteRaw\n"));
   a = cons(etext("\n"), a); ;
#undef a
}
YY_ACTION(void) yy_3_BlockQuoteRaw(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define a G->val[-1]
  yyprintf((stderr, "do yy_3_BlockQuoteRaw\n"));
   a = cons(yy, a); ;
#undef a
}
YY_ACTION(void) yy_2_BlockQuoteRaw(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define a G->val[-1]
  yyprintf((stderr, "do yy_2_BlockQuoteRaw\n"));
   a = cons(yy, a); ;
#undef a
}
YY_ACTION(void) yy_1_BlockQuoteRaw(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define a G->val[-1]
  yyprintf((stderr, "do yy_1_BlockQuoteRaw\n"));
   ADD(elem(pmh_BLOCKQUOTE)); ;
#undef a
}
YY_ACTION(void) yy_1_BlockQuote(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define a G->val[-1]
  yyprintf((stderr, "do yy_1_BlockQuote\n"));
   pmh_realelement *rawlist = mk_element((parser_data *)G->data, pmh_RAW_LIST, 0,0);
              rawlist->children = reverse(a);
              ADD(rawlist);
            ;
#undef a
}
YY_ACTION(void) yy_1_SetextHeading2(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_1_SetextHeading2\n"));
   ADD(elem_s(pmh_H2)); ;
#undef s
}
YY_ACTION(void) yy_1_SetextHeading1(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_1_SetextHeading1\n"));
   ADD(elem_s(pmh_H1)); ;
#undef s
}
YY_ACTION(void) yy_1_AtxHeading(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_1_AtxHeading\n"));
   ADD(elem_s(s->type)); ;
#undef s
}
YY_ACTION(void) yy_1_AtxStart(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
  yyprintf((stderr, "do yy_1_AtxStart\n"));
   yy = elem((pmh_element_type)(pmh_H1 + (strlen(yytext) - 1))); ;
}
YY_ACTION(void) yy_1_LocMarker(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
  yyprintf((stderr, "do yy_1_LocMarker\n"));
   yy = elem(pmh_NO_TYPE); ;
}

YY_RULE(int) yy_RawNoteBlock(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "RawNoteBlock"));
  {  int yypos4= G->pos, yythunkpos4= G->thunkpos;  if (!yy_BlankLine(G)) { goto l4; }  goto l1;
  l4:;	  G->pos= yypos4; G->thunkpos= yythunkpos4;
  }  if (!yy_OptionallyIndentedLine(G)) { goto l1; }
  l2:;	
  {  int yypos3= G->pos, yythunkpos3= G->thunkpos;
  {  int yypos5= G->pos, yythunkpos5= G->thunkpos;  if (!yy_BlankLine(G)) { goto l5; }  goto l3;
  l5:;	  G->pos= yypos5; G->thunkpos= yythunkpos5;
  }  if (!yy_OptionallyIndentedLine(G)) { goto l3; }  goto l2;
  l3:;	  G->pos= yypos3; G->thunkpos= yythunkpos3;
  }
  l6:;	
  {  int yypos7= G->pos, yythunkpos7= G->thunkpos;  if (!yy_BlankLine(G)) { goto l7; }  goto l6;
  l7:;	  G->pos= yypos7; G->thunkpos= yythunkpos7;
  }
  yyprintf((stderr, "  ok   %s @ %s\n", "RawNoteBlock", G->buf+G->pos));
  return 1;
  l1:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "RawNoteBlock", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_RawNoteReference(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "RawNoteReference"));  if (!yymatchString(G, "[^")) goto l8;
  {  int yypos11= G->pos, yythunkpos11= G->thunkpos;  if (!yy_Newline(G)) { goto l11; }  goto l8;
  l11:;	  G->pos= yypos11; G->thunkpos= yythunkpos11;
  }
  {  int yypos12= G->pos, yythunkpos12= G->thunkpos;  if (!yymatchChar(G, ']')) goto l12;  goto l8;
  l12:;	  G->pos= yypos12; G->thunkpos= yythunkpos12;
  }  if (!yymatchDot(G)) goto l8;
  l9:;	
  {  int yypos10= G->pos, yythunkpos10= G->thunkpos;
  {  int yypos13= G->pos, yythunkpos13= G->thunkpos;  if (!yy_Newline(G)) { goto l13; }  goto l10;
  l13:;	  G->pos= yypos13; G->thunkpos= yythunkpos13;
  }
  {  int yypos14= G->pos, yythunkpos14= G->thunkpos;  if (!yymatchChar(G, ']')) goto l14;  goto l10;
  l14:;	  G->pos= yypos14; G->thunkpos= yythunkpos14;
  }  if (!yymatchDot(G)) goto l10;  goto l9;
  l10:;	  G->pos= yypos10; G->thunkpos= yythunkpos10;
  }  if (!yymatchChar(G, ']')) goto l8;
  yyprintf((stderr, "  ok   %s @ %s\n", "RawNoteReference", G->buf+G->pos));
  return 1;
  l8:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "RawNoteReference", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_ExtendedSpecialChar(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "ExtendedSpecialChar"));  yyText(G, G->begin, G->end);  if (!( EXT(pmh_EXT_NOTES) )) goto l15;  if (!yymatchChar(G, '^')) goto l15;
  yyprintf((stderr, "  ok   %s @ %s\n", "ExtendedSpecialChar", G->buf+G->pos));
  return 1;
  l15:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "ExtendedSpecialChar", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_AlphanumericAscii(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "AlphanumericAscii"));  if (!yymatchClass(G, (unsigned char *)"\000\000\000\000\000\000\377\003\376\377\377\007\376\377\377\007\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l16;
  yyprintf((stderr, "  ok   %s @ %s\n", "AlphanumericAscii", G->buf+G->pos));
  return 1;
  l16:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "AlphanumericAscii", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Quoted(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Quoted"));
  {  int yypos18= G->pos, yythunkpos18= G->thunkpos;  if (!yymatchChar(G, '"')) goto l19;
  l20:;	
  {  int yypos21= G->pos, yythunkpos21= G->thunkpos;
  {  int yypos22= G->pos, yythunkpos22= G->thunkpos;  if (!yymatchChar(G, '"')) goto l22;  goto l21;
  l22:;	  G->pos= yypos22; G->thunkpos= yythunkpos22;
  }  if (!yymatchDot(G)) goto l21;  goto l20;
  l21:;	  G->pos= yypos21; G->thunkpos= yythunkpos21;
  }  if (!yymatchChar(G, '"')) goto l19;  goto l18;
  l19:;	  G->pos= yypos18; G->thunkpos= yythunkpos18;  if (!yymatchChar(G, '\'')) goto l17;
  l23:;	
  {  int yypos24= G->pos, yythunkpos24= G->thunkpos;
  {  int yypos25= G->pos, yythunkpos25= G->thunkpos;  if (!yymatchChar(G, '\'')) goto l25;  goto l24;
  l25:;	  G->pos= yypos25; G->thunkpos= yythunkpos25;
  }  if (!yymatchDot(G)) goto l24;  goto l23;
  l24:;	  G->pos= yypos24; G->thunkpos= yythunkpos24;
  }  if (!yymatchChar(G, '\'')) goto l17;
  }
  l18:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "Quoted", G->buf+G->pos));
  return 1;
  l17:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Quoted", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlTag(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlTag"));  if (!yymatchChar(G, '<')) goto l26;  if (!yy_Spnl(G)) { goto l26; }
  {  int yypos27= G->pos, yythunkpos27= G->thunkpos;  if (!yymatchChar(G, '/')) goto l27;  goto l28;
  l27:;	  G->pos= yypos27; G->thunkpos= yythunkpos27;
  }
  l28:;	  if (!yy_AlphanumericAscii(G)) { goto l26; }
  l29:;	
  {  int yypos30= G->pos, yythunkpos30= G->thunkpos;  if (!yy_AlphanumericAscii(G)) { goto l30; }  goto l29;
  l30:;	  G->pos= yypos30; G->thunkpos= yythunkpos30;
  }  if (!yy_Spnl(G)) { goto l26; }
  l31:;	
  {  int yypos32= G->pos, yythunkpos32= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l32; }  goto l31;
  l32:;	  G->pos= yypos32; G->thunkpos= yythunkpos32;
  }
  {  int yypos33= G->pos, yythunkpos33= G->thunkpos;  if (!yymatchChar(G, '/')) goto l33;  goto l34;
  l33:;	  G->pos= yypos33; G->thunkpos= yythunkpos33;
  }
  l34:;	  if (!yy_Spnl(G)) { goto l26; }  if (!yymatchChar(G, '>')) goto l26;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlTag", G->buf+G->pos));
  return 1;
  l26:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlTag", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Ticks5(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Ticks5"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l35;  if (!yymatchString(G, "`````")) goto l35;  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l35;
  {  int yypos36= G->pos, yythunkpos36= G->thunkpos;  if (!yymatchChar(G, '`')) goto l36;  goto l35;
  l36:;	  G->pos= yypos36; G->thunkpos= yythunkpos36;
  }  yyDo(G, yy_1_Ticks5, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "Ticks5", G->buf+G->pos));
  return 1;
  l35:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Ticks5", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Ticks4(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Ticks4"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l37;  if (!yymatchString(G, "````")) goto l37;  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l37;
  {  int yypos38= G->pos, yythunkpos38= G->thunkpos;  if (!yymatchChar(G, '`')) goto l38;  goto l37;
  l38:;	  G->pos= yypos38; G->thunkpos= yythunkpos38;
  }  yyDo(G, yy_1_Ticks4, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "Ticks4", G->buf+G->pos));
  return 1;
  l37:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Ticks4", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Ticks3(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Ticks3"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l39;  if (!yymatchString(G, "```")) goto l39;  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l39;
  {  int yypos40= G->pos, yythunkpos40= G->thunkpos;  if (!yymatchChar(G, '`')) goto l40;  goto l39;
  l40:;	  G->pos= yypos40; G->thunkpos= yythunkpos40;
  }  yyDo(G, yy_1_Ticks3, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "Ticks3", G->buf+G->pos));
  return 1;
  l39:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Ticks3", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Ticks2(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Ticks2"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l41;  if (!yymatchString(G, "``")) goto l41;  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l41;
  {  int yypos42= G->pos, yythunkpos42= G->thunkpos;  if (!yymatchChar(G, '`')) goto l42;  goto l41;
  l42:;	  G->pos= yypos42; G->thunkpos= yythunkpos42;
  }  yyDo(G, yy_1_Ticks2, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "Ticks2", G->buf+G->pos));
  return 1;
  l41:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Ticks2", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Ticks1(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Ticks1"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l43;  if (!yymatchChar(G, '`')) goto l43;  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l43;
  {  int yypos44= G->pos, yythunkpos44= G->thunkpos;  if (!yymatchChar(G, '`')) goto l44;  goto l43;
  l44:;	  G->pos= yypos44; G->thunkpos= yythunkpos44;
  }  yyDo(G, yy_1_Ticks1, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "Ticks1", G->buf+G->pos));
  return 1;
  l43:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Ticks1", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_SkipBlock(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "SkipBlock"));
  {  int yypos46= G->pos, yythunkpos46= G->thunkpos;
  {  int yypos50= G->pos, yythunkpos50= G->thunkpos;  if (!yy_BlankLine(G)) { goto l50; }  goto l47;
  l50:;	  G->pos= yypos50; G->thunkpos= yythunkpos50;
  }  if (!yy_RawLine(G)) { goto l47; }
  l48:;	
  {  int yypos49= G->pos, yythunkpos49= G->thunkpos;
  {  int yypos51= G->pos, yythunkpos51= G->thunkpos;  if (!yy_BlankLine(G)) { goto l51; }  goto l49;
  l51:;	  G->pos= yypos51; G->thunkpos= yythunkpos51;
  }  if (!yy_RawLine(G)) { goto l49; }  goto l48;
  l49:;	  G->pos= yypos49; G->thunkpos= yythunkpos49;
  }
  l52:;	
  {  int yypos53= G->pos, yythunkpos53= G->thunkpos;  if (!yy_BlankLine(G)) { goto l53; }  goto l52;
  l53:;	  G->pos= yypos53; G->thunkpos= yythunkpos53;
  }  goto l46;
  l47:;	  G->pos= yypos46; G->thunkpos= yythunkpos46;  if (!yy_BlankLine(G)) { goto l45; }
  l54:;	
  {  int yypos55= G->pos, yythunkpos55= G->thunkpos;  if (!yy_BlankLine(G)) { goto l55; }  goto l54;
  l55:;	  G->pos= yypos55; G->thunkpos= yythunkpos55;
  }
  }
  l46:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "SkipBlock", G->buf+G->pos));
  return 1;
  l45:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "SkipBlock", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_References(GREG *G)
{
  yyprintf((stderr, "%s\n", "References"));
  l57:;	
  {  int yypos58= G->pos, yythunkpos58= G->thunkpos;
  {  int yypos59= G->pos, yythunkpos59= G->thunkpos;  if (!yy_Reference(G)) { goto l60; }  goto l59;
  l60:;	  G->pos= yypos59; G->thunkpos= yythunkpos59;  if (!yy_SkipBlock(G)) { goto l58; }
  }
  l59:;	  goto l57;
  l58:;	  G->pos= yypos58; G->thunkpos= yythunkpos58;
  }
  yyprintf((stderr, "  ok   %s @ %s\n", "References", G->buf+G->pos));
  return 1;
}
YY_RULE(int) yy_EmptyTitle(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "EmptyTitle"));  if (!yymatchString(G, "")) goto l61;
  yyprintf((stderr, "  ok   %s @ %s\n", "EmptyTitle", G->buf+G->pos));
  return 1;
  l61:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "EmptyTitle", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_RefTitleParens(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "RefTitleParens"));  if (!yy_Spnl(G)) { goto l62; }  if (!yymatchChar(G, '(')) goto l62;
  l63:;	
  {  int yypos64= G->pos, yythunkpos64= G->thunkpos;
  {  int yypos65= G->pos, yythunkpos65= G->thunkpos;
  {  int yypos66= G->pos, yythunkpos66= G->thunkpos;  if (!yymatchChar(G, ')')) goto l67;  if (!yy_Sp(G)) { goto l67; }  if (!yy_Newline(G)) { goto l67; }  goto l66;
  l67:;	  G->pos= yypos66; G->thunkpos= yythunkpos66;  if (!yy_Newline(G)) { goto l65; }
  }
  l66:;	  goto l64;
  l65:;	  G->pos= yypos65; G->thunkpos= yythunkpos65;
  }  if (!yymatchDot(G)) goto l64;  goto l63;
  l64:;	  G->pos= yypos64; G->thunkpos= yythunkpos64;
  }  if (!yymatchChar(G, ')')) goto l62;
  yyprintf((stderr, "  ok   %s @ %s\n", "RefTitleParens", G->buf+G->pos));
  return 1;
  l62:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "RefTitleParens", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_RefTitleDouble(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "RefTitleDouble"));  if (!yy_Spnl(G)) { goto l68; }  if (!yymatchChar(G, '"')) goto l68;
  l69:;	
  {  int yypos70= G->pos, yythunkpos70= G->thunkpos;
  {  int yypos71= G->pos, yythunkpos71= G->thunkpos;
  {  int yypos72= G->pos, yythunkpos72= G->thunkpos;  if (!yymatchChar(G, '"')) goto l73;  if (!yy_Sp(G)) { goto l73; }  if (!yy_Newline(G)) { goto l73; }  goto l72;
  l73:;	  G->pos= yypos72; G->thunkpos= yythunkpos72;  if (!yy_Newline(G)) { goto l71; }
  }
  l72:;	  goto l70;
  l71:;	  G->pos= yypos71; G->thunkpos= yythunkpos71;
  }  if (!yymatchDot(G)) goto l70;  goto l69;
  l70:;	  G->pos= yypos70; G->thunkpos= yythunkpos70;
  }  if (!yymatchChar(G, '"')) goto l68;
  yyprintf((stderr, "  ok   %s @ %s\n", "RefTitleDouble", G->buf+G->pos));
  return 1;
  l68:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "RefTitleDouble", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_RefTitleSingle(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "RefTitleSingle"));  if (!yy_Spnl(G)) { goto l74; }  if (!yymatchChar(G, '\'')) goto l74;
  l75:;	
  {  int yypos76= G->pos, yythunkpos76= G->thunkpos;
  {  int yypos77= G->pos, yythunkpos77= G->thunkpos;
  {  int yypos78= G->pos, yythunkpos78= G->thunkpos;  if (!yymatchChar(G, '\'')) goto l79;  if (!yy_Sp(G)) { goto l79; }  if (!yy_Newline(G)) { goto l79; }  goto l78;
  l79:;	  G->pos= yypos78; G->thunkpos= yythunkpos78;  if (!yy_Newline(G)) { goto l77; }
  }
  l78:;	  goto l76;
  l77:;	  G->pos= yypos77; G->thunkpos= yythunkpos77;
  }  if (!yymatchDot(G)) goto l76;  goto l75;
  l76:;	  G->pos= yypos76; G->thunkpos= yythunkpos76;
  }  if (!yymatchChar(G, '\'')) goto l74;
  yyprintf((stderr, "  ok   %s @ %s\n", "RefTitleSingle", G->buf+G->pos));
  return 1;
  l74:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "RefTitleSingle", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_RefTitle(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "RefTitle"));
  {  int yypos81= G->pos, yythunkpos81= G->thunkpos;  if (!yy_RefTitleSingle(G)) { goto l82; }  goto l81;
  l82:;	  G->pos= yypos81; G->thunkpos= yythunkpos81;  if (!yy_RefTitleDouble(G)) { goto l83; }  goto l81;
  l83:;	  G->pos= yypos81; G->thunkpos= yythunkpos81;  if (!yy_RefTitleParens(G)) { goto l84; }  goto l81;
  l84:;	  G->pos= yypos81; G->thunkpos= yythunkpos81;  if (!yy_EmptyTitle(G)) { goto l80; }
  }
  l81:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "RefTitle", G->buf+G->pos));
  return 1;
  l80:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "RefTitle", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_RefSrc(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "RefSrc"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l85;  if (!yy_Nonspacechar(G)) { goto l85; }
  l86:;	
  {  int yypos87= G->pos, yythunkpos87= G->thunkpos;  if (!yy_Nonspacechar(G)) { goto l87; }  goto l86;
  l87:;	  G->pos= yypos87; G->thunkpos= yythunkpos87;
  }  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l85;  yyDo(G, yy_1_RefSrc, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "RefSrc", G->buf+G->pos));
  return 1;
  l85:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "RefSrc", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_AutoLinkEmail(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "AutoLinkEmail"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l88;  if (!yy_LocMarker(G)) { goto l88; }  yyDo(G, yySet, -1, 0);  yyDo(G, yy_1_AutoLinkEmail, G->begin, G->end);  if (!yymatchChar(G, '<')) goto l88;
  {  int yypos89= G->pos, yythunkpos89= G->thunkpos;  if (!yymatchString(G, "mailto:")) goto l89;  goto l90;
  l89:;	  G->pos= yypos89; G->thunkpos= yythunkpos89;
  }
  l90:;	  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l88;  if (!yymatchClass(G, (unsigned char *)"\000\000\000\000\062\350\377\003\376\377\377\207\376\377\377\107\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l88;
  l91:;	
  {  int yypos92= G->pos, yythunkpos92= G->thunkpos;  if (!yymatchClass(G, (unsigned char *)"\000\000\000\000\062\350\377\003\376\377\377\207\376\377\377\107\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l92;  goto l91;
  l92:;	  G->pos= yypos92; G->thunkpos= yythunkpos92;
  }  if (!yymatchChar(G, '@')) goto l88;
  {  int yypos95= G->pos, yythunkpos95= G->thunkpos;  if (!yy_Newline(G)) { goto l95; }  goto l88;
  l95:;	  G->pos= yypos95; G->thunkpos= yythunkpos95;
  }
  {  int yypos96= G->pos, yythunkpos96= G->thunkpos;  if (!yymatchChar(G, '>')) goto l96;  goto l88;
  l96:;	  G->pos= yypos96; G->thunkpos= yythunkpos96;
  }  if (!yymatchDot(G)) goto l88;
  l93:;	
  {  int yypos94= G->pos, yythunkpos94= G->thunkpos;
  {  int yypos97= G->pos, yythunkpos97= G->thunkpos;  if (!yy_Newline(G)) { goto l97; }  goto l94;
  l97:;	  G->pos= yypos97; G->thunkpos= yythunkpos97;
  }
  {  int yypos98= G->pos, yythunkpos98= G->thunkpos;  if (!yymatchChar(G, '>')) goto l98;  goto l94;
  l98:;	  G->pos= yypos98; G->thunkpos= yythunkpos98;
  }  if (!yymatchDot(G)) goto l94;  goto l93;
  l94:;	  G->pos= yypos94; G->thunkpos= yythunkpos94;
  }  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l88;  yyDo(G, yy_2_AutoLinkEmail, G->begin, G->end);  if (!yymatchChar(G, '>')) goto l88;  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l88;  yyDo(G, yy_3_AutoLinkEmail, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "AutoLinkEmail", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l88:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "AutoLinkEmail", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_AutoLinkUrl(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "AutoLinkUrl"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l99;  if (!yy_LocMarker(G)) { goto l99; }  yyDo(G, yySet, -1, 0);  yyDo(G, yy_1_AutoLinkUrl, G->begin, G->end);  if (!yymatchChar(G, '<')) goto l99;  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l99;  if (!yymatchClass(G, (unsigned char *)"\000\000\000\000\000\000\000\000\376\377\377\007\376\377\377\007\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l99;
  l100:;	
  {  int yypos101= G->pos, yythunkpos101= G->thunkpos;  if (!yymatchClass(G, (unsigned char *)"\000\000\000\000\000\000\000\000\376\377\377\007\376\377\377\007\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l101;  goto l100;
  l101:;	  G->pos= yypos101; G->thunkpos= yythunkpos101;
  }  if (!yymatchString(G, "://")) goto l99;
  {  int yypos104= G->pos, yythunkpos104= G->thunkpos;  if (!yy_Newline(G)) { goto l104; }  goto l99;
  l104:;	  G->pos= yypos104; G->thunkpos= yythunkpos104;
  }
  {  int yypos105= G->pos, yythunkpos105= G->thunkpos;  if (!yymatchChar(G, '>')) goto l105;  goto l99;
  l105:;	  G->pos= yypos105; G->thunkpos= yythunkpos105;
  }  if (!yymatchDot(G)) goto l99;
  l102:;	
  {  int yypos103= G->pos, yythunkpos103= G->thunkpos;
  {  int yypos106= G->pos, yythunkpos106= G->thunkpos;  if (!yy_Newline(G)) { goto l106; }  goto l103;
  l106:;	  G->pos= yypos106; G->thunkpos= yythunkpos106;
  }
  {  int yypos107= G->pos, yythunkpos107= G->thunkpos;  if (!yymatchChar(G, '>')) goto l107;  goto l103;
  l107:;	  G->pos= yypos107; G->thunkpos= yythunkpos107;
  }  if (!yymatchDot(G)) goto l103;  goto l102;
  l103:;	  G->pos= yypos103; G->thunkpos= yythunkpos103;
  }  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l99;  yyDo(G, yy_2_AutoLinkUrl, G->begin, G->end);  if (!yymatchChar(G, '>')) goto l99;  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l99;  yyDo(G, yy_3_AutoLinkUrl, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "AutoLinkUrl", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l99:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "AutoLinkUrl", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_TitleDouble(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "TitleDouble"));  if (!yymatchChar(G, '"')) goto l108;
  l109:;	
  {  int yypos110= G->pos, yythunkpos110= G->thunkpos;
  {  int yypos111= G->pos, yythunkpos111= G->thunkpos;  if (!yymatchChar(G, '"')) goto l111;  if (!yy_Sp(G)) { goto l111; }
  {  int yypos112= G->pos, yythunkpos112= G->thunkpos;  if (!yymatchChar(G, ')')) goto l113;  goto l112;
  l113:;	  G->pos= yypos112; G->thunkpos= yythunkpos112;  if (!yy_Newline(G)) { goto l111; }
  }
  l112:;	  goto l110;
  l111:;	  G->pos= yypos111; G->thunkpos= yythunkpos111;
  }  if (!yymatchDot(G)) goto l110;  goto l109;
  l110:;	  G->pos= yypos110; G->thunkpos= yythunkpos110;
  }  if (!yymatchChar(G, '"')) goto l108;
  yyprintf((stderr, "  ok   %s @ %s\n", "TitleDouble", G->buf+G->pos));
  return 1;
  l108:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "TitleDouble", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_TitleSingle(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "TitleSingle"));  if (!yymatchChar(G, '\'')) goto l114;
  l115:;	
  {  int yypos116= G->pos, yythunkpos116= G->thunkpos;
  {  int yypos117= G->pos, yythunkpos117= G->thunkpos;  if (!yymatchChar(G, '\'')) goto l117;  if (!yy_Sp(G)) { goto l117; }
  {  int yypos118= G->pos, yythunkpos118= G->thunkpos;  if (!yymatchChar(G, ')')) goto l119;  goto l118;
  l119:;	  G->pos= yypos118; G->thunkpos= yythunkpos118;  if (!yy_Newline(G)) { goto l117; }
  }
  l118:;	  goto l116;
  l117:;	  G->pos= yypos117; G->thunkpos= yythunkpos117;
  }  if (!yymatchDot(G)) goto l116;  goto l115;
  l116:;	  G->pos= yypos116; G->thunkpos= yythunkpos116;
  }  if (!yymatchChar(G, '\'')) goto l114;
  yyprintf((stderr, "  ok   %s @ %s\n", "TitleSingle", G->buf+G->pos));
  return 1;
  l114:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "TitleSingle", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Nonspacechar(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Nonspacechar"));
  {  int yypos121= G->pos, yythunkpos121= G->thunkpos;  if (!yy_Spacechar(G)) { goto l121; }  goto l120;
  l121:;	  G->pos= yypos121; G->thunkpos= yythunkpos121;
  }
  {  int yypos122= G->pos, yythunkpos122= G->thunkpos;  if (!yy_Newline(G)) { goto l122; }  goto l120;
  l122:;	  G->pos= yypos122; G->thunkpos= yythunkpos122;
  }  if (!yymatchDot(G)) goto l120;
  yyprintf((stderr, "  ok   %s @ %s\n", "Nonspacechar", G->buf+G->pos));
  return 1;
  l120:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Nonspacechar", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_SourceContents(GREG *G)
{
  yyprintf((stderr, "%s\n", "SourceContents"));
  l124:;	
  {  int yypos125= G->pos, yythunkpos125= G->thunkpos;
  {  int yypos126= G->pos, yythunkpos126= G->thunkpos;
  {  int yypos130= G->pos, yythunkpos130= G->thunkpos;  if (!yymatchChar(G, '(')) goto l130;  goto l127;
  l130:;	  G->pos= yypos130; G->thunkpos= yythunkpos130;
  }
  {  int yypos131= G->pos, yythunkpos131= G->thunkpos;  if (!yymatchChar(G, ')')) goto l131;  goto l127;
  l131:;	  G->pos= yypos131; G->thunkpos= yythunkpos131;
  }
  {  int yypos132= G->pos, yythunkpos132= G->thunkpos;  if (!yymatchChar(G, '>')) goto l132;  goto l127;
  l132:;	  G->pos= yypos132; G->thunkpos= yythunkpos132;
  }  if (!yy_Nonspacechar(G)) { goto l127; }
  l128:;	
  {  int yypos129= G->pos, yythunkpos129= G->thunkpos;
  {  int yypos133= G->pos, yythunkpos133= G->thunkpos;  if (!yymatchChar(G, '(')) goto l133;  goto l129;
  l133:;	  G->pos= yypos133; G->thunkpos= yythunkpos133;
  }
  {  int yypos134= G->pos, yythunkpos134= G->thunkpos;  if (!yymatchChar(G, ')')) goto l134;  goto l129;
  l134:;	  G->pos= yypos134; G->thunkpos= yythunkpos134;
  }
  {  int yypos135= G->pos, yythunkpos135= G->thunkpos;  if (!yymatchChar(G, '>')) goto l135;  goto l129;
  l135:;	  G->pos= yypos135; G->thunkpos= yythunkpos135;
  }  if (!yy_Nonspacechar(G)) { goto l129; }  goto l128;
  l129:;	  G->pos= yypos129; G->thunkpos= yythunkpos129;
  }  goto l126;
  l127:;	  G->pos= yypos126; G->thunkpos= yythunkpos126;  if (!yymatchChar(G, '(')) goto l125;  if (!yy_SourceContents(G)) { goto l125; }  if (!yymatchChar(G, ')')) goto l125;
  }
  l126:;	  goto l124;
  l125:;	  G->pos= yypos125; G->thunkpos= yythunkpos125;
  }
  yyprintf((stderr, "  ok   %s @ %s\n", "SourceContents", G->buf+G->pos));
  return 1;
}
YY_RULE(int) yy_Title(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Title"));
  {  int yypos137= G->pos, yythunkpos137= G->thunkpos;  if (!yy_TitleSingle(G)) { goto l138; }  goto l137;
  l138:;	  G->pos= yypos137; G->thunkpos= yythunkpos137;  if (!yy_TitleDouble(G)) { goto l139; }  goto l137;
  l139:;	  G->pos= yypos137; G->thunkpos= yythunkpos137;  if (!yymatchString(G, "")) goto l136;
  }
  l137:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "Title", G->buf+G->pos));
  return 1;
  l136:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Title", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Source(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Source"));  yyDo(G, yy_1_Source, G->begin, G->end);
  {  int yypos141= G->pos, yythunkpos141= G->thunkpos;  if (!yymatchChar(G, '<')) goto l142;  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l142;  if (!yy_SourceContents(G)) { goto l142; }  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l142;  yyDo(G, yy_2_Source, G->begin, G->end);  if (!yymatchChar(G, '>')) goto l142;  goto l141;
  l142:;	  G->pos= yypos141; G->thunkpos= yythunkpos141;  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l140;  if (!yy_SourceContents(G)) { goto l140; }  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l140;  yyDo(G, yy_3_Source, G->begin, G->end);
  }
  l141:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "Source", G->buf+G->pos));
  return 1;
  l140:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Source", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Label(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "Label"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l143;  if (!yy_LocMarker(G)) { goto l143; }  yyDo(G, yySet, -1, 0);  if (!yymatchChar(G, '[')) goto l143;
  {  int yypos144= G->pos, yythunkpos144= G->thunkpos;
  {  int yypos146= G->pos, yythunkpos146= G->thunkpos;  if (!yymatchChar(G, '^')) goto l146;  goto l145;
  l146:;	  G->pos= yypos146; G->thunkpos= yythunkpos146;
  }  yyText(G, G->begin, G->end);  if (!( EXT(pmh_EXT_NOTES) )) goto l145;  goto l144;
  l145:;	  G->pos= yypos144; G->thunkpos= yythunkpos144;
  {  int yypos147= G->pos, yythunkpos147= G->thunkpos;  if (!yymatchDot(G)) goto l143;  G->pos= yypos147; G->thunkpos= yythunkpos147;
  }  yyText(G, G->begin, G->end);  if (!( !EXT(pmh_EXT_NOTES) )) goto l143;
  }
  l144:;	  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l143;
  l148:;	
  {  int yypos149= G->pos, yythunkpos149= G->thunkpos;
  {  int yypos150= G->pos, yythunkpos150= G->thunkpos;  if (!yymatchChar(G, ']')) goto l150;  goto l149;
  l150:;	  G->pos= yypos150; G->thunkpos= yythunkpos150;
  }  if (!yy_Inline(G)) { goto l149; }  goto l148;
  l149:;	  G->pos= yypos149; G->thunkpos= yythunkpos149;
  }  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l143;  yyDo(G, yy_1_Label, G->begin, G->end);  if (!yymatchChar(G, ']')) goto l143;  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l143;  yyDo(G, yy_2_Label, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "Label", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l143:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Label", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_ReferenceLinkSingle(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "ReferenceLinkSingle"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l151;  if (!yy_Label(G)) { goto l151; }  yyDo(G, yySet, -1, 0);
  {  int yypos152= G->pos, yythunkpos152= G->thunkpos;  if (!yy_Spnl(G)) { goto l152; }  if (!yymatchString(G, "[]")) goto l152;  goto l153;
  l152:;	  G->pos= yypos152; G->thunkpos= yythunkpos152;
  }
  l153:;	  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l151;  yyDo(G, yy_1_ReferenceLinkSingle, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "ReferenceLinkSingle", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l151:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "ReferenceLinkSingle", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_ReferenceLinkDouble(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 2, 0);
  yyprintf((stderr, "%s\n", "ReferenceLinkDouble"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l154;  if (!yy_Label(G)) { goto l154; }  yyDo(G, yySet, -2, 0);  if (!yy_Spnl(G)) { goto l154; }
  {  int yypos155= G->pos, yythunkpos155= G->thunkpos;  if (!yymatchString(G, "[]")) goto l155;  goto l154;
  l155:;	  G->pos= yypos155; G->thunkpos= yythunkpos155;
  }  if (!yy_Label(G)) { goto l154; }  yyDo(G, yySet, -1, 0);  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l154;  yyDo(G, yy_1_ReferenceLinkDouble, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "ReferenceLinkDouble", G->buf+G->pos));  yyDo(G, yyPop, 2, 0);
  return 1;
  l154:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "ReferenceLinkDouble", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_AutoLink(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "AutoLink"));
  {  int yypos157= G->pos, yythunkpos157= G->thunkpos;  if (!yy_AutoLinkUrl(G)) { goto l158; }  goto l157;
  l158:;	  G->pos= yypos157; G->thunkpos= yythunkpos157;  if (!yy_AutoLinkEmail(G)) { goto l156; }
  }
  l157:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "AutoLink", G->buf+G->pos));
  return 1;
  l156:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "AutoLink", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_ReferenceLink(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "ReferenceLink"));
  {  int yypos160= G->pos, yythunkpos160= G->thunkpos;  if (!yy_ReferenceLinkDouble(G)) { goto l161; }  goto l160;
  l161:;	  G->pos= yypos160; G->thunkpos= yythunkpos160;  if (!yy_ReferenceLinkSingle(G)) { goto l159; }
  }
  l160:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "ReferenceLink", G->buf+G->pos));
  return 1;
  l159:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "ReferenceLink", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_ExplicitLink(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 2, 0);
  yyprintf((stderr, "%s\n", "ExplicitLink"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l162;  if (!yy_Label(G)) { goto l162; }  yyDo(G, yySet, -2, 0);  if (!yy_Spnl(G)) { goto l162; }  if (!yymatchChar(G, '(')) goto l162;  if (!yy_Sp(G)) { goto l162; }  if (!yy_Source(G)) { goto l162; }  yyDo(G, yySet, -1, 0);  if (!yy_Spnl(G)) { goto l162; }  if (!yy_Title(G)) { goto l162; }  if (!yy_Sp(G)) { goto l162; }  if (!yymatchChar(G, ')')) goto l162;  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l162;  yyDo(G, yy_1_ExplicitLink, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "ExplicitLink", G->buf+G->pos));  yyDo(G, yyPop, 2, 0);
  return 1;
  l162:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "ExplicitLink", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_TwoUlClose(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "TwoUlClose"));
  {  int yypos164= G->pos, yythunkpos164= G->thunkpos;  if (!yy_Spacechar(G)) { goto l164; }  goto l163;
  l164:;	  G->pos= yypos164; G->thunkpos= yythunkpos164;
  }
  {  int yypos165= G->pos, yythunkpos165= G->thunkpos;  if (!yy_Newline(G)) { goto l165; }  goto l163;
  l165:;	  G->pos= yypos165; G->thunkpos= yythunkpos165;
  }  if (!yy_Inline(G)) { goto l163; }  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l163;  if (!yymatchString(G, "__")) goto l163;  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l163;
  {  int yypos166= G->pos, yythunkpos166= G->thunkpos;  if (!yy_Alphanumeric(G)) { goto l166; }  goto l163;
  l166:;	  G->pos= yypos166; G->thunkpos= yythunkpos166;
  }  yyDo(G, yy_1_TwoUlClose, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "TwoUlClose", G->buf+G->pos));
  return 1;
  l163:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "TwoUlClose", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_TwoUlOpen(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "TwoUlOpen"));
  {  int yypos168= G->pos, yythunkpos168= G->thunkpos;  if (!yy_UlLine(G)) { goto l168; }  goto l167;
  l168:;	  G->pos= yypos168; G->thunkpos= yythunkpos168;
  }  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l167;  if (!yymatchString(G, "__")) goto l167;  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l167;
  {  int yypos169= G->pos, yythunkpos169= G->thunkpos;  if (!yy_Spacechar(G)) { goto l169; }  goto l167;
  l169:;	  G->pos= yypos169; G->thunkpos= yythunkpos169;
  }
  {  int yypos170= G->pos, yythunkpos170= G->thunkpos;  if (!yy_Newline(G)) { goto l170; }  goto l167;
  l170:;	  G->pos= yypos170; G->thunkpos= yythunkpos170;
  }  yyDo(G, yy_1_TwoUlOpen, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "TwoUlOpen", G->buf+G->pos));
  return 1;
  l167:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "TwoUlOpen", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_TwoStarClose(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "TwoStarClose"));
  {  int yypos172= G->pos, yythunkpos172= G->thunkpos;  if (!yy_Spacechar(G)) { goto l172; }  goto l171;
  l172:;	  G->pos= yypos172; G->thunkpos= yythunkpos172;
  }
  {  int yypos173= G->pos, yythunkpos173= G->thunkpos;  if (!yy_Newline(G)) { goto l173; }  goto l171;
  l173:;	  G->pos= yypos173; G->thunkpos= yythunkpos173;
  }  if (!yy_Inline(G)) { goto l171; }  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l171;  if (!yymatchString(G, "**")) goto l171;  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l171;  yyDo(G, yy_1_TwoStarClose, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "TwoStarClose", G->buf+G->pos));
  return 1;
  l171:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "TwoStarClose", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_TwoStarOpen(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "TwoStarOpen"));
  {  int yypos175= G->pos, yythunkpos175= G->thunkpos;  if (!yy_StarLine(G)) { goto l175; }  goto l174;
  l175:;	  G->pos= yypos175; G->thunkpos= yythunkpos175;
  }  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l174;  if (!yymatchString(G, "**")) goto l174;  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l174;
  {  int yypos176= G->pos, yythunkpos176= G->thunkpos;  if (!yy_Spacechar(G)) { goto l176; }  goto l174;
  l176:;	  G->pos= yypos176; G->thunkpos= yythunkpos176;
  }
  {  int yypos177= G->pos, yythunkpos177= G->thunkpos;  if (!yy_Newline(G)) { goto l177; }  goto l174;
  l177:;	  G->pos= yypos177; G->thunkpos= yythunkpos177;
  }  yyDo(G, yy_1_TwoStarOpen, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "TwoStarOpen", G->buf+G->pos));
  return 1;
  l174:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "TwoStarOpen", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_StrongUl(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "StrongUl"));  if (!yy_TwoUlOpen(G)) { goto l178; }  yyDo(G, yySet, -1, 0);
  l179:;	
  {  int yypos180= G->pos, yythunkpos180= G->thunkpos;
  {  int yypos181= G->pos, yythunkpos181= G->thunkpos;  if (!yy_TwoUlClose(G)) { goto l181; }  goto l180;
  l181:;	  G->pos= yypos181; G->thunkpos= yythunkpos181;
  }  if (!yy_Inline(G)) { goto l180; }  goto l179;
  l180:;	  G->pos= yypos180; G->thunkpos= yythunkpos180;
  }  if (!yy_TwoUlClose(G)) { goto l178; }  yyDo(G, yy_1_StrongUl, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "StrongUl", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l178:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "StrongUl", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_OneUlClose(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "OneUlClose"));
  {  int yypos183= G->pos, yythunkpos183= G->thunkpos;  if (!yy_Spacechar(G)) { goto l183; }  goto l182;
  l183:;	  G->pos= yypos183; G->thunkpos= yythunkpos183;
  }
  {  int yypos184= G->pos, yythunkpos184= G->thunkpos;  if (!yy_Newline(G)) { goto l184; }  goto l182;
  l184:;	  G->pos= yypos184; G->thunkpos= yythunkpos184;
  }  if (!yy_Inline(G)) { goto l182; }
  {  int yypos185= G->pos, yythunkpos185= G->thunkpos;  if (!yy_StrongUl(G)) { goto l185; }  goto l182;
  l185:;	  G->pos= yypos185; G->thunkpos= yythunkpos185;
  }  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l182;  if (!yymatchChar(G, '_')) goto l182;  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l182;
  {  int yypos186= G->pos, yythunkpos186= G->thunkpos;  if (!yy_Alphanumeric(G)) { goto l186; }  goto l182;
  l186:;	  G->pos= yypos186; G->thunkpos= yythunkpos186;
  }  yyDo(G, yy_1_OneUlClose, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "OneUlClose", G->buf+G->pos));
  return 1;
  l182:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "OneUlClose", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_OneUlOpen(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "OneUlOpen"));
  {  int yypos188= G->pos, yythunkpos188= G->thunkpos;  if (!yy_UlLine(G)) { goto l188; }  goto l187;
  l188:;	  G->pos= yypos188; G->thunkpos= yythunkpos188;
  }  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l187;  if (!yymatchChar(G, '_')) goto l187;  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l187;
  {  int yypos189= G->pos, yythunkpos189= G->thunkpos;  if (!yy_Spacechar(G)) { goto l189; }  goto l187;
  l189:;	  G->pos= yypos189; G->thunkpos= yythunkpos189;
  }
  {  int yypos190= G->pos, yythunkpos190= G->thunkpos;  if (!yy_Newline(G)) { goto l190; }  goto l187;
  l190:;	  G->pos= yypos190; G->thunkpos= yythunkpos190;
  }  yyDo(G, yy_1_OneUlOpen, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "OneUlOpen", G->buf+G->pos));
  return 1;
  l187:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "OneUlOpen", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_StrongStar(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "StrongStar"));  if (!yy_TwoStarOpen(G)) { goto l191; }  yyDo(G, yySet, -1, 0);
  l192:;	
  {  int yypos193= G->pos, yythunkpos193= G->thunkpos;
  {  int yypos194= G->pos, yythunkpos194= G->thunkpos;  if (!yy_TwoStarClose(G)) { goto l194; }  goto l193;
  l194:;	  G->pos= yypos194; G->thunkpos= yythunkpos194;
  }  if (!yy_Inline(G)) { goto l193; }  goto l192;
  l193:;	  G->pos= yypos193; G->thunkpos= yythunkpos193;
  }  if (!yy_TwoStarClose(G)) { goto l191; }  yyDo(G, yy_1_StrongStar, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "StrongStar", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l191:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "StrongStar", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_OneStarClose(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "OneStarClose"));
  {  int yypos196= G->pos, yythunkpos196= G->thunkpos;  if (!yy_Spacechar(G)) { goto l196; }  goto l195;
  l196:;	  G->pos= yypos196; G->thunkpos= yythunkpos196;
  }
  {  int yypos197= G->pos, yythunkpos197= G->thunkpos;  if (!yy_Newline(G)) { goto l197; }  goto l195;
  l197:;	  G->pos= yypos197; G->thunkpos= yythunkpos197;
  }  if (!yy_Inline(G)) { goto l195; }
  {  int yypos198= G->pos, yythunkpos198= G->thunkpos;  if (!yy_StrongStar(G)) { goto l198; }  goto l195;
  l198:;	  G->pos= yypos198; G->thunkpos= yythunkpos198;
  }  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l195;  if (!yymatchChar(G, '*')) goto l195;  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l195;  yyDo(G, yy_1_OneStarClose, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "OneStarClose", G->buf+G->pos));
  return 1;
  l195:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "OneStarClose", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_OneStarOpen(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "OneStarOpen"));
  {  int yypos200= G->pos, yythunkpos200= G->thunkpos;  if (!yy_StarLine(G)) { goto l200; }  goto l199;
  l200:;	  G->pos= yypos200; G->thunkpos= yythunkpos200;
  }  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l199;  if (!yymatchChar(G, '*')) goto l199;  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l199;
  {  int yypos201= G->pos, yythunkpos201= G->thunkpos;  if (!yy_Spacechar(G)) { goto l201; }  goto l199;
  l201:;	  G->pos= yypos201; G->thunkpos= yythunkpos201;
  }
  {  int yypos202= G->pos, yythunkpos202= G->thunkpos;  if (!yy_Newline(G)) { goto l202; }  goto l199;
  l202:;	  G->pos= yypos202; G->thunkpos= yythunkpos202;
  }  yyDo(G, yy_1_OneStarOpen, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "OneStarOpen", G->buf+G->pos));
  return 1;
  l199:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "OneStarOpen", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_EmphUl(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "EmphUl"));  if (!yy_OneUlOpen(G)) { goto l203; }  yyDo(G, yySet, -1, 0);
  l204:;	
  {  int yypos205= G->pos, yythunkpos205= G->thunkpos;
  {  int yypos206= G->pos, yythunkpos206= G->thunkpos;  if (!yy_OneUlClose(G)) { goto l206; }  goto l205;
  l206:;	  G->pos= yypos206; G->thunkpos= yythunkpos206;
  }  if (!yy_Inline(G)) { goto l205; }  goto l204;
  l205:;	  G->pos= yypos205; G->thunkpos= yythunkpos205;
  }  if (!yy_OneUlClose(G)) { goto l203; }  yyDo(G, yy_1_EmphUl, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "EmphUl", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l203:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "EmphUl", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_EmphStar(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "EmphStar"));  if (!yy_OneStarOpen(G)) { goto l207; }  yyDo(G, yySet, -1, 0);
  l208:;	
  {  int yypos209= G->pos, yythunkpos209= G->thunkpos;
  {  int yypos210= G->pos, yythunkpos210= G->thunkpos;  if (!yy_OneStarClose(G)) { goto l210; }  goto l209;
  l210:;	  G->pos= yypos210; G->thunkpos= yythunkpos210;
  }  if (!yy_Inline(G)) { goto l209; }  goto l208;
  l209:;	  G->pos= yypos209; G->thunkpos= yythunkpos209;
  }  if (!yy_OneStarClose(G)) { goto l207; }  yyDo(G, yy_1_EmphStar, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "EmphStar", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l207:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "EmphStar", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_StarLine(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "StarLine"));
  {  int yypos212= G->pos, yythunkpos212= G->thunkpos;  if (!yymatchString(G, "****")) goto l213;
  l214:;	
  {  int yypos215= G->pos, yythunkpos215= G->thunkpos;  if (!yymatchChar(G, '*')) goto l215;  goto l214;
  l215:;	  G->pos= yypos215; G->thunkpos= yythunkpos215;
  }  goto l212;
  l213:;	  G->pos= yypos212; G->thunkpos= yythunkpos212;  if (!yy_Spacechar(G)) { goto l211; }  if (!yymatchChar(G, '*')) goto l211;
  l216:;	
  {  int yypos217= G->pos, yythunkpos217= G->thunkpos;  if (!yymatchChar(G, '*')) goto l217;  goto l216;
  l217:;	  G->pos= yypos217; G->thunkpos= yythunkpos217;
  }
  {  int yypos218= G->pos, yythunkpos218= G->thunkpos;  if (!yy_Spacechar(G)) { goto l211; }  G->pos= yypos218; G->thunkpos= yythunkpos218;
  }
  }
  l212:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "StarLine", G->buf+G->pos));
  return 1;
  l211:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "StarLine", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_UlLine(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "UlLine"));
  {  int yypos220= G->pos, yythunkpos220= G->thunkpos;  if (!yymatchString(G, "____")) goto l221;
  l222:;	
  {  int yypos223= G->pos, yythunkpos223= G->thunkpos;  if (!yymatchChar(G, '_')) goto l223;  goto l222;
  l223:;	  G->pos= yypos223; G->thunkpos= yythunkpos223;
  }  goto l220;
  l221:;	  G->pos= yypos220; G->thunkpos= yythunkpos220;  if (!yy_Spacechar(G)) { goto l219; }  if (!yymatchChar(G, '_')) goto l219;
  l224:;	
  {  int yypos225= G->pos, yythunkpos225= G->thunkpos;  if (!yymatchChar(G, '_')) goto l225;  goto l224;
  l225:;	  G->pos= yypos225; G->thunkpos= yythunkpos225;
  }
  {  int yypos226= G->pos, yythunkpos226= G->thunkpos;  if (!yy_Spacechar(G)) { goto l219; }  G->pos= yypos226; G->thunkpos= yythunkpos226;
  }
  }
  l220:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "UlLine", G->buf+G->pos));
  return 1;
  l219:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "UlLine", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_SpecialChar(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "SpecialChar"));
  {  int yypos228= G->pos, yythunkpos228= G->thunkpos;  if (!yymatchChar(G, '*')) goto l229;  goto l228;
  l229:;	  G->pos= yypos228; G->thunkpos= yythunkpos228;  if (!yymatchChar(G, '_')) goto l230;  goto l228;
  l230:;	  G->pos= yypos228; G->thunkpos= yythunkpos228;  if (!yymatchChar(G, '`')) goto l231;  goto l228;
  l231:;	  G->pos= yypos228; G->thunkpos= yythunkpos228;  if (!yymatchChar(G, '&')) goto l232;  goto l228;
  l232:;	  G->pos= yypos228; G->thunkpos= yythunkpos228;  if (!yymatchChar(G, '[')) goto l233;  goto l228;
  l233:;	  G->pos= yypos228; G->thunkpos= yythunkpos228;  if (!yymatchChar(G, ']')) goto l234;  goto l228;
  l234:;	  G->pos= yypos228; G->thunkpos= yythunkpos228;  if (!yymatchChar(G, '(')) goto l235;  goto l228;
  l235:;	  G->pos= yypos228; G->thunkpos= yythunkpos228;  if (!yymatchChar(G, ')')) goto l236;  goto l228;
  l236:;	  G->pos= yypos228; G->thunkpos= yythunkpos228;  if (!yymatchChar(G, '<')) goto l237;  goto l228;
  l237:;	  G->pos= yypos228; G->thunkpos= yythunkpos228;  if (!yymatchChar(G, '!')) goto l238;  goto l228;
  l238:;	  G->pos= yypos228; G->thunkpos= yythunkpos228;  if (!yymatchChar(G, '#')) goto l239;  goto l228;
  l239:;	  G->pos= yypos228; G->thunkpos= yythunkpos228;  if (!yymatchChar(G, '\\')) goto l240;  goto l228;
  l240:;	  G->pos= yypos228; G->thunkpos= yythunkpos228;  if (!yymatchChar(G, '\'')) goto l241;  goto l228;
  l241:;	  G->pos= yypos228; G->thunkpos= yythunkpos228;  if (!yymatchChar(G, '"')) goto l242;  goto l228;
  l242:;	  G->pos= yypos228; G->thunkpos= yythunkpos228;  if (!yy_ExtendedSpecialChar(G)) { goto l227; }
  }
  l228:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "SpecialChar", G->buf+G->pos));
  return 1;
  l227:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "SpecialChar", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Eof(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Eof"));
  {  int yypos244= G->pos, yythunkpos244= G->thunkpos;  if (!yymatchDot(G)) goto l244;  goto l243;
  l244:;	  G->pos= yypos244; G->thunkpos= yythunkpos244;
  }
  yyprintf((stderr, "  ok   %s @ %s\n", "Eof", G->buf+G->pos));
  return 1;
  l243:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Eof", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_NormalEndline(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "NormalEndline"));  if (!yy_Sp(G)) { goto l245; }  if (!yy_Newline(G)) { goto l245; }
  {  int yypos246= G->pos, yythunkpos246= G->thunkpos;  if (!yy_BlankLine(G)) { goto l246; }  goto l245;
  l246:;	  G->pos= yypos246; G->thunkpos= yythunkpos246;
  }
  {  int yypos247= G->pos, yythunkpos247= G->thunkpos;  if (!yymatchChar(G, '>')) goto l247;  goto l245;
  l247:;	  G->pos= yypos247; G->thunkpos= yythunkpos247;
  }
  {  int yypos248= G->pos, yythunkpos248= G->thunkpos;  if (!yy_AtxStart(G)) { goto l248; }  goto l245;
  l248:;	  G->pos= yypos248; G->thunkpos= yythunkpos248;
  }
  {  int yypos249= G->pos, yythunkpos249= G->thunkpos;  if (!yy_Line(G)) { goto l249; }
  {  int yypos250= G->pos, yythunkpos250= G->thunkpos;  if (!yymatchString(G, "===")) goto l251;
  l252:;	
  {  int yypos253= G->pos, yythunkpos253= G->thunkpos;  if (!yymatchChar(G, '=')) goto l253;  goto l252;
  l253:;	  G->pos= yypos253; G->thunkpos= yythunkpos253;
  }  goto l250;
  l251:;	  G->pos= yypos250; G->thunkpos= yythunkpos250;  if (!yymatchString(G, "---")) goto l249;
  l254:;	
  {  int yypos255= G->pos, yythunkpos255= G->thunkpos;  if (!yymatchChar(G, '-')) goto l255;  goto l254;
  l255:;	  G->pos= yypos255; G->thunkpos= yythunkpos255;
  }
  }
  l250:;	  if (!yy_Newline(G)) { goto l249; }  goto l245;
  l249:;	  G->pos= yypos249; G->thunkpos= yythunkpos249;
  }
  yyprintf((stderr, "  ok   %s @ %s\n", "NormalEndline", G->buf+G->pos));
  return 1;
  l245:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "NormalEndline", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_TerminalEndline(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "TerminalEndline"));  if (!yy_Sp(G)) { goto l256; }  if (!yy_Newline(G)) { goto l256; }  if (!yy_Eof(G)) { goto l256; }
  yyprintf((stderr, "  ok   %s @ %s\n", "TerminalEndline", G->buf+G->pos));
  return 1;
  l256:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "TerminalEndline", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_LineBreak(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "LineBreak"));  if (!yymatchString(G, "  ")) goto l257;  if (!yy_NormalEndline(G)) { goto l257; }
  yyprintf((stderr, "  ok   %s @ %s\n", "LineBreak", G->buf+G->pos));
  return 1;
  l257:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "LineBreak", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_CharEntity(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "CharEntity"));  if (!yymatchChar(G, '&')) goto l258;  if (!yymatchClass(G, (unsigned char *)"\000\000\000\000\000\000\377\003\376\377\377\007\376\377\377\007\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l258;
  l259:;	
  {  int yypos260= G->pos, yythunkpos260= G->thunkpos;  if (!yymatchClass(G, (unsigned char *)"\000\000\000\000\000\000\377\003\376\377\377\007\376\377\377\007\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l260;  goto l259;
  l260:;	  G->pos= yypos260; G->thunkpos= yythunkpos260;
  }  if (!yymatchChar(G, ';')) goto l258;
  yyprintf((stderr, "  ok   %s @ %s\n", "CharEntity", G->buf+G->pos));
  return 1;
  l258:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "CharEntity", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_DecEntity(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "DecEntity"));  if (!yymatchChar(G, '&')) goto l261;  if (!yymatchChar(G, '#')) goto l261;  if (!yymatchClass(G, (unsigned char *)"\000\000\000\000\000\000\377\003\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l261;
  l262:;	
  {  int yypos263= G->pos, yythunkpos263= G->thunkpos;  if (!yymatchClass(G, (unsigned char *)"\000\000\000\000\000\000\377\003\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l263;  goto l262;
  l263:;	  G->pos= yypos263; G->thunkpos= yythunkpos263;
  }  if (!yymatchChar(G, ';')) goto l261;
  yyprintf((stderr, "  ok   %s @ %s\n", "DecEntity", G->buf+G->pos));
  return 1;
  l261:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "DecEntity", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HexEntity(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HexEntity"));  if (!yymatchChar(G, '&')) goto l264;  if (!yymatchChar(G, '#')) goto l264;  if (!yymatchClass(G, (unsigned char *)"\000\000\000\000\000\000\000\000\000\000\000\001\000\000\000\001\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l264;  if (!yymatchClass(G, (unsigned char *)"\000\000\000\000\000\000\377\003\176\000\000\000\176\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l264;
  l265:;	
  {  int yypos266= G->pos, yythunkpos266= G->thunkpos;  if (!yymatchClass(G, (unsigned char *)"\000\000\000\000\000\000\377\003\176\000\000\000\176\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l266;  goto l265;
  l266:;	  G->pos= yypos266; G->thunkpos= yythunkpos266;
  }  if (!yymatchChar(G, ';')) goto l264;
  yyprintf((stderr, "  ok   %s @ %s\n", "HexEntity", G->buf+G->pos));
  return 1;
  l264:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HexEntity", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Alphanumeric(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Alphanumeric"));
  {  int yypos268= G->pos, yythunkpos268= G->thunkpos;  if (!yymatchClass(G, (unsigned char *)"\000\000\000\000\000\000\377\003\376\377\377\007\376\377\377\007\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l269;  goto l268;
  l269:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\200")) goto l270;  goto l268;
  l270:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\201")) goto l271;  goto l268;
  l271:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\202")) goto l272;  goto l268;
  l272:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\203")) goto l273;  goto l268;
  l273:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\204")) goto l274;  goto l268;
  l274:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\205")) goto l275;  goto l268;
  l275:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\206")) goto l276;  goto l268;
  l276:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\207")) goto l277;  goto l268;
  l277:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\210")) goto l278;  goto l268;
  l278:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\211")) goto l279;  goto l268;
  l279:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\212")) goto l280;  goto l268;
  l280:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\213")) goto l281;  goto l268;
  l281:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\214")) goto l282;  goto l268;
  l282:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\215")) goto l283;  goto l268;
  l283:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\216")) goto l284;  goto l268;
  l284:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\217")) goto l285;  goto l268;
  l285:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\220")) goto l286;  goto l268;
  l286:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\221")) goto l287;  goto l268;
  l287:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\222")) goto l288;  goto l268;
  l288:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\223")) goto l289;  goto l268;
  l289:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\224")) goto l290;  goto l268;
  l290:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\225")) goto l291;  goto l268;
  l291:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\226")) goto l292;  goto l268;
  l292:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\227")) goto l293;  goto l268;
  l293:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\230")) goto l294;  goto l268;
  l294:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\231")) goto l295;  goto l268;
  l295:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\232")) goto l296;  goto l268;
  l296:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\233")) goto l297;  goto l268;
  l297:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\234")) goto l298;  goto l268;
  l298:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\235")) goto l299;  goto l268;
  l299:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\236")) goto l300;  goto l268;
  l300:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\237")) goto l301;  goto l268;
  l301:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\240")) goto l302;  goto l268;
  l302:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\241")) goto l303;  goto l268;
  l303:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\242")) goto l304;  goto l268;
  l304:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\243")) goto l305;  goto l268;
  l305:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\244")) goto l306;  goto l268;
  l306:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\245")) goto l307;  goto l268;
  l307:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\246")) goto l308;  goto l268;
  l308:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\247")) goto l309;  goto l268;
  l309:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\250")) goto l310;  goto l268;
  l310:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\251")) goto l311;  goto l268;
  l311:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\252")) goto l312;  goto l268;
  l312:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\253")) goto l313;  goto l268;
  l313:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\254")) goto l314;  goto l268;
  l314:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\255")) goto l315;  goto l268;
  l315:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\256")) goto l316;  goto l268;
  l316:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\257")) goto l317;  goto l268;
  l317:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\260")) goto l318;  goto l268;
  l318:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\261")) goto l319;  goto l268;
  l319:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\262")) goto l320;  goto l268;
  l320:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\263")) goto l321;  goto l268;
  l321:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\264")) goto l322;  goto l268;
  l322:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\265")) goto l323;  goto l268;
  l323:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\266")) goto l324;  goto l268;
  l324:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\267")) goto l325;  goto l268;
  l325:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\270")) goto l326;  goto l268;
  l326:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\271")) goto l327;  goto l268;
  l327:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\272")) goto l328;  goto l268;
  l328:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\273")) goto l329;  goto l268;
  l329:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\274")) goto l330;  goto l268;
  l330:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\275")) goto l331;  goto l268;
  l331:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\276")) goto l332;  goto l268;
  l332:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\277")) goto l333;  goto l268;
  l333:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\300")) goto l334;  goto l268;
  l334:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\301")) goto l335;  goto l268;
  l335:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\302")) goto l336;  goto l268;
  l336:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\303")) goto l337;  goto l268;
  l337:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\304")) goto l338;  goto l268;
  l338:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\305")) goto l339;  goto l268;
  l339:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\306")) goto l340;  goto l268;
  l340:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\307")) goto l341;  goto l268;
  l341:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\310")) goto l342;  goto l268;
  l342:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\311")) goto l343;  goto l268;
  l343:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\312")) goto l344;  goto l268;
  l344:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\313")) goto l345;  goto l268;
  l345:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\314")) goto l346;  goto l268;
  l346:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\315")) goto l347;  goto l268;
  l347:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\316")) goto l348;  goto l268;
  l348:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\317")) goto l349;  goto l268;
  l349:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\320")) goto l350;  goto l268;
  l350:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\321")) goto l351;  goto l268;
  l351:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\322")) goto l352;  goto l268;
  l352:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\323")) goto l353;  goto l268;
  l353:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\324")) goto l354;  goto l268;
  l354:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\325")) goto l355;  goto l268;
  l355:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\326")) goto l356;  goto l268;
  l356:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\327")) goto l357;  goto l268;
  l357:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\330")) goto l358;  goto l268;
  l358:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\331")) goto l359;  goto l268;
  l359:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\332")) goto l360;  goto l268;
  l360:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\333")) goto l361;  goto l268;
  l361:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\334")) goto l362;  goto l268;
  l362:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\335")) goto l363;  goto l268;
  l363:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\336")) goto l364;  goto l268;
  l364:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\337")) goto l365;  goto l268;
  l365:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\340")) goto l366;  goto l268;
  l366:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\341")) goto l367;  goto l268;
  l367:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\342")) goto l368;  goto l268;
  l368:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\343")) goto l369;  goto l268;
  l369:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\344")) goto l370;  goto l268;
  l370:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\345")) goto l371;  goto l268;
  l371:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\346")) goto l372;  goto l268;
  l372:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\347")) goto l373;  goto l268;
  l373:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\350")) goto l374;  goto l268;
  l374:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\351")) goto l375;  goto l268;
  l375:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\352")) goto l376;  goto l268;
  l376:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\353")) goto l377;  goto l268;
  l377:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\354")) goto l378;  goto l268;
  l378:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\355")) goto l379;  goto l268;
  l379:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\356")) goto l380;  goto l268;
  l380:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\357")) goto l381;  goto l268;
  l381:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\360")) goto l382;  goto l268;
  l382:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\361")) goto l383;  goto l268;
  l383:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\362")) goto l384;  goto l268;
  l384:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\363")) goto l385;  goto l268;
  l385:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\364")) goto l386;  goto l268;
  l386:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\365")) goto l387;  goto l268;
  l387:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\366")) goto l388;  goto l268;
  l388:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\367")) goto l389;  goto l268;
  l389:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\370")) goto l390;  goto l268;
  l390:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\371")) goto l391;  goto l268;
  l391:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\372")) goto l392;  goto l268;
  l392:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\373")) goto l393;  goto l268;
  l393:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\374")) goto l394;  goto l268;
  l394:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\375")) goto l395;  goto l268;
  l395:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\376")) goto l396;  goto l268;
  l396:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;  if (!yymatchString(G, "\377")) goto l267;
  }
  l268:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "Alphanumeric", G->buf+G->pos));
  return 1;
  l267:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Alphanumeric", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_NormalChar(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "NormalChar"));
  {  int yypos398= G->pos, yythunkpos398= G->thunkpos;
  {  int yypos399= G->pos, yythunkpos399= G->thunkpos;  if (!yy_SpecialChar(G)) { goto l400; }  goto l399;
  l400:;	  G->pos= yypos399; G->thunkpos= yythunkpos399;  if (!yy_Spacechar(G)) { goto l401; }  goto l399;
  l401:;	  G->pos= yypos399; G->thunkpos= yythunkpos399;  if (!yy_Newline(G)) { goto l398; }
  }
  l399:;	  goto l397;
  l398:;	  G->pos= yypos398; G->thunkpos= yythunkpos398;
  }  if (!yymatchDot(G)) goto l397;
  yyprintf((stderr, "  ok   %s @ %s\n", "NormalChar", G->buf+G->pos));
  return 1;
  l397:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "NormalChar", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Symbol(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Symbol"));  if (!yy_SpecialChar(G)) { goto l402; }
  yyprintf((stderr, "  ok   %s @ %s\n", "Symbol", G->buf+G->pos));
  return 1;
  l402:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Symbol", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_EscapedChar(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "EscapedChar"));  if (!yymatchChar(G, '\\')) goto l403;
  {  int yypos404= G->pos, yythunkpos404= G->thunkpos;  if (!yy_Newline(G)) { goto l404; }  goto l403;
  l404:;	  G->pos= yypos404; G->thunkpos= yythunkpos404;
  }  if (!yymatchClass(G, (unsigned char *)"\000\000\000\000\012\157\000\120\000\000\000\270\001\000\000\070\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l403;
  yyprintf((stderr, "  ok   %s @ %s\n", "EscapedChar", G->buf+G->pos));
  return 1;
  l403:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "EscapedChar", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Entity(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "Entity"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l405;  if (!yy_LocMarker(G)) { goto l405; }  yyDo(G, yySet, -1, 0);
  {  int yypos406= G->pos, yythunkpos406= G->thunkpos;  if (!yy_HexEntity(G)) { goto l407; }  goto l406;
  l407:;	  G->pos= yypos406; G->thunkpos= yythunkpos406;  if (!yy_DecEntity(G)) { goto l408; }  goto l406;
  l408:;	  G->pos= yypos406; G->thunkpos= yythunkpos406;  if (!yy_CharEntity(G)) { goto l405; }
  }
  l406:;	  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l405;  yyDo(G, yy_1_Entity, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "Entity", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l405:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Entity", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_RawHtml(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "RawHtml"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l409;  if (!yy_LocMarker(G)) { goto l409; }  yyDo(G, yySet, -1, 0);
  {  int yypos410= G->pos, yythunkpos410= G->thunkpos;  if (!yy_HtmlComment(G)) { goto l411; }  goto l410;
  l411:;	  G->pos= yypos410; G->thunkpos= yythunkpos410;  if (!yy_HtmlBlockScript(G)) { goto l412; }  goto l410;
  l412:;	  G->pos= yypos410; G->thunkpos= yythunkpos410;  if (!yy_HtmlTag(G)) { goto l409; }
  }
  l410:;	  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l409;  yyDo(G, yy_1_RawHtml, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "RawHtml", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l409:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "RawHtml", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Code(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "Code"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l413;
  {  int yypos414= G->pos, yythunkpos414= G->thunkpos;  if (!yy_Ticks1(G)) { goto l415; }  yyDo(G, yySet, -1, 0);  if (!yy_Sp(G)) { goto l415; }
  {  int yypos418= G->pos, yythunkpos418= G->thunkpos;
  {  int yypos422= G->pos, yythunkpos422= G->thunkpos;  if (!yymatchChar(G, '`')) goto l422;  goto l419;
  l422:;	  G->pos= yypos422; G->thunkpos= yythunkpos422;
  }  if (!yy_Nonspacechar(G)) { goto l419; }
  l420:;	
  {  int yypos421= G->pos, yythunkpos421= G->thunkpos;
  {  int yypos423= G->pos, yythunkpos423= G->thunkpos;  if (!yymatchChar(G, '`')) goto l423;  goto l421;
  l423:;	  G->pos= yypos423; G->thunkpos= yythunkpos423;
  }  if (!yy_Nonspacechar(G)) { goto l421; }  goto l420;
  l421:;	  G->pos= yypos421; G->thunkpos= yythunkpos421;
  }  goto l418;
  l419:;	  G->pos= yypos418; G->thunkpos= yythunkpos418;
  {  int yypos425= G->pos, yythunkpos425= G->thunkpos;  if (!yy_Ticks1(G)) { goto l425; }  goto l424;
  l425:;	  G->pos= yypos425; G->thunkpos= yythunkpos425;
  }  if (!yymatchChar(G, '`')) goto l424;
  l426:;	
  {  int yypos427= G->pos, yythunkpos427= G->thunkpos;  if (!yymatchChar(G, '`')) goto l427;  goto l426;
  l427:;	  G->pos= yypos427; G->thunkpos= yythunkpos427;
  }  goto l418;
  l424:;	  G->pos= yypos418; G->thunkpos= yythunkpos418;
  {  int yypos428= G->pos, yythunkpos428= G->thunkpos;  if (!yy_Sp(G)) { goto l428; }  if (!yy_Ticks1(G)) { goto l428; }  goto l415;
  l428:;	  G->pos= yypos428; G->thunkpos= yythunkpos428;
  }
  {  int yypos429= G->pos, yythunkpos429= G->thunkpos;  if (!yy_Spacechar(G)) { goto l430; }  goto l429;
  l430:;	  G->pos= yypos429; G->thunkpos= yythunkpos429;  if (!yy_Newline(G)) { goto l415; }
  {  int yypos431= G->pos, yythunkpos431= G->thunkpos;  if (!yy_BlankLine(G)) { goto l431; }  goto l415;
  l431:;	  G->pos= yypos431; G->thunkpos= yythunkpos431;
  }
  }
  l429:;	
  }
  l418:;	
  l416:;	
  {  int yypos417= G->pos, yythunkpos417= G->thunkpos;
  {  int yypos432= G->pos, yythunkpos432= G->thunkpos;
  {  int yypos436= G->pos, yythunkpos436= G->thunkpos;  if (!yymatchChar(G, '`')) goto l436;  goto l433;
  l436:;	  G->pos= yypos436; G->thunkpos= yythunkpos436;
  }  if (!yy_Nonspacechar(G)) { goto l433; }
  l434:;	
  {  int yypos435= G->pos, yythunkpos435= G->thunkpos;
  {  int yypos437= G->pos, yythunkpos437= G->thunkpos;  if (!yymatchChar(G, '`')) goto l437;  goto l435;
  l437:;	  G->pos= yypos437; G->thunkpos= yythunkpos437;
  }  if (!yy_Nonspacechar(G)) { goto l435; }  goto l434;
  l435:;	  G->pos= yypos435; G->thunkpos= yythunkpos435;
  }  goto l432;
  l433:;	  G->pos= yypos432; G->thunkpos= yythunkpos432;
  {  int yypos439= G->pos, yythunkpos439= G->thunkpos;  if (!yy_Ticks1(G)) { goto l439; }  goto l438;
  l439:;	  G->pos= yypos439; G->thunkpos= yythunkpos439;
  }  if (!yymatchChar(G, '`')) goto l438;
  l440:;	
  {  int yypos441= G->pos, yythunkpos441= G->thunkpos;  if (!yymatchChar(G, '`')) goto l441;  goto l440;
  l441:;	  G->pos= yypos441; G->thunkpos= yythunkpos441;
  }  goto l432;
  l438:;	  G->pos= yypos432; G->thunkpos= yythunkpos432;
  {  int yypos442= G->pos, yythunkpos442= G->thunkpos;  if (!yy_Sp(G)) { goto l442; }  if (!yy_Ticks1(G)) { goto l442; }  goto l417;
  l442:;	  G->pos= yypos442; G->thunkpos= yythunkpos442;
  }
  {  int yypos443= G->pos, yythunkpos443= G->thunkpos;  if (!yy_Spacechar(G)) { goto l444; }  goto l443;
  l444:;	  G->pos= yypos443; G->thunkpos= yythunkpos443;  if (!yy_Newline(G)) { goto l417; }
  {  int yypos445= G->pos, yythunkpos445= G->thunkpos;  if (!yy_BlankLine(G)) { goto l445; }  goto l417;
  l445:;	  G->pos= yypos445; G->thunkpos= yythunkpos445;
  }
  }
  l443:;	
  }
  l432:;	  goto l416;
  l417:;	  G->pos= yypos417; G->thunkpos= yythunkpos417;
  }  if (!yy_Sp(G)) { goto l415; }  if (!yy_Ticks1(G)) { goto l415; }  goto l414;
  l415:;	  G->pos= yypos414; G->thunkpos= yythunkpos414;  if (!yy_Ticks2(G)) { goto l446; }  yyDo(G, yySet, -1, 0);  if (!yy_Sp(G)) { goto l446; }
  {  int yypos449= G->pos, yythunkpos449= G->thunkpos;
  {  int yypos453= G->pos, yythunkpos453= G->thunkpos;  if (!yymatchChar(G, '`')) goto l453;  goto l450;
  l453:;	  G->pos= yypos453; G->thunkpos= yythunkpos453;
  }  if (!yy_Nonspacechar(G)) { goto l450; }
  l451:;	
  {  int yypos452= G->pos, yythunkpos452= G->thunkpos;
  {  int yypos454= G->pos, yythunkpos454= G->thunkpos;  if (!yymatchChar(G, '`')) goto l454;  goto l452;
  l454:;	  G->pos= yypos454; G->thunkpos= yythunkpos454;
  }  if (!yy_Nonspacechar(G)) { goto l452; }  goto l451;
  l452:;	  G->pos= yypos452; G->thunkpos= yythunkpos452;
  }  goto l449;
  l450:;	  G->pos= yypos449; G->thunkpos= yythunkpos449;
  {  int yypos456= G->pos, yythunkpos456= G->thunkpos;  if (!yy_Ticks2(G)) { goto l456; }  goto l455;
  l456:;	  G->pos= yypos456; G->thunkpos= yythunkpos456;
  }  if (!yymatchChar(G, '`')) goto l455;
  l457:;	
  {  int yypos458= G->pos, yythunkpos458= G->thunkpos;  if (!yymatchChar(G, '`')) goto l458;  goto l457;
  l458:;	  G->pos= yypos458; G->thunkpos= yythunkpos458;
  }  goto l449;
  l455:;	  G->pos= yypos449; G->thunkpos= yythunkpos449;
  {  int yypos459= G->pos, yythunkpos459= G->thunkpos;  if (!yy_Sp(G)) { goto l459; }  if (!yy_Ticks2(G)) { goto l459; }  goto l446;
  l459:;	  G->pos= yypos459; G->thunkpos= yythunkpos459;
  }
  {  int yypos460= G->pos, yythunkpos460= G->thunkpos;  if (!yy_Spacechar(G)) { goto l461; }  goto l460;
  l461:;	  G->pos= yypos460; G->thunkpos= yythunkpos460;  if (!yy_Newline(G)) { goto l446; }
  {  int yypos462= G->pos, yythunkpos462= G->thunkpos;  if (!yy_BlankLine(G)) { goto l462; }  goto l446;
  l462:;	  G->pos= yypos462; G->thunkpos= yythunkpos462;
  }
  }
  l460:;	
  }
  l449:;	
  l447:;	
  {  int yypos448= G->pos, yythunkpos448= G->thunkpos;
  {  int yypos463= G->pos, yythunkpos463= G->thunkpos;
  {  int yypos467= G->pos, yythunkpos467= G->thunkpos;  if (!yymatchChar(G, '`')) goto l467;  goto l464;
  l467:;	  G->pos= yypos467; G->thunkpos= yythunkpos467;
  }  if (!yy_Nonspacechar(G)) { goto l464; }
  l465:;	
  {  int yypos466= G->pos, yythunkpos466= G->thunkpos;
  {  int yypos468= G->pos, yythunkpos468= G->thunkpos;  if (!yymatchChar(G, '`')) goto l468;  goto l466;
  l468:;	  G->pos= yypos468; G->thunkpos= yythunkpos468;
  }  if (!yy_Nonspacechar(G)) { goto l466; }  goto l465;
  l466:;	  G->pos= yypos466; G->thunkpos= yythunkpos466;
  }  goto l463;
  l464:;	  G->pos= yypos463; G->thunkpos= yythunkpos463;
  {  int yypos470= G->pos, yythunkpos470= G->thunkpos;  if (!yy_Ticks2(G)) { goto l470; }  goto l469;
  l470:;	  G->pos= yypos470; G->thunkpos= yythunkpos470;
  }  if (!yymatchChar(G, '`')) goto l469;
  l471:;	
  {  int yypos472= G->pos, yythunkpos472= G->thunkpos;  if (!yymatchChar(G, '`')) goto l472;  goto l471;
  l472:;	  G->pos= yypos472; G->thunkpos= yythunkpos472;
  }  goto l463;
  l469:;	  G->pos= yypos463; G->thunkpos= yythunkpos463;
  {  int yypos473= G->pos, yythunkpos473= G->thunkpos;  if (!yy_Sp(G)) { goto l473; }  if (!yy_Ticks2(G)) { goto l473; }  goto l448;
  l473:;	  G->pos= yypos473; G->thunkpos= yythunkpos473;
  }
  {  int yypos474= G->pos, yythunkpos474= G->thunkpos;  if (!yy_Spacechar(G)) { goto l475; }  goto l474;
  l475:;	  G->pos= yypos474; G->thunkpos= yythunkpos474;  if (!yy_Newline(G)) { goto l448; }
  {  int yypos476= G->pos, yythunkpos476= G->thunkpos;  if (!yy_BlankLine(G)) { goto l476; }  goto l448;
  l476:;	  G->pos= yypos476; G->thunkpos= yythunkpos476;
  }
  }
  l474:;	
  }
  l463:;	  goto l447;
  l448:;	  G->pos= yypos448; G->thunkpos= yythunkpos448;
  }  if (!yy_Sp(G)) { goto l446; }  if (!yy_Ticks2(G)) { goto l446; }  goto l414;
  l446:;	  G->pos= yypos414; G->thunkpos= yythunkpos414;  if (!yy_Ticks3(G)) { goto l477; }  yyDo(G, yySet, -1, 0);  if (!yy_Sp(G)) { goto l477; }
  {  int yypos480= G->pos, yythunkpos480= G->thunkpos;
  {  int yypos484= G->pos, yythunkpos484= G->thunkpos;  if (!yymatchChar(G, '`')) goto l484;  goto l481;
  l484:;	  G->pos= yypos484; G->thunkpos= yythunkpos484;
  }  if (!yy_Nonspacechar(G)) { goto l481; }
  l482:;	
  {  int yypos483= G->pos, yythunkpos483= G->thunkpos;
  {  int yypos485= G->pos, yythunkpos485= G->thunkpos;  if (!yymatchChar(G, '`')) goto l485;  goto l483;
  l485:;	  G->pos= yypos485; G->thunkpos= yythunkpos485;
  }  if (!yy_Nonspacechar(G)) { goto l483; }  goto l482;
  l483:;	  G->pos= yypos483; G->thunkpos= yythunkpos483;
  }  goto l480;
  l481:;	  G->pos= yypos480; G->thunkpos= yythunkpos480;
  {  int yypos487= G->pos, yythunkpos487= G->thunkpos;  if (!yy_Ticks3(G)) { goto l487; }  goto l486;
  l487:;	  G->pos= yypos487; G->thunkpos= yythunkpos487;
  }  if (!yymatchChar(G, '`')) goto l486;
  l488:;	
  {  int yypos489= G->pos, yythunkpos489= G->thunkpos;  if (!yymatchChar(G, '`')) goto l489;  goto l488;
  l489:;	  G->pos= yypos489; G->thunkpos= yythunkpos489;
  }  goto l480;
  l486:;	  G->pos= yypos480; G->thunkpos= yythunkpos480;
  {  int yypos490= G->pos, yythunkpos490= G->thunkpos;  if (!yy_Sp(G)) { goto l490; }  if (!yy_Ticks3(G)) { goto l490; }  goto l477;
  l490:;	  G->pos= yypos490; G->thunkpos= yythunkpos490;
  }
  {  int yypos491= G->pos, yythunkpos491= G->thunkpos;  if (!yy_Spacechar(G)) { goto l492; }  goto l491;
  l492:;	  G->pos= yypos491; G->thunkpos= yythunkpos491;  if (!yy_Newline(G)) { goto l477; }
  {  int yypos493= G->pos, yythunkpos493= G->thunkpos;  if (!yy_BlankLine(G)) { goto l493; }  goto l477;
  l493:;	  G->pos= yypos493; G->thunkpos= yythunkpos493;
  }
  }
  l491:;	
  }
  l480:;	
  l478:;	
  {  int yypos479= G->pos, yythunkpos479= G->thunkpos;
  {  int yypos494= G->pos, yythunkpos494= G->thunkpos;
  {  int yypos498= G->pos, yythunkpos498= G->thunkpos;  if (!yymatchChar(G, '`')) goto l498;  goto l495;
  l498:;	  G->pos= yypos498; G->thunkpos= yythunkpos498;
  }  if (!yy_Nonspacechar(G)) { goto l495; }
  l496:;	
  {  int yypos497= G->pos, yythunkpos497= G->thunkpos;
  {  int yypos499= G->pos, yythunkpos499= G->thunkpos;  if (!yymatchChar(G, '`')) goto l499;  goto l497;
  l499:;	  G->pos= yypos499; G->thunkpos= yythunkpos499;
  }  if (!yy_Nonspacechar(G)) { goto l497; }  goto l496;
  l497:;	  G->pos= yypos497; G->thunkpos= yythunkpos497;
  }  goto l494;
  l495:;	  G->pos= yypos494; G->thunkpos= yythunkpos494;
  {  int yypos501= G->pos, yythunkpos501= G->thunkpos;  if (!yy_Ticks3(G)) { goto l501; }  goto l500;
  l501:;	  G->pos= yypos501; G->thunkpos= yythunkpos501;
  }  if (!yymatchChar(G, '`')) goto l500;
  l502:;	
  {  int yypos503= G->pos, yythunkpos503= G->thunkpos;  if (!yymatchChar(G, '`')) goto l503;  goto l502;
  l503:;	  G->pos= yypos503; G->thunkpos= yythunkpos503;
  }  goto l494;
  l500:;	  G->pos= yypos494; G->thunkpos= yythunkpos494;
  {  int yypos504= G->pos, yythunkpos504= G->thunkpos;  if (!yy_Sp(G)) { goto l504; }  if (!yy_Ticks3(G)) { goto l504; }  goto l479;
  l504:;	  G->pos= yypos504; G->thunkpos= yythunkpos504;
  }
  {  int yypos505= G->pos, yythunkpos505= G->thunkpos;  if (!yy_Spacechar(G)) { goto l506; }  goto l505;
  l506:;	  G->pos= yypos505; G->thunkpos= yythunkpos505;  if (!yy_Newline(G)) { goto l479; }
  {  int yypos507= G->pos, yythunkpos507= G->thunkpos;  if (!yy_BlankLine(G)) { goto l507; }  goto l479;
  l507:;	  G->pos= yypos507; G->thunkpos= yythunkpos507;
  }
  }
  l505:;	
  }
  l494:;	  goto l478;
  l479:;	  G->pos= yypos479; G->thunkpos= yythunkpos479;
  }  if (!yy_Sp(G)) { goto l477; }  if (!yy_Ticks3(G)) { goto l477; }  goto l414;
  l477:;	  G->pos= yypos414; G->thunkpos= yythunkpos414;  if (!yy_Ticks4(G)) { goto l508; }  yyDo(G, yySet, -1, 0);  if (!yy_Sp(G)) { goto l508; }
  {  int yypos511= G->pos, yythunkpos511= G->thunkpos;
  {  int yypos515= G->pos, yythunkpos515= G->thunkpos;  if (!yymatchChar(G, '`')) goto l515;  goto l512;
  l515:;	  G->pos= yypos515; G->thunkpos= yythunkpos515;
  }  if (!yy_Nonspacechar(G)) { goto l512; }
  l513:;	
  {  int yypos514= G->pos, yythunkpos514= G->thunkpos;
  {  int yypos516= G->pos, yythunkpos516= G->thunkpos;  if (!yymatchChar(G, '`')) goto l516;  goto l514;
  l516:;	  G->pos= yypos516; G->thunkpos= yythunkpos516;
  }  if (!yy_Nonspacechar(G)) { goto l514; }  goto l513;
  l514:;	  G->pos= yypos514; G->thunkpos= yythunkpos514;
  }  goto l511;
  l512:;	  G->pos= yypos511; G->thunkpos= yythunkpos511;
  {  int yypos518= G->pos, yythunkpos518= G->thunkpos;  if (!yy_Ticks4(G)) { goto l518; }  goto l517;
  l518:;	  G->pos= yypos518; G->thunkpos= yythunkpos518;
  }  if (!yymatchChar(G, '`')) goto l517;
  l519:;	
  {  int yypos520= G->pos, yythunkpos520= G->thunkpos;  if (!yymatchChar(G, '`')) goto l520;  goto l519;
  l520:;	  G->pos= yypos520; G->thunkpos= yythunkpos520;
  }  goto l511;
  l517:;	  G->pos= yypos511; G->thunkpos= yythunkpos511;
  {  int yypos521= G->pos, yythunkpos521= G->thunkpos;  if (!yy_Sp(G)) { goto l521; }  if (!yy_Ticks4(G)) { goto l521; }  goto l508;
  l521:;	  G->pos= yypos521; G->thunkpos= yythunkpos521;
  }
  {  int yypos522= G->pos, yythunkpos522= G->thunkpos;  if (!yy_Spacechar(G)) { goto l523; }  goto l522;
  l523:;	  G->pos= yypos522; G->thunkpos= yythunkpos522;  if (!yy_Newline(G)) { goto l508; }
  {  int yypos524= G->pos, yythunkpos524= G->thunkpos;  if (!yy_BlankLine(G)) { goto l524; }  goto l508;
  l524:;	  G->pos= yypos524; G->thunkpos= yythunkpos524;
  }
  }
  l522:;	
  }
  l511:;	
  l509:;	
  {  int yypos510= G->pos, yythunkpos510= G->thunkpos;
  {  int yypos525= G->pos, yythunkpos525= G->thunkpos;
  {  int yypos529= G->pos, yythunkpos529= G->thunkpos;  if (!yymatchChar(G, '`')) goto l529;  goto l526;
  l529:;	  G->pos= yypos529; G->thunkpos= yythunkpos529;
  }  if (!yy_Nonspacechar(G)) { goto l526; }
  l527:;	
  {  int yypos528= G->pos, yythunkpos528= G->thunkpos;
  {  int yypos530= G->pos, yythunkpos530= G->thunkpos;  if (!yymatchChar(G, '`')) goto l530;  goto l528;
  l530:;	  G->pos= yypos530; G->thunkpos= yythunkpos530;
  }  if (!yy_Nonspacechar(G)) { goto l528; }  goto l527;
  l528:;	  G->pos= yypos528; G->thunkpos= yythunkpos528;
  }  goto l525;
  l526:;	  G->pos= yypos525; G->thunkpos= yythunkpos525;
  {  int yypos532= G->pos, yythunkpos532= G->thunkpos;  if (!yy_Ticks4(G)) { goto l532; }  goto l531;
  l532:;	  G->pos= yypos532; G->thunkpos= yythunkpos532;
  }  if (!yymatchChar(G, '`')) goto l531;
  l533:;	
  {  int yypos534= G->pos, yythunkpos534= G->thunkpos;  if (!yymatchChar(G, '`')) goto l534;  goto l533;
  l534:;	  G->pos= yypos534; G->thunkpos= yythunkpos534;
  }  goto l525;
  l531:;	  G->pos= yypos525; G->thunkpos= yythunkpos525;
  {  int yypos535= G->pos, yythunkpos535= G->thunkpos;  if (!yy_Sp(G)) { goto l535; }  if (!yy_Ticks4(G)) { goto l535; }  goto l510;
  l535:;	  G->pos= yypos535; G->thunkpos= yythunkpos535;
  }
  {  int yypos536= G->pos, yythunkpos536= G->thunkpos;  if (!yy_Spacechar(G)) { goto l537; }  goto l536;
  l537:;	  G->pos= yypos536; G->thunkpos= yythunkpos536;  if (!yy_Newline(G)) { goto l510; }
  {  int yypos538= G->pos, yythunkpos538= G->thunkpos;  if (!yy_BlankLine(G)) { goto l538; }  goto l510;
  l538:;	  G->pos= yypos538; G->thunkpos= yythunkpos538;
  }
  }
  l536:;	
  }
  l525:;	  goto l509;
  l510:;	  G->pos= yypos510; G->thunkpos= yythunkpos510;
  }  if (!yy_Sp(G)) { goto l508; }  if (!yy_Ticks4(G)) { goto l508; }  goto l414;
  l508:;	  G->pos= yypos414; G->thunkpos= yythunkpos414;  if (!yy_Ticks5(G)) { goto l413; }  yyDo(G, yySet, -1, 0);  if (!yy_Sp(G)) { goto l413; }
  {  int yypos541= G->pos, yythunkpos541= G->thunkpos;
  {  int yypos545= G->pos, yythunkpos545= G->thunkpos;  if (!yymatchChar(G, '`')) goto l545;  goto l542;
  l545:;	  G->pos= yypos545; G->thunkpos= yythunkpos545;
  }  if (!yy_Nonspacechar(G)) { goto l542; }
  l543:;	
  {  int yypos544= G->pos, yythunkpos544= G->thunkpos;
  {  int yypos546= G->pos, yythunkpos546= G->thunkpos;  if (!yymatchChar(G, '`')) goto l546;  goto l544;
  l546:;	  G->pos= yypos546; G->thunkpos= yythunkpos546;
  }  if (!yy_Nonspacechar(G)) { goto l544; }  goto l543;
  l544:;	  G->pos= yypos544; G->thunkpos= yythunkpos544;
  }  goto l541;
  l542:;	  G->pos= yypos541; G->thunkpos= yythunkpos541;
  {  int yypos548= G->pos, yythunkpos548= G->thunkpos;  if (!yy_Ticks5(G)) { goto l548; }  goto l547;
  l548:;	  G->pos= yypos548; G->thunkpos= yythunkpos548;
  }  if (!yymatchChar(G, '`')) goto l547;
  l549:;	
  {  int yypos550= G->pos, yythunkpos550= G->thunkpos;  if (!yymatchChar(G, '`')) goto l550;  goto l549;
  l550:;	  G->pos= yypos550; G->thunkpos= yythunkpos550;
  }  goto l541;
  l547:;	  G->pos= yypos541; G->thunkpos= yythunkpos541;
  {  int yypos551= G->pos, yythunkpos551= G->thunkpos;  if (!yy_Sp(G)) { goto l551; }  if (!yy_Ticks5(G)) { goto l551; }  goto l413;
  l551:;	  G->pos= yypos551; G->thunkpos= yythunkpos551;
  }
  {  int yypos552= G->pos, yythunkpos552= G->thunkpos;  if (!yy_Spacechar(G)) { goto l553; }  goto l552;
  l553:;	  G->pos= yypos552; G->thunkpos= yythunkpos552;  if (!yy_Newline(G)) { goto l413; }
  {  int yypos554= G->pos, yythunkpos554= G->thunkpos;  if (!yy_BlankLine(G)) { goto l554; }  goto l413;
  l554:;	  G->pos= yypos554; G->thunkpos= yythunkpos554;
  }
  }
  l552:;	
  }
  l541:;	
  l539:;	
  {  int yypos540= G->pos, yythunkpos540= G->thunkpos;
  {  int yypos555= G->pos, yythunkpos555= G->thunkpos;
  {  int yypos559= G->pos, yythunkpos559= G->thunkpos;  if (!yymatchChar(G, '`')) goto l559;  goto l556;
  l559:;	  G->pos= yypos559; G->thunkpos= yythunkpos559;
  }  if (!yy_Nonspacechar(G)) { goto l556; }
  l557:;	
  {  int yypos558= G->pos, yythunkpos558= G->thunkpos;
  {  int yypos560= G->pos, yythunkpos560= G->thunkpos;  if (!yymatchChar(G, '`')) goto l560;  goto l558;
  l560:;	  G->pos= yypos560; G->thunkpos= yythunkpos560;
  }  if (!yy_Nonspacechar(G)) { goto l558; }  goto l557;
  l558:;	  G->pos= yypos558; G->thunkpos= yythunkpos558;
  }  goto l555;
  l556:;	  G->pos= yypos555; G->thunkpos= yythunkpos555;
  {  int yypos562= G->pos, yythunkpos562= G->thunkpos;  if (!yy_Ticks5(G)) { goto l562; }  goto l561;
  l562:;	  G->pos= yypos562; G->thunkpos= yythunkpos562;
  }  if (!yymatchChar(G, '`')) goto l561;
  l563:;	
  {  int yypos564= G->pos, yythunkpos564= G->thunkpos;  if (!yymatchChar(G, '`')) goto l564;  goto l563;
  l564:;	  G->pos= yypos564; G->thunkpos= yythunkpos564;
  }  goto l555;
  l561:;	  G->pos= yypos555; G->thunkpos= yythunkpos555;
  {  int yypos565= G->pos, yythunkpos565= G->thunkpos;  if (!yy_Sp(G)) { goto l565; }  if (!yy_Ticks5(G)) { goto l565; }  goto l540;
  l565:;	  G->pos= yypos565; G->thunkpos= yythunkpos565;
  }
  {  int yypos566= G->pos, yythunkpos566= G->thunkpos;  if (!yy_Spacechar(G)) { goto l567; }  goto l566;
  l567:;	  G->pos= yypos566; G->thunkpos= yythunkpos566;  if (!yy_Newline(G)) { goto l540; }
  {  int yypos568= G->pos, yythunkpos568= G->thunkpos;  if (!yy_BlankLine(G)) { goto l568; }  goto l540;
  l568:;	  G->pos= yypos568; G->thunkpos= yythunkpos568;
  }
  }
  l566:;	
  }
  l555:;	  goto l539;
  l540:;	  G->pos= yypos540; G->thunkpos= yythunkpos540;
  }  if (!yy_Sp(G)) { goto l413; }  if (!yy_Ticks5(G)) { goto l413; }
  }
  l414:;	  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l413;  yyDo(G, yy_1_Code, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "Code", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l413:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Code", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_InlineNote(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "InlineNote"));  yyText(G, G->begin, G->end);  if (!( EXT(pmh_EXT_NOTES) )) goto l569;  if (!yymatchString(G, "^[")) goto l569;
  {  int yypos572= G->pos, yythunkpos572= G->thunkpos;  if (!yymatchChar(G, ']')) goto l572;  goto l569;
  l572:;	  G->pos= yypos572; G->thunkpos= yythunkpos572;
  }  if (!yy_Inline(G)) { goto l569; }
  l570:;	
  {  int yypos571= G->pos, yythunkpos571= G->thunkpos;
  {  int yypos573= G->pos, yythunkpos573= G->thunkpos;  if (!yymatchChar(G, ']')) goto l573;  goto l571;
  l573:;	  G->pos= yypos573; G->thunkpos= yythunkpos573;
  }  if (!yy_Inline(G)) { goto l571; }  goto l570;
  l571:;	  G->pos= yypos571; G->thunkpos= yythunkpos571;
  }  if (!yymatchChar(G, ']')) goto l569;
  yyprintf((stderr, "  ok   %s @ %s\n", "InlineNote", G->buf+G->pos));
  return 1;
  l569:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "InlineNote", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_NoteReference(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "NoteReference"));  yyText(G, G->begin, G->end);  if (!( EXT(pmh_EXT_NOTES) )) goto l574;  if (!yy_RawNoteReference(G)) { goto l574; }
  yyprintf((stderr, "  ok   %s @ %s\n", "NoteReference", G->buf+G->pos));
  return 1;
  l574:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "NoteReference", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Link(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Link"));
  {  int yypos576= G->pos, yythunkpos576= G->thunkpos;  if (!yy_ExplicitLink(G)) { goto l577; }  goto l576;
  l577:;	  G->pos= yypos576; G->thunkpos= yythunkpos576;  if (!yy_ReferenceLink(G)) { goto l578; }  goto l576;
  l578:;	  G->pos= yypos576; G->thunkpos= yythunkpos576;  if (!yy_AutoLink(G)) { goto l575; }
  }
  l576:;	  yyDo(G, yy_1_Link, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "Link", G->buf+G->pos));
  return 1;
  l575:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Link", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Image(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Image"));  if (!yymatchChar(G, '!')) goto l579;
  {  int yypos580= G->pos, yythunkpos580= G->thunkpos;  if (!yy_ExplicitLink(G)) { goto l581; }  goto l580;
  l581:;	  G->pos= yypos580; G->thunkpos= yythunkpos580;  if (!yy_ReferenceLink(G)) { goto l579; }
  }
  l580:;	  yyDo(G, yy_1_Image, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "Image", G->buf+G->pos));
  return 1;
  l579:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Image", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Emph(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Emph"));
  {  int yypos583= G->pos, yythunkpos583= G->thunkpos;  if (!yy_EmphStar(G)) { goto l584; }  goto l583;
  l584:;	  G->pos= yypos583; G->thunkpos= yythunkpos583;  if (!yy_EmphUl(G)) { goto l582; }
  }
  l583:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "Emph", G->buf+G->pos));
  return 1;
  l582:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Emph", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Strong(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Strong"));
  {  int yypos586= G->pos, yythunkpos586= G->thunkpos;  if (!yy_StrongStar(G)) { goto l587; }  goto l586;
  l587:;	  G->pos= yypos586; G->thunkpos= yythunkpos586;  if (!yy_StrongUl(G)) { goto l585; }
  }
  l586:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "Strong", G->buf+G->pos));
  return 1;
  l585:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Strong", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Space(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Space"));  if (!yy_Spacechar(G)) { goto l588; }
  l589:;	
  {  int yypos590= G->pos, yythunkpos590= G->thunkpos;  if (!yy_Spacechar(G)) { goto l590; }  goto l589;
  l590:;	  G->pos= yypos590; G->thunkpos= yythunkpos590;
  }
  yyprintf((stderr, "  ok   %s @ %s\n", "Space", G->buf+G->pos));
  return 1;
  l588:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Space", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_UlOrStarLine(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "UlOrStarLine"));
  {  int yypos592= G->pos, yythunkpos592= G->thunkpos;  if (!yy_UlLine(G)) { goto l593; }  goto l592;
  l593:;	  G->pos= yypos592; G->thunkpos= yythunkpos592;  if (!yy_StarLine(G)) { goto l591; }
  }
  l592:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "UlOrStarLine", G->buf+G->pos));
  return 1;
  l591:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "UlOrStarLine", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Str(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Str"));  if (!yy_NormalChar(G)) { goto l594; }
  l595:;	
  {  int yypos596= G->pos, yythunkpos596= G->thunkpos;
  {  int yypos597= G->pos, yythunkpos597= G->thunkpos;  if (!yy_NormalChar(G)) { goto l598; }  goto l597;
  l598:;	  G->pos= yypos597; G->thunkpos= yythunkpos597;  if (!yymatchChar(G, '_')) goto l596;
  l599:;	
  {  int yypos600= G->pos, yythunkpos600= G->thunkpos;  if (!yymatchChar(G, '_')) goto l600;  goto l599;
  l600:;	  G->pos= yypos600; G->thunkpos= yythunkpos600;
  }
  {  int yypos601= G->pos, yythunkpos601= G->thunkpos;  if (!yy_Alphanumeric(G)) { goto l596; }  G->pos= yypos601; G->thunkpos= yythunkpos601;
  }
  }
  l597:;	  goto l595;
  l596:;	  G->pos= yypos596; G->thunkpos= yythunkpos596;
  }
  yyprintf((stderr, "  ok   %s @ %s\n", "Str", G->buf+G->pos));
  return 1;
  l594:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Str", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_InStyleTags(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "InStyleTags"));  if (!yy_StyleOpen(G)) { goto l602; }
  l603:;	
  {  int yypos604= G->pos, yythunkpos604= G->thunkpos;
  {  int yypos605= G->pos, yythunkpos605= G->thunkpos;  if (!yy_StyleClose(G)) { goto l605; }  goto l604;
  l605:;	  G->pos= yypos605; G->thunkpos= yythunkpos605;
  }  if (!yymatchDot(G)) goto l604;  goto l603;
  l604:;	  G->pos= yypos604; G->thunkpos= yythunkpos604;
  }  if (!yy_StyleClose(G)) { goto l602; }
  yyprintf((stderr, "  ok   %s @ %s\n", "InStyleTags", G->buf+G->pos));
  return 1;
  l602:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "InStyleTags", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_StyleClose(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "StyleClose"));  if (!yymatchChar(G, '<')) goto l606;  if (!yy_Spnl(G)) { goto l606; }  if (!yymatchChar(G, '/')) goto l606;
  {  int yypos607= G->pos, yythunkpos607= G->thunkpos;  if (!yymatchString(G, "style")) goto l608;  goto l607;
  l608:;	  G->pos= yypos607; G->thunkpos= yythunkpos607;  if (!yymatchString(G, "STYLE")) goto l606;
  }
  l607:;	  if (!yy_Spnl(G)) { goto l606; }  if (!yymatchChar(G, '>')) goto l606;
  yyprintf((stderr, "  ok   %s @ %s\n", "StyleClose", G->buf+G->pos));
  return 1;
  l606:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "StyleClose", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_StyleOpen(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "StyleOpen"));  if (!yymatchChar(G, '<')) goto l609;  if (!yy_Spnl(G)) { goto l609; }
  {  int yypos610= G->pos, yythunkpos610= G->thunkpos;  if (!yymatchString(G, "style")) goto l611;  goto l610;
  l611:;	  G->pos= yypos610; G->thunkpos= yythunkpos610;  if (!yymatchString(G, "STYLE")) goto l609;
  }
  l610:;	  if (!yy_Spnl(G)) { goto l609; }
  l612:;	
  {  int yypos613= G->pos, yythunkpos613= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l613; }  goto l612;
  l613:;	  G->pos= yypos613; G->thunkpos= yythunkpos613;
  }  if (!yymatchChar(G, '>')) goto l609;
  yyprintf((stderr, "  ok   %s @ %s\n", "StyleOpen", G->buf+G->pos));
  return 1;
  l609:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "StyleOpen", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockType(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockType"));
  {  int yypos615= G->pos, yythunkpos615= G->thunkpos;  if (!yymatchString(G, "address")) goto l616;  goto l615;
  l616:;	  G->pos= yypos615; G->thunkpos= yythunkpos615;  if (!yymatchString(G, "blockquote")) goto l617;  goto l615;
  l617:;	  G->pos= yypos615; G->thunkpos= yythunkpos615;  if (!yymatchString(G, "center")) goto l618;  goto l615;
  l618:;	  G->pos= yypos615; G->thunkpos= yythunkpos615;  if (!yymatchString(G, "dir")) goto l619;  goto l615;
  l619:;	  G->pos= yypos615; G->thunkpos= yythunkpos615;  if (!yymatchString(G, "div")) goto l620;  goto l615;
  l620:;	  G->pos= yypos615; G->thunkpos= yythunkpos615;  if (!yymatchString(G, "dl")) goto l621;  goto l615;
  l621:;	  G->pos= yypos615; G->thunkpos= yythunkpos615;  if (!yymatchString(G, "fieldset")) goto l622;  goto l615;
  l622:;	  G->pos= yypos615; G->thunkpos= yythunkpos615;  if (!yymatchString(G, "form")) goto l623;  goto l615;
  l623:;	  G->pos= yypos615; G->thunkpos= yythunkpos615;  if (!yymatchString(G, "h1")) goto l624;  goto l615;
  l624:;	  G->pos= yypos615; G->thunkpos= yythunkpos615;  if (!yymatchString(G, "h2")) goto l625;  goto l615;
  l625:;	  G->pos= yypos615; G->thunkpos= yythunkpos615;  if (!yymatchString(G, "h3")) goto l626;  goto l615;
  l626:;	  G->pos= yypos615; G->thunkpos= yythunkpos615;  if (!yymatchString(G, "h4")) goto l627;  goto l615;
  l627:;	  G->pos= yypos615; G->thunkpos= yythunkpos615;  if (!yymatchString(G, "h5")) goto l628;  goto l615;
  l628:;	  G->pos= yypos615; G->thunkpos= yythunkpos615;  if (!yymatchString(G, "h6")) goto l629;  goto l615;
  l629:;	  G->pos= yypos615; G->thunkpos= yythunkpos615;  if (!yymatchString(G, "hr")) goto l630;  goto l615;
  l630:;	  G->pos= yypos615; G->thunkpos= yythunkpos615;  if (!yymatchString(G, "isindex")) goto l631;  goto l615;
  l631:;	  G->pos= yypos615; G->thunkpos= yythunkpos615;  if (!yymatchString(G, "menu")) goto l632;  goto l615;
  l632:;	  G->pos= yypos615; G->thunkpos= yythunkpos615;  if (!yymatchString(G, "noframes")) goto l633;  goto l615;
  l633:;	  G->pos= yypos615; G->thunkpos= yythunkpos615;  if (!yymatchString(G, "noscript")) goto l634;  goto l615;
  l634:;	  G->pos= yypos615; G->thunkpos= yythunkpos615;  if (!yymatchString(G, "ol")) goto l635;  goto l615;
  l635:;	  G->pos= yypos615; G->thunkpos= yythunkpos615;  if (!yymatchChar(G, 'p')) goto l636;  goto l615;
  l636:;	  G->pos= yypos615; G->thunkpos= yythunkpos615;  if (!yymatchString(G, "pre")) goto l637;  goto l615;
  l637:;	  G->pos= yypos615; G->thunkpos= yythunkpos615;  if (!yymatchString(G, "table")) goto l638;  goto l615;
  l638:;	  G->pos= yypos615; G->thunkpos= yythunkpos615;  if (!yymatchString(G, "ul")) goto l639;  goto l615;
  l639:;	  G->pos= yypos615; G->thunkpos= yythunkpos615;  if (!yymatchString(G, "dd")) goto l640;  goto l615;
  l640:;	  G->pos= yypos615; G->thunkpos= yythunkpos615;  if (!yymatchString(G, "dt")) goto l641;  goto l615;
  l641:;	  G->pos= yypos615; G->thunkpos= yythunkpos615;  if (!yymatchString(G, "frameset")) goto l642;  goto l615;
  l642:;	  G->pos= yypos615; G->thunkpos= yythunkpos615;  if (!yymatchString(G, "li")) goto l643;  goto l615;
  l643:;	  G->pos= yypos615; G->thunkpos= yythunkpos615;  if (!yymatchString(G, "tbody")) goto l644;  goto l615;
  l644:;	  G->pos= yypos615; G->thunkpos= yythunkpos615;  if (!yymatchString(G, "td")) goto l645;  goto l615;
  l645:;	  G->pos= yypos615; G->thunkpos= yythunkpos615;  if (!yymatchString(G, "tfoot")) goto l646;  goto l615;
  l646:;	  G->pos= yypos615; G->thunkpos= yythunkpos615;  if (!yymatchString(G, "th")) goto l647;  goto l615;
  l647:;	  G->pos= yypos615; G->thunkpos= yythunkpos615;  if (!yymatchString(G, "thead")) goto l648;  goto l615;
  l648:;	  G->pos= yypos615; G->thunkpos= yythunkpos615;  if (!yymatchString(G, "tr")) goto l649;  goto l615;
  l649:;	  G->pos= yypos615; G->thunkpos= yythunkpos615;  if (!yymatchString(G, "script")) goto l650;  goto l615;
  l650:;	  G->pos= yypos615; G->thunkpos= yythunkpos615;  if (!yymatchString(G, "ADDRESS")) goto l651;  goto l615;
  l651:;	  G->pos= yypos615; G->thunkpos= yythunkpos615;  if (!yymatchString(G, "BLOCKQUOTE")) goto l652;  goto l615;
  l652:;	  G->pos= yypos615; G->thunkpos= yythunkpos615;  if (!yymatchString(G, "CENTER")) goto l653;  goto l615;
  l653:;	  G->pos= yypos615; G->thunkpos= yythunkpos615;  if (!yymatchString(G, "DIR")) goto l654;  goto l615;
  l654:;	  G->pos= yypos615; G->thunkpos= yythunkpos615;  if (!yymatchString(G, "DIV")) goto l655;  goto l615;
  l655:;	  G->pos= yypos615; G->thunkpos= yythunkpos615;  if (!yymatchString(G, "DL")) goto l656;  goto l615;
  l656:;	  G->pos= yypos615; G->thunkpos= yythunkpos615;  if (!yymatchString(G, "FIELDSET")) goto l657;  goto l615;
  l657:;	  G->pos= yypos615; G->thunkpos= yythunkpos615;  if (!yymatchString(G, "FORM")) goto l658;  goto l615;
  l658:;	  G->pos= yypos615; G->thunkpos= yythunkpos615;  if (!yymatchString(G, "H1")) goto l659;  goto l615;
  l659:;	  G->pos= yypos615; G->thunkpos= yythunkpos615;  if (!yymatchString(G, "H2")) goto l660;  goto l615;
  l660:;	  G->pos= yypos615; G->thunkpos= yythunkpos615;  if (!yymatchString(G, "H3")) goto l661;  goto l615;
  l661:;	  G->pos= yypos615; G->thunkpos= yythunkpos615;  if (!yymatchString(G, "H4")) goto l662;  goto l615;
  l662:;	  G->pos= yypos615; G->thunkpos= yythunkpos615;  if (!yymatchString(G, "H5")) goto l663;  goto l615;
  l663:;	  G->pos= yypos615; G->thunkpos= yythunkpos615;  if (!yymatchString(G, "H6")) goto l664;  goto l615;
  l664:;	  G->pos= yypos615; G->thunkpos= yythunkpos615;  if (!yymatchString(G, "HR")) goto l665;  goto l615;
  l665:;	  G->pos= yypos615; G->thunkpos= yythunkpos615;  if (!yymatchString(G, "ISINDEX")) goto l666;  goto l615;
  l666:;	  G->pos= yypos615; G->thunkpos= yythunkpos615;  if (!yymatchString(G, "MENU")) goto l667;  goto l615;
  l667:;	  G->pos= yypos615; G->thunkpos= yythunkpos615;  if (!yymatchString(G, "NOFRAMES")) goto l668;  goto l615;
  l668:;	  G->pos= yypos615; G->thunkpos= yythunkpos615;  if (!yymatchString(G, "NOSCRIPT")) goto l669;  goto l615;
  l669:;	  G->pos= yypos615; G->thunkpos= yythunkpos615;  if (!yymatchString(G, "OL")) goto l670;  goto l615;
  l670:;	  G->pos= yypos615; G->thunkpos= yythunkpos615;  if (!yymatchChar(G, 'P')) goto l671;  goto l615;
  l671:;	  G->pos= yypos615; G->thunkpos= yythunkpos615;  if (!yymatchString(G, "PRE")) goto l672;  goto l615;
  l672:;	  G->pos= yypos615; G->thunkpos= yythunkpos615;  if (!yymatchString(G, "TABLE")) goto l673;  goto l615;
  l673:;	  G->pos= yypos615; G->thunkpos= yythunkpos615;  if (!yymatchString(G, "UL")) goto l674;  goto l615;
  l674:;	  G->pos= yypos615; G->thunkpos= yythunkpos615;  if (!yymatchString(G, "DD")) goto l675;  goto l615;
  l675:;	  G->pos= yypos615; G->thunkpos= yythunkpos615;  if (!yymatchString(G, "DT")) goto l676;  goto l615;
  l676:;	  G->pos= yypos615; G->thunkpos= yythunkpos615;  if (!yymatchString(G, "FRAMESET")) goto l677;  goto l615;
  l677:;	  G->pos= yypos615; G->thunkpos= yythunkpos615;  if (!yymatchString(G, "LI")) goto l678;  goto l615;
  l678:;	  G->pos= yypos615; G->thunkpos= yythunkpos615;  if (!yymatchString(G, "TBODY")) goto l679;  goto l615;
  l679:;	  G->pos= yypos615; G->thunkpos= yythunkpos615;  if (!yymatchString(G, "TD")) goto l680;  goto l615;
  l680:;	  G->pos= yypos615; G->thunkpos= yythunkpos615;  if (!yymatchString(G, "TFOOT")) goto l681;  goto l615;
  l681:;	  G->pos= yypos615; G->thunkpos= yythunkpos615;  if (!yymatchString(G, "TH")) goto l682;  goto l615;
  l682:;	  G->pos= yypos615; G->thunkpos= yythunkpos615;  if (!yymatchString(G, "THEAD")) goto l683;  goto l615;
  l683:;	  G->pos= yypos615; G->thunkpos= yythunkpos615;  if (!yymatchString(G, "TR")) goto l684;  goto l615;
  l684:;	  G->pos= yypos615; G->thunkpos= yythunkpos615;  if (!yymatchString(G, "SCRIPT")) goto l614;
  }
  l615:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockType", G->buf+G->pos));
  return 1;
  l614:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockType", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockSelfClosing(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockSelfClosing"));  if (!yymatchChar(G, '<')) goto l685;  if (!yy_Spnl(G)) { goto l685; }  if (!yy_HtmlBlockType(G)) { goto l685; }  if (!yy_Spnl(G)) { goto l685; }
  l686:;	
  {  int yypos687= G->pos, yythunkpos687= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l687; }  goto l686;
  l687:;	  G->pos= yypos687; G->thunkpos= yythunkpos687;
  }  if (!yymatchChar(G, '/')) goto l685;  if (!yy_Spnl(G)) { goto l685; }  if (!yymatchChar(G, '>')) goto l685;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockSelfClosing", G->buf+G->pos));
  return 1;
  l685:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockSelfClosing", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlComment(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "HtmlComment"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l688;  if (!yy_LocMarker(G)) { goto l688; }  yyDo(G, yySet, -1, 0);  if (!yymatchString(G, "<!--")) goto l688;
  l689:;	
  {  int yypos690= G->pos, yythunkpos690= G->thunkpos;
  {  int yypos691= G->pos, yythunkpos691= G->thunkpos;  if (!yymatchString(G, "-->")) goto l691;  goto l690;
  l691:;	  G->pos= yypos691; G->thunkpos= yythunkpos691;
  }  if (!yymatchDot(G)) goto l690;  goto l689;
  l690:;	  G->pos= yypos690; G->thunkpos= yythunkpos690;
  }  if (!yymatchString(G, "-->")) goto l688;  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l688;  yyDo(G, yy_1_HtmlComment, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlComment", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l688:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlComment", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockInTags(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockInTags"));
  {  int yypos693= G->pos, yythunkpos693= G->thunkpos;  if (!yy_HtmlBlockAddress(G)) { goto l694; }  goto l693;
  l694:;	  G->pos= yypos693; G->thunkpos= yythunkpos693;  if (!yy_HtmlBlockBlockquote(G)) { goto l695; }  goto l693;
  l695:;	  G->pos= yypos693; G->thunkpos= yythunkpos693;  if (!yy_HtmlBlockCenter(G)) { goto l696; }  goto l693;
  l696:;	  G->pos= yypos693; G->thunkpos= yythunkpos693;  if (!yy_HtmlBlockDir(G)) { goto l697; }  goto l693;
  l697:;	  G->pos= yypos693; G->thunkpos= yythunkpos693;  if (!yy_HtmlBlockDiv(G)) { goto l698; }  goto l693;
  l698:;	  G->pos= yypos693; G->thunkpos= yythunkpos693;  if (!yy_HtmlBlockDl(G)) { goto l699; }  goto l693;
  l699:;	  G->pos= yypos693; G->thunkpos= yythunkpos693;  if (!yy_HtmlBlockFieldset(G)) { goto l700; }  goto l693;
  l700:;	  G->pos= yypos693; G->thunkpos= yythunkpos693;  if (!yy_HtmlBlockForm(G)) { goto l701; }  goto l693;
  l701:;	  G->pos= yypos693; G->thunkpos= yythunkpos693;  if (!yy_HtmlBlockH1(G)) { goto l702; }  goto l693;
  l702:;	  G->pos= yypos693; G->thunkpos= yythunkpos693;  if (!yy_HtmlBlockH2(G)) { goto l703; }  goto l693;
  l703:;	  G->pos= yypos693; G->thunkpos= yythunkpos693;  if (!yy_HtmlBlockH3(G)) { goto l704; }  goto l693;
  l704:;	  G->pos= yypos693; G->thunkpos= yythunkpos693;  if (!yy_HtmlBlockH4(G)) { goto l705; }  goto l693;
  l705:;	  G->pos= yypos693; G->thunkpos= yythunkpos693;  if (!yy_HtmlBlockH5(G)) { goto l706; }  goto l693;
  l706:;	  G->pos= yypos693; G->thunkpos= yythunkpos693;  if (!yy_HtmlBlockH6(G)) { goto l707; }  goto l693;
  l707:;	  G->pos= yypos693; G->thunkpos= yythunkpos693;  if (!yy_HtmlBlockMenu(G)) { goto l708; }  goto l693;
  l708:;	  G->pos= yypos693; G->thunkpos= yythunkpos693;  if (!yy_HtmlBlockNoframes(G)) { goto l709; }  goto l693;
  l709:;	  G->pos= yypos693; G->thunkpos= yythunkpos693;  if (!yy_HtmlBlockNoscript(G)) { goto l710; }  goto l693;
  l710:;	  G->pos= yypos693; G->thunkpos= yythunkpos693;  if (!yy_HtmlBlockOl(G)) { goto l711; }  goto l693;
  l711:;	  G->pos= yypos693; G->thunkpos= yythunkpos693;  if (!yy_HtmlBlockP(G)) { goto l712; }  goto l693;
  l712:;	  G->pos= yypos693; G->thunkpos= yythunkpos693;  if (!yy_HtmlBlockPre(G)) { goto l713; }  goto l693;
  l713:;	  G->pos= yypos693; G->thunkpos= yythunkpos693;  if (!yy_HtmlBlockTable(G)) { goto l714; }  goto l693;
  l714:;	  G->pos= yypos693; G->thunkpos= yythunkpos693;  if (!yy_HtmlBlockUl(G)) { goto l715; }  goto l693;
  l715:;	  G->pos= yypos693; G->thunkpos= yythunkpos693;  if (!yy_HtmlBlockDd(G)) { goto l716; }  goto l693;
  l716:;	  G->pos= yypos693; G->thunkpos= yythunkpos693;  if (!yy_HtmlBlockDt(G)) { goto l717; }  goto l693;
  l717:;	  G->pos= yypos693; G->thunkpos= yythunkpos693;  if (!yy_HtmlBlockFrameset(G)) { goto l718; }  goto l693;
  l718:;	  G->pos= yypos693; G->thunkpos= yythunkpos693;  if (!yy_HtmlBlockLi(G)) { goto l719; }  goto l693;
  l719:;	  G->pos= yypos693; G->thunkpos= yythunkpos693;  if (!yy_HtmlBlockTbody(G)) { goto l720; }  goto l693;
  l720:;	  G->pos= yypos693; G->thunkpos= yythunkpos693;  if (!yy_HtmlBlockTd(G)) { goto l721; }  goto l693;
  l721:;	  G->pos= yypos693; G->thunkpos= yythunkpos693;  if (!yy_HtmlBlockTfoot(G)) { goto l722; }  goto l693;
  l722:;	  G->pos= yypos693; G->thunkpos= yythunkpos693;  if (!yy_HtmlBlockTh(G)) { goto l723; }  goto l693;
  l723:;	  G->pos= yypos693; G->thunkpos= yythunkpos693;  if (!yy_HtmlBlockThead(G)) { goto l724; }  goto l693;
  l724:;	  G->pos= yypos693; G->thunkpos= yythunkpos693;  if (!yy_HtmlBlockTr(G)) { goto l725; }  goto l693;
  l725:;	  G->pos= yypos693; G->thunkpos= yythunkpos693;  if (!yy_HtmlBlockScript(G)) { goto l692; }
  }
  l693:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockInTags", G->buf+G->pos));
  return 1;
  l692:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockInTags", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockScript(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockScript"));  if (!yy_HtmlBlockOpenScript(G)) { goto l726; }
  l727:;	
  {  int yypos728= G->pos, yythunkpos728= G->thunkpos;
  {  int yypos729= G->pos, yythunkpos729= G->thunkpos;  if (!yy_HtmlBlockCloseScript(G)) { goto l729; }  goto l728;
  l729:;	  G->pos= yypos729; G->thunkpos= yythunkpos729;
  }  if (!yymatchDot(G)) goto l728;  goto l727;
  l728:;	  G->pos= yypos728; G->thunkpos= yythunkpos728;
  }  if (!yy_HtmlBlockCloseScript(G)) { goto l726; }
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockScript", G->buf+G->pos));
  return 1;
  l726:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockScript", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseScript(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseScript"));  if (!yymatchChar(G, '<')) goto l730;  if (!yy_Spnl(G)) { goto l730; }  if (!yymatchChar(G, '/')) goto l730;
  {  int yypos731= G->pos, yythunkpos731= G->thunkpos;  if (!yymatchString(G, "script")) goto l732;  goto l731;
  l732:;	  G->pos= yypos731; G->thunkpos= yythunkpos731;  if (!yymatchString(G, "SCRIPT")) goto l730;
  }
  l731:;	  if (!yy_Spnl(G)) { goto l730; }  if (!yymatchChar(G, '>')) goto l730;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseScript", G->buf+G->pos));
  return 1;
  l730:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseScript", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenScript(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenScript"));  if (!yymatchChar(G, '<')) goto l733;  if (!yy_Spnl(G)) { goto l733; }
  {  int yypos734= G->pos, yythunkpos734= G->thunkpos;  if (!yymatchString(G, "script")) goto l735;  goto l734;
  l735:;	  G->pos= yypos734; G->thunkpos= yythunkpos734;  if (!yymatchString(G, "SCRIPT")) goto l733;
  }
  l734:;	  if (!yy_Spnl(G)) { goto l733; }
  l736:;	
  {  int yypos737= G->pos, yythunkpos737= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l737; }  goto l736;
  l737:;	  G->pos= yypos737; G->thunkpos= yythunkpos737;
  }  if (!yymatchChar(G, '>')) goto l733;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenScript", G->buf+G->pos));
  return 1;
  l733:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenScript", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockTr(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockTr"));  if (!yy_HtmlBlockOpenTr(G)) { goto l738; }
  l739:;	
  {  int yypos740= G->pos, yythunkpos740= G->thunkpos;
  {  int yypos741= G->pos, yythunkpos741= G->thunkpos;  if (!yy_HtmlBlockTr(G)) { goto l742; }  goto l741;
  l742:;	  G->pos= yypos741; G->thunkpos= yythunkpos741;
  {  int yypos743= G->pos, yythunkpos743= G->thunkpos;  if (!yy_HtmlBlockCloseTr(G)) { goto l743; }  goto l740;
  l743:;	  G->pos= yypos743; G->thunkpos= yythunkpos743;
  }  if (!yymatchDot(G)) goto l740;
  }
  l741:;	  goto l739;
  l740:;	  G->pos= yypos740; G->thunkpos= yythunkpos740;
  }  if (!yy_HtmlBlockCloseTr(G)) { goto l738; }
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockTr", G->buf+G->pos));
  return 1;
  l738:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockTr", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseTr(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseTr"));  if (!yymatchChar(G, '<')) goto l744;  if (!yy_Spnl(G)) { goto l744; }  if (!yymatchChar(G, '/')) goto l744;
  {  int yypos745= G->pos, yythunkpos745= G->thunkpos;  if (!yymatchString(G, "tr")) goto l746;  goto l745;
  l746:;	  G->pos= yypos745; G->thunkpos= yythunkpos745;  if (!yymatchString(G, "TR")) goto l744;
  }
  l745:;	  if (!yy_Spnl(G)) { goto l744; }  if (!yymatchChar(G, '>')) goto l744;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseTr", G->buf+G->pos));
  return 1;
  l744:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseTr", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenTr(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenTr"));  if (!yymatchChar(G, '<')) goto l747;  if (!yy_Spnl(G)) { goto l747; }
  {  int yypos748= G->pos, yythunkpos748= G->thunkpos;  if (!yymatchString(G, "tr")) goto l749;  goto l748;
  l749:;	  G->pos= yypos748; G->thunkpos= yythunkpos748;  if (!yymatchString(G, "TR")) goto l747;
  }
  l748:;	  if (!yy_Spnl(G)) { goto l747; }
  l750:;	
  {  int yypos751= G->pos, yythunkpos751= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l751; }  goto l750;
  l751:;	  G->pos= yypos751; G->thunkpos= yythunkpos751;
  }  if (!yymatchChar(G, '>')) goto l747;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenTr", G->buf+G->pos));
  return 1;
  l747:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenTr", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockThead(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockThead"));  if (!yy_HtmlBlockOpenThead(G)) { goto l752; }
  l753:;	
  {  int yypos754= G->pos, yythunkpos754= G->thunkpos;
  {  int yypos755= G->pos, yythunkpos755= G->thunkpos;  if (!yy_HtmlBlockThead(G)) { goto l756; }  goto l755;
  l756:;	  G->pos= yypos755; G->thunkpos= yythunkpos755;
  {  int yypos757= G->pos, yythunkpos757= G->thunkpos;  if (!yy_HtmlBlockCloseThead(G)) { goto l757; }  goto l754;
  l757:;	  G->pos= yypos757; G->thunkpos= yythunkpos757;
  }  if (!yymatchDot(G)) goto l754;
  }
  l755:;	  goto l753;
  l754:;	  G->pos= yypos754; G->thunkpos= yythunkpos754;
  }  if (!yy_HtmlBlockCloseThead(G)) { goto l752; }
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockThead", G->buf+G->pos));
  return 1;
  l752:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockThead", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseThead(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseThead"));  if (!yymatchChar(G, '<')) goto l758;  if (!yy_Spnl(G)) { goto l758; }  if (!yymatchChar(G, '/')) goto l758;
  {  int yypos759= G->pos, yythunkpos759= G->thunkpos;  if (!yymatchString(G, "thead")) goto l760;  goto l759;
  l760:;	  G->pos= yypos759; G->thunkpos= yythunkpos759;  if (!yymatchString(G, "THEAD")) goto l758;
  }
  l759:;	  if (!yy_Spnl(G)) { goto l758; }  if (!yymatchChar(G, '>')) goto l758;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseThead", G->buf+G->pos));
  return 1;
  l758:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseThead", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenThead(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenThead"));  if (!yymatchChar(G, '<')) goto l761;  if (!yy_Spnl(G)) { goto l761; }
  {  int yypos762= G->pos, yythunkpos762= G->thunkpos;  if (!yymatchString(G, "thead")) goto l763;  goto l762;
  l763:;	  G->pos= yypos762; G->thunkpos= yythunkpos762;  if (!yymatchString(G, "THEAD")) goto l761;
  }
  l762:;	  if (!yy_Spnl(G)) { goto l761; }
  l764:;	
  {  int yypos765= G->pos, yythunkpos765= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l765; }  goto l764;
  l765:;	  G->pos= yypos765; G->thunkpos= yythunkpos765;
  }  if (!yymatchChar(G, '>')) goto l761;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenThead", G->buf+G->pos));
  return 1;
  l761:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenThead", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockTh(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockTh"));  if (!yy_HtmlBlockOpenTh(G)) { goto l766; }
  l767:;	
  {  int yypos768= G->pos, yythunkpos768= G->thunkpos;
  {  int yypos769= G->pos, yythunkpos769= G->thunkpos;  if (!yy_HtmlBlockTh(G)) { goto l770; }  goto l769;
  l770:;	  G->pos= yypos769; G->thunkpos= yythunkpos769;
  {  int yypos771= G->pos, yythunkpos771= G->thunkpos;  if (!yy_HtmlBlockCloseTh(G)) { goto l771; }  goto l768;
  l771:;	  G->pos= yypos771; G->thunkpos= yythunkpos771;
  }  if (!yymatchDot(G)) goto l768;
  }
  l769:;	  goto l767;
  l768:;	  G->pos= yypos768; G->thunkpos= yythunkpos768;
  }  if (!yy_HtmlBlockCloseTh(G)) { goto l766; }
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockTh", G->buf+G->pos));
  return 1;
  l766:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockTh", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseTh(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseTh"));  if (!yymatchChar(G, '<')) goto l772;  if (!yy_Spnl(G)) { goto l772; }  if (!yymatchChar(G, '/')) goto l772;
  {  int yypos773= G->pos, yythunkpos773= G->thunkpos;  if (!yymatchString(G, "th")) goto l774;  goto l773;
  l774:;	  G->pos= yypos773; G->thunkpos= yythunkpos773;  if (!yymatchString(G, "TH")) goto l772;
  }
  l773:;	  if (!yy_Spnl(G)) { goto l772; }  if (!yymatchChar(G, '>')) goto l772;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseTh", G->buf+G->pos));
  return 1;
  l772:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseTh", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenTh(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenTh"));  if (!yymatchChar(G, '<')) goto l775;  if (!yy_Spnl(G)) { goto l775; }
  {  int yypos776= G->pos, yythunkpos776= G->thunkpos;  if (!yymatchString(G, "th")) goto l777;  goto l776;
  l777:;	  G->pos= yypos776; G->thunkpos= yythunkpos776;  if (!yymatchString(G, "TH")) goto l775;
  }
  l776:;	  if (!yy_Spnl(G)) { goto l775; }
  l778:;	
  {  int yypos779= G->pos, yythunkpos779= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l779; }  goto l778;
  l779:;	  G->pos= yypos779; G->thunkpos= yythunkpos779;
  }  if (!yymatchChar(G, '>')) goto l775;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenTh", G->buf+G->pos));
  return 1;
  l775:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenTh", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockTfoot(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockTfoot"));  if (!yy_HtmlBlockOpenTfoot(G)) { goto l780; }
  l781:;	
  {  int yypos782= G->pos, yythunkpos782= G->thunkpos;
  {  int yypos783= G->pos, yythunkpos783= G->thunkpos;  if (!yy_HtmlBlockTfoot(G)) { goto l784; }  goto l783;
  l784:;	  G->pos= yypos783; G->thunkpos= yythunkpos783;
  {  int yypos785= G->pos, yythunkpos785= G->thunkpos;  if (!yy_HtmlBlockCloseTfoot(G)) { goto l785; }  goto l782;
  l785:;	  G->pos= yypos785; G->thunkpos= yythunkpos785;
  }  if (!yymatchDot(G)) goto l782;
  }
  l783:;	  goto l781;
  l782:;	  G->pos= yypos782; G->thunkpos= yythunkpos782;
  }  if (!yy_HtmlBlockCloseTfoot(G)) { goto l780; }
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockTfoot", G->buf+G->pos));
  return 1;
  l780:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockTfoot", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseTfoot(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseTfoot"));  if (!yymatchChar(G, '<')) goto l786;  if (!yy_Spnl(G)) { goto l786; }  if (!yymatchChar(G, '/')) goto l786;
  {  int yypos787= G->pos, yythunkpos787= G->thunkpos;  if (!yymatchString(G, "tfoot")) goto l788;  goto l787;
  l788:;	  G->pos= yypos787; G->thunkpos= yythunkpos787;  if (!yymatchString(G, "TFOOT")) goto l786;
  }
  l787:;	  if (!yy_Spnl(G)) { goto l786; }  if (!yymatchChar(G, '>')) goto l786;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseTfoot", G->buf+G->pos));
  return 1;
  l786:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseTfoot", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenTfoot(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenTfoot"));  if (!yymatchChar(G, '<')) goto l789;  if (!yy_Spnl(G)) { goto l789; }
  {  int yypos790= G->pos, yythunkpos790= G->thunkpos;  if (!yymatchString(G, "tfoot")) goto l791;  goto l790;
  l791:;	  G->pos= yypos790; G->thunkpos= yythunkpos790;  if (!yymatchString(G, "TFOOT")) goto l789;
  }
  l790:;	  if (!yy_Spnl(G)) { goto l789; }
  l792:;	
  {  int yypos793= G->pos, yythunkpos793= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l793; }  goto l792;
  l793:;	  G->pos= yypos793; G->thunkpos= yythunkpos793;
  }  if (!yymatchChar(G, '>')) goto l789;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenTfoot", G->buf+G->pos));
  return 1;
  l789:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenTfoot", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockTd(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockTd"));  if (!yy_HtmlBlockOpenTd(G)) { goto l794; }
  l795:;	
  {  int yypos796= G->pos, yythunkpos796= G->thunkpos;
  {  int yypos797= G->pos, yythunkpos797= G->thunkpos;  if (!yy_HtmlBlockTd(G)) { goto l798; }  goto l797;
  l798:;	  G->pos= yypos797; G->thunkpos= yythunkpos797;
  {  int yypos799= G->pos, yythunkpos799= G->thunkpos;  if (!yy_HtmlBlockCloseTd(G)) { goto l799; }  goto l796;
  l799:;	  G->pos= yypos799; G->thunkpos= yythunkpos799;
  }  if (!yymatchDot(G)) goto l796;
  }
  l797:;	  goto l795;
  l796:;	  G->pos= yypos796; G->thunkpos= yythunkpos796;
  }  if (!yy_HtmlBlockCloseTd(G)) { goto l794; }
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockTd", G->buf+G->pos));
  return 1;
  l794:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockTd", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseTd(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseTd"));  if (!yymatchChar(G, '<')) goto l800;  if (!yy_Spnl(G)) { goto l800; }  if (!yymatchChar(G, '/')) goto l800;
  {  int yypos801= G->pos, yythunkpos801= G->thunkpos;  if (!yymatchString(G, "td")) goto l802;  goto l801;
  l802:;	  G->pos= yypos801; G->thunkpos= yythunkpos801;  if (!yymatchString(G, "TD")) goto l800;
  }
  l801:;	  if (!yy_Spnl(G)) { goto l800; }  if (!yymatchChar(G, '>')) goto l800;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseTd", G->buf+G->pos));
  return 1;
  l800:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseTd", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenTd(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenTd"));  if (!yymatchChar(G, '<')) goto l803;  if (!yy_Spnl(G)) { goto l803; }
  {  int yypos804= G->pos, yythunkpos804= G->thunkpos;  if (!yymatchString(G, "td")) goto l805;  goto l804;
  l805:;	  G->pos= yypos804; G->thunkpos= yythunkpos804;  if (!yymatchString(G, "TD")) goto l803;
  }
  l804:;	  if (!yy_Spnl(G)) { goto l803; }
  l806:;	
  {  int yypos807= G->pos, yythunkpos807= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l807; }  goto l806;
  l807:;	  G->pos= yypos807; G->thunkpos= yythunkpos807;
  }  if (!yymatchChar(G, '>')) goto l803;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenTd", G->buf+G->pos));
  return 1;
  l803:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenTd", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockTbody(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockTbody"));  if (!yy_HtmlBlockOpenTbody(G)) { goto l808; }
  l809:;	
  {  int yypos810= G->pos, yythunkpos810= G->thunkpos;
  {  int yypos811= G->pos, yythunkpos811= G->thunkpos;  if (!yy_HtmlBlockTbody(G)) { goto l812; }  goto l811;
  l812:;	  G->pos= yypos811; G->thunkpos= yythunkpos811;
  {  int yypos813= G->pos, yythunkpos813= G->thunkpos;  if (!yy_HtmlBlockCloseTbody(G)) { goto l813; }  goto l810;
  l813:;	  G->pos= yypos813; G->thunkpos= yythunkpos813;
  }  if (!yymatchDot(G)) goto l810;
  }
  l811:;	  goto l809;
  l810:;	  G->pos= yypos810; G->thunkpos= yythunkpos810;
  }  if (!yy_HtmlBlockCloseTbody(G)) { goto l808; }
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockTbody", G->buf+G->pos));
  return 1;
  l808:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockTbody", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseTbody(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseTbody"));  if (!yymatchChar(G, '<')) goto l814;  if (!yy_Spnl(G)) { goto l814; }  if (!yymatchChar(G, '/')) goto l814;
  {  int yypos815= G->pos, yythunkpos815= G->thunkpos;  if (!yymatchString(G, "tbody")) goto l816;  goto l815;
  l816:;	  G->pos= yypos815; G->thunkpos= yythunkpos815;  if (!yymatchString(G, "TBODY")) goto l814;
  }
  l815:;	  if (!yy_Spnl(G)) { goto l814; }  if (!yymatchChar(G, '>')) goto l814;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseTbody", G->buf+G->pos));
  return 1;
  l814:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseTbody", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenTbody(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenTbody"));  if (!yymatchChar(G, '<')) goto l817;  if (!yy_Spnl(G)) { goto l817; }
  {  int yypos818= G->pos, yythunkpos818= G->thunkpos;  if (!yymatchString(G, "tbody")) goto l819;  goto l818;
  l819:;	  G->pos= yypos818; G->thunkpos= yythunkpos818;  if (!yymatchString(G, "TBODY")) goto l817;
  }
  l818:;	  if (!yy_Spnl(G)) { goto l817; }
  l820:;	
  {  int yypos821= G->pos, yythunkpos821= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l821; }  goto l820;
  l821:;	  G->pos= yypos821; G->thunkpos= yythunkpos821;
  }  if (!yymatchChar(G, '>')) goto l817;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenTbody", G->buf+G->pos));
  return 1;
  l817:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenTbody", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockLi(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockLi"));  if (!yy_HtmlBlockOpenLi(G)) { goto l822; }
  l823:;	
  {  int yypos824= G->pos, yythunkpos824= G->thunkpos;
  {  int yypos825= G->pos, yythunkpos825= G->thunkpos;  if (!yy_HtmlBlockLi(G)) { goto l826; }  goto l825;
  l826:;	  G->pos= yypos825; G->thunkpos= yythunkpos825;
  {  int yypos827= G->pos, yythunkpos827= G->thunkpos;  if (!yy_HtmlBlockCloseLi(G)) { goto l827; }  goto l824;
  l827:;	  G->pos= yypos827; G->thunkpos= yythunkpos827;
  }  if (!yymatchDot(G)) goto l824;
  }
  l825:;	  goto l823;
  l824:;	  G->pos= yypos824; G->thunkpos= yythunkpos824;
  }  if (!yy_HtmlBlockCloseLi(G)) { goto l822; }
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockLi", G->buf+G->pos));
  return 1;
  l822:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockLi", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseLi(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseLi"));  if (!yymatchChar(G, '<')) goto l828;  if (!yy_Spnl(G)) { goto l828; }  if (!yymatchChar(G, '/')) goto l828;
  {  int yypos829= G->pos, yythunkpos829= G->thunkpos;  if (!yymatchString(G, "li")) goto l830;  goto l829;
  l830:;	  G->pos= yypos829; G->thunkpos= yythunkpos829;  if (!yymatchString(G, "LI")) goto l828;
  }
  l829:;	  if (!yy_Spnl(G)) { goto l828; }  if (!yymatchChar(G, '>')) goto l828;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseLi", G->buf+G->pos));
  return 1;
  l828:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseLi", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenLi(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenLi"));  if (!yymatchChar(G, '<')) goto l831;  if (!yy_Spnl(G)) { goto l831; }
  {  int yypos832= G->pos, yythunkpos832= G->thunkpos;  if (!yymatchString(G, "li")) goto l833;  goto l832;
  l833:;	  G->pos= yypos832; G->thunkpos= yythunkpos832;  if (!yymatchString(G, "LI")) goto l831;
  }
  l832:;	  if (!yy_Spnl(G)) { goto l831; }
  l834:;	
  {  int yypos835= G->pos, yythunkpos835= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l835; }  goto l834;
  l835:;	  G->pos= yypos835; G->thunkpos= yythunkpos835;
  }  if (!yymatchChar(G, '>')) goto l831;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenLi", G->buf+G->pos));
  return 1;
  l831:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenLi", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockFrameset(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockFrameset"));  if (!yy_HtmlBlockOpenFrameset(G)) { goto l836; }
  l837:;	
  {  int yypos838= G->pos, yythunkpos838= G->thunkpos;
  {  int yypos839= G->pos, yythunkpos839= G->thunkpos;  if (!yy_HtmlBlockFrameset(G)) { goto l840; }  goto l839;
  l840:;	  G->pos= yypos839; G->thunkpos= yythunkpos839;
  {  int yypos841= G->pos, yythunkpos841= G->thunkpos;  if (!yy_HtmlBlockCloseFrameset(G)) { goto l841; }  goto l838;
  l841:;	  G->pos= yypos841; G->thunkpos= yythunkpos841;
  }  if (!yymatchDot(G)) goto l838;
  }
  l839:;	  goto l837;
  l838:;	  G->pos= yypos838; G->thunkpos= yythunkpos838;
  }  if (!yy_HtmlBlockCloseFrameset(G)) { goto l836; }
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockFrameset", G->buf+G->pos));
  return 1;
  l836:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockFrameset", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseFrameset(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseFrameset"));  if (!yymatchChar(G, '<')) goto l842;  if (!yy_Spnl(G)) { goto l842; }  if (!yymatchChar(G, '/')) goto l842;
  {  int yypos843= G->pos, yythunkpos843= G->thunkpos;  if (!yymatchString(G, "frameset")) goto l844;  goto l843;
  l844:;	  G->pos= yypos843; G->thunkpos= yythunkpos843;  if (!yymatchString(G, "FRAMESET")) goto l842;
  }
  l843:;	  if (!yy_Spnl(G)) { goto l842; }  if (!yymatchChar(G, '>')) goto l842;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseFrameset", G->buf+G->pos));
  return 1;
  l842:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseFrameset", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenFrameset(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenFrameset"));  if (!yymatchChar(G, '<')) goto l845;  if (!yy_Spnl(G)) { goto l845; }
  {  int yypos846= G->pos, yythunkpos846= G->thunkpos;  if (!yymatchString(G, "frameset")) goto l847;  goto l846;
  l847:;	  G->pos= yypos846; G->thunkpos= yythunkpos846;  if (!yymatchString(G, "FRAMESET")) goto l845;
  }
  l846:;	  if (!yy_Spnl(G)) { goto l845; }
  l848:;	
  {  int yypos849= G->pos, yythunkpos849= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l849; }  goto l848;
  l849:;	  G->pos= yypos849; G->thunkpos= yythunkpos849;
  }  if (!yymatchChar(G, '>')) goto l845;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenFrameset", G->buf+G->pos));
  return 1;
  l845:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenFrameset", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockDt(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockDt"));  if (!yy_HtmlBlockOpenDt(G)) { goto l850; }
  l851:;	
  {  int yypos852= G->pos, yythunkpos852= G->thunkpos;
  {  int yypos853= G->pos, yythunkpos853= G->thunkpos;  if (!yy_HtmlBlockDt(G)) { goto l854; }  goto l853;
  l854:;	  G->pos= yypos853; G->thunkpos= yythunkpos853;
  {  int yypos855= G->pos, yythunkpos855= G->thunkpos;  if (!yy_HtmlBlockCloseDt(G)) { goto l855; }  goto l852;
  l855:;	  G->pos= yypos855; G->thunkpos= yythunkpos855;
  }  if (!yymatchDot(G)) goto l852;
  }
  l853:;	  goto l851;
  l852:;	  G->pos= yypos852; G->thunkpos= yythunkpos852;
  }  if (!yy_HtmlBlockCloseDt(G)) { goto l850; }
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockDt", G->buf+G->pos));
  return 1;
  l850:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockDt", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseDt(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseDt"));  if (!yymatchChar(G, '<')) goto l856;  if (!yy_Spnl(G)) { goto l856; }  if (!yymatchChar(G, '/')) goto l856;
  {  int yypos857= G->pos, yythunkpos857= G->thunkpos;  if (!yymatchString(G, "dt")) goto l858;  goto l857;
  l858:;	  G->pos= yypos857; G->thunkpos= yythunkpos857;  if (!yymatchString(G, "DT")) goto l856;
  }
  l857:;	  if (!yy_Spnl(G)) { goto l856; }  if (!yymatchChar(G, '>')) goto l856;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseDt", G->buf+G->pos));
  return 1;
  l856:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseDt", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenDt(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenDt"));  if (!yymatchChar(G, '<')) goto l859;  if (!yy_Spnl(G)) { goto l859; }
  {  int yypos860= G->pos, yythunkpos860= G->thunkpos;  if (!yymatchString(G, "dt")) goto l861;  goto l860;
  l861:;	  G->pos= yypos860; G->thunkpos= yythunkpos860;  if (!yymatchString(G, "DT")) goto l859;
  }
  l860:;	  if (!yy_Spnl(G)) { goto l859; }
  l862:;	
  {  int yypos863= G->pos, yythunkpos863= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l863; }  goto l862;
  l863:;	  G->pos= yypos863; G->thunkpos= yythunkpos863;
  }  if (!yymatchChar(G, '>')) goto l859;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenDt", G->buf+G->pos));
  return 1;
  l859:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenDt", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockDd(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockDd"));  if (!yy_HtmlBlockOpenDd(G)) { goto l864; }
  l865:;	
  {  int yypos866= G->pos, yythunkpos866= G->thunkpos;
  {  int yypos867= G->pos, yythunkpos867= G->thunkpos;  if (!yy_HtmlBlockDd(G)) { goto l868; }  goto l867;
  l868:;	  G->pos= yypos867; G->thunkpos= yythunkpos867;
  {  int yypos869= G->pos, yythunkpos869= G->thunkpos;  if (!yy_HtmlBlockCloseDd(G)) { goto l869; }  goto l866;
  l869:;	  G->pos= yypos869; G->thunkpos= yythunkpos869;
  }  if (!yymatchDot(G)) goto l866;
  }
  l867:;	  goto l865;
  l866:;	  G->pos= yypos866; G->thunkpos= yythunkpos866;
  }  if (!yy_HtmlBlockCloseDd(G)) { goto l864; }
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockDd", G->buf+G->pos));
  return 1;
  l864:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockDd", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseDd(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseDd"));  if (!yymatchChar(G, '<')) goto l870;  if (!yy_Spnl(G)) { goto l870; }  if (!yymatchChar(G, '/')) goto l870;
  {  int yypos871= G->pos, yythunkpos871= G->thunkpos;  if (!yymatchString(G, "dd")) goto l872;  goto l871;
  l872:;	  G->pos= yypos871; G->thunkpos= yythunkpos871;  if (!yymatchString(G, "DD")) goto l870;
  }
  l871:;	  if (!yy_Spnl(G)) { goto l870; }  if (!yymatchChar(G, '>')) goto l870;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseDd", G->buf+G->pos));
  return 1;
  l870:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseDd", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenDd(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenDd"));  if (!yymatchChar(G, '<')) goto l873;  if (!yy_Spnl(G)) { goto l873; }
  {  int yypos874= G->pos, yythunkpos874= G->thunkpos;  if (!yymatchString(G, "dd")) goto l875;  goto l874;
  l875:;	  G->pos= yypos874; G->thunkpos= yythunkpos874;  if (!yymatchString(G, "DD")) goto l873;
  }
  l874:;	  if (!yy_Spnl(G)) { goto l873; }
  l876:;	
  {  int yypos877= G->pos, yythunkpos877= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l877; }  goto l876;
  l877:;	  G->pos= yypos877; G->thunkpos= yythunkpos877;
  }  if (!yymatchChar(G, '>')) goto l873;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenDd", G->buf+G->pos));
  return 1;
  l873:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenDd", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockUl(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockUl"));  if (!yy_HtmlBlockOpenUl(G)) { goto l878; }
  l879:;	
  {  int yypos880= G->pos, yythunkpos880= G->thunkpos;
  {  int yypos881= G->pos, yythunkpos881= G->thunkpos;  if (!yy_HtmlBlockUl(G)) { goto l882; }  goto l881;
  l882:;	  G->pos= yypos881; G->thunkpos= yythunkpos881;
  {  int yypos883= G->pos, yythunkpos883= G->thunkpos;  if (!yy_HtmlBlockCloseUl(G)) { goto l883; }  goto l880;
  l883:;	  G->pos= yypos883; G->thunkpos= yythunkpos883;
  }  if (!yymatchDot(G)) goto l880;
  }
  l881:;	  goto l879;
  l880:;	  G->pos= yypos880; G->thunkpos= yythunkpos880;
  }  if (!yy_HtmlBlockCloseUl(G)) { goto l878; }
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockUl", G->buf+G->pos));
  return 1;
  l878:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockUl", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseUl(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseUl"));  if (!yymatchChar(G, '<')) goto l884;  if (!yy_Spnl(G)) { goto l884; }  if (!yymatchChar(G, '/')) goto l884;
  {  int yypos885= G->pos, yythunkpos885= G->thunkpos;  if (!yymatchString(G, "ul")) goto l886;  goto l885;
  l886:;	  G->pos= yypos885; G->thunkpos= yythunkpos885;  if (!yymatchString(G, "UL")) goto l884;
  }
  l885:;	  if (!yy_Spnl(G)) { goto l884; }  if (!yymatchChar(G, '>')) goto l884;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseUl", G->buf+G->pos));
  return 1;
  l884:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseUl", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenUl(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenUl"));  if (!yymatchChar(G, '<')) goto l887;  if (!yy_Spnl(G)) { goto l887; }
  {  int yypos888= G->pos, yythunkpos888= G->thunkpos;  if (!yymatchString(G, "ul")) goto l889;  goto l888;
  l889:;	  G->pos= yypos888; G->thunkpos= yythunkpos888;  if (!yymatchString(G, "UL")) goto l887;
  }
  l888:;	  if (!yy_Spnl(G)) { goto l887; }
  l890:;	
  {  int yypos891= G->pos, yythunkpos891= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l891; }  goto l890;
  l891:;	  G->pos= yypos891; G->thunkpos= yythunkpos891;
  }  if (!yymatchChar(G, '>')) goto l887;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenUl", G->buf+G->pos));
  return 1;
  l887:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenUl", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockTable(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockTable"));  if (!yy_HtmlBlockOpenTable(G)) { goto l892; }
  l893:;	
  {  int yypos894= G->pos, yythunkpos894= G->thunkpos;
  {  int yypos895= G->pos, yythunkpos895= G->thunkpos;  if (!yy_HtmlBlockTable(G)) { goto l896; }  goto l895;
  l896:;	  G->pos= yypos895; G->thunkpos= yythunkpos895;
  {  int yypos897= G->pos, yythunkpos897= G->thunkpos;  if (!yy_HtmlBlockCloseTable(G)) { goto l897; }  goto l894;
  l897:;	  G->pos= yypos897; G->thunkpos= yythunkpos897;
  }  if (!yymatchDot(G)) goto l894;
  }
  l895:;	  goto l893;
  l894:;	  G->pos= yypos894; G->thunkpos= yythunkpos894;
  }  if (!yy_HtmlBlockCloseTable(G)) { goto l892; }
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockTable", G->buf+G->pos));
  return 1;
  l892:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockTable", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseTable(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseTable"));  if (!yymatchChar(G, '<')) goto l898;  if (!yy_Spnl(G)) { goto l898; }  if (!yymatchChar(G, '/')) goto l898;
  {  int yypos899= G->pos, yythunkpos899= G->thunkpos;  if (!yymatchString(G, "table")) goto l900;  goto l899;
  l900:;	  G->pos= yypos899; G->thunkpos= yythunkpos899;  if (!yymatchString(G, "TABLE")) goto l898;
  }
  l899:;	  if (!yy_Spnl(G)) { goto l898; }  if (!yymatchChar(G, '>')) goto l898;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseTable", G->buf+G->pos));
  return 1;
  l898:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseTable", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenTable(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenTable"));  if (!yymatchChar(G, '<')) goto l901;  if (!yy_Spnl(G)) { goto l901; }
  {  int yypos902= G->pos, yythunkpos902= G->thunkpos;  if (!yymatchString(G, "table")) goto l903;  goto l902;
  l903:;	  G->pos= yypos902; G->thunkpos= yythunkpos902;  if (!yymatchString(G, "TABLE")) goto l901;
  }
  l902:;	  if (!yy_Spnl(G)) { goto l901; }
  l904:;	
  {  int yypos905= G->pos, yythunkpos905= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l905; }  goto l904;
  l905:;	  G->pos= yypos905; G->thunkpos= yythunkpos905;
  }  if (!yymatchChar(G, '>')) goto l901;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenTable", G->buf+G->pos));
  return 1;
  l901:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenTable", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockPre(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockPre"));  if (!yy_HtmlBlockOpenPre(G)) { goto l906; }
  l907:;	
  {  int yypos908= G->pos, yythunkpos908= G->thunkpos;
  {  int yypos909= G->pos, yythunkpos909= G->thunkpos;  if (!yy_HtmlBlockPre(G)) { goto l910; }  goto l909;
  l910:;	  G->pos= yypos909; G->thunkpos= yythunkpos909;
  {  int yypos911= G->pos, yythunkpos911= G->thunkpos;  if (!yy_HtmlBlockClosePre(G)) { goto l911; }  goto l908;
  l911:;	  G->pos= yypos911; G->thunkpos= yythunkpos911;
  }  if (!yymatchDot(G)) goto l908;
  }
  l909:;	  goto l907;
  l908:;	  G->pos= yypos908; G->thunkpos= yythunkpos908;
  }  if (!yy_HtmlBlockClosePre(G)) { goto l906; }
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockPre", G->buf+G->pos));
  return 1;
  l906:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockPre", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockClosePre(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockClosePre"));  if (!yymatchChar(G, '<')) goto l912;  if (!yy_Spnl(G)) { goto l912; }  if (!yymatchChar(G, '/')) goto l912;
  {  int yypos913= G->pos, yythunkpos913= G->thunkpos;  if (!yymatchString(G, "pre")) goto l914;  goto l913;
  l914:;	  G->pos= yypos913; G->thunkpos= yythunkpos913;  if (!yymatchString(G, "PRE")) goto l912;
  }
  l913:;	  if (!yy_Spnl(G)) { goto l912; }  if (!yymatchChar(G, '>')) goto l912;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockClosePre", G->buf+G->pos));
  return 1;
  l912:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockClosePre", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenPre(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenPre"));  if (!yymatchChar(G, '<')) goto l915;  if (!yy_Spnl(G)) { goto l915; }
  {  int yypos916= G->pos, yythunkpos916= G->thunkpos;  if (!yymatchString(G, "pre")) goto l917;  goto l916;
  l917:;	  G->pos= yypos916; G->thunkpos= yythunkpos916;  if (!yymatchString(G, "PRE")) goto l915;
  }
  l916:;	  if (!yy_Spnl(G)) { goto l915; }
  l918:;	
  {  int yypos919= G->pos, yythunkpos919= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l919; }  goto l918;
  l919:;	  G->pos= yypos919; G->thunkpos= yythunkpos919;
  }  if (!yymatchChar(G, '>')) goto l915;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenPre", G->buf+G->pos));
  return 1;
  l915:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenPre", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockP(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockP"));  if (!yy_HtmlBlockOpenP(G)) { goto l920; }
  l921:;	
  {  int yypos922= G->pos, yythunkpos922= G->thunkpos;
  {  int yypos923= G->pos, yythunkpos923= G->thunkpos;  if (!yy_HtmlBlockP(G)) { goto l924; }  goto l923;
  l924:;	  G->pos= yypos923; G->thunkpos= yythunkpos923;
  {  int yypos925= G->pos, yythunkpos925= G->thunkpos;  if (!yy_HtmlBlockCloseP(G)) { goto l925; }  goto l922;
  l925:;	  G->pos= yypos925; G->thunkpos= yythunkpos925;
  }  if (!yymatchDot(G)) goto l922;
  }
  l923:;	  goto l921;
  l922:;	  G->pos= yypos922; G->thunkpos= yythunkpos922;
  }  if (!yy_HtmlBlockCloseP(G)) { goto l920; }
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockP", G->buf+G->pos));
  return 1;
  l920:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockP", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseP(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseP"));  if (!yymatchChar(G, '<')) goto l926;  if (!yy_Spnl(G)) { goto l926; }  if (!yymatchChar(G, '/')) goto l926;
  {  int yypos927= G->pos, yythunkpos927= G->thunkpos;  if (!yymatchChar(G, 'p')) goto l928;  goto l927;
  l928:;	  G->pos= yypos927; G->thunkpos= yythunkpos927;  if (!yymatchChar(G, 'P')) goto l926;
  }
  l927:;	  if (!yy_Spnl(G)) { goto l926; }  if (!yymatchChar(G, '>')) goto l926;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseP", G->buf+G->pos));
  return 1;
  l926:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseP", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenP(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenP"));  if (!yymatchChar(G, '<')) goto l929;  if (!yy_Spnl(G)) { goto l929; }
  {  int yypos930= G->pos, yythunkpos930= G->thunkpos;  if (!yymatchChar(G, 'p')) goto l931;  goto l930;
  l931:;	  G->pos= yypos930; G->thunkpos= yythunkpos930;  if (!yymatchChar(G, 'P')) goto l929;
  }
  l930:;	  if (!yy_Spnl(G)) { goto l929; }
  l932:;	
  {  int yypos933= G->pos, yythunkpos933= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l933; }  goto l932;
  l933:;	  G->pos= yypos933; G->thunkpos= yythunkpos933;
  }  if (!yymatchChar(G, '>')) goto l929;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenP", G->buf+G->pos));
  return 1;
  l929:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenP", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOl(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOl"));  if (!yy_HtmlBlockOpenOl(G)) { goto l934; }
  l935:;	
  {  int yypos936= G->pos, yythunkpos936= G->thunkpos;
  {  int yypos937= G->pos, yythunkpos937= G->thunkpos;  if (!yy_HtmlBlockOl(G)) { goto l938; }  goto l937;
  l938:;	  G->pos= yypos937; G->thunkpos= yythunkpos937;
  {  int yypos939= G->pos, yythunkpos939= G->thunkpos;  if (!yy_HtmlBlockCloseOl(G)) { goto l939; }  goto l936;
  l939:;	  G->pos= yypos939; G->thunkpos= yythunkpos939;
  }  if (!yymatchDot(G)) goto l936;
  }
  l937:;	  goto l935;
  l936:;	  G->pos= yypos936; G->thunkpos= yythunkpos936;
  }  if (!yy_HtmlBlockCloseOl(G)) { goto l934; }
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOl", G->buf+G->pos));
  return 1;
  l934:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOl", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseOl(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseOl"));  if (!yymatchChar(G, '<')) goto l940;  if (!yy_Spnl(G)) { goto l940; }  if (!yymatchChar(G, '/')) goto l940;
  {  int yypos941= G->pos, yythunkpos941= G->thunkpos;  if (!yymatchString(G, "ol")) goto l942;  goto l941;
  l942:;	  G->pos= yypos941; G->thunkpos= yythunkpos941;  if (!yymatchString(G, "OL")) goto l940;
  }
  l941:;	  if (!yy_Spnl(G)) { goto l940; }  if (!yymatchChar(G, '>')) goto l940;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseOl", G->buf+G->pos));
  return 1;
  l940:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseOl", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenOl(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenOl"));  if (!yymatchChar(G, '<')) goto l943;  if (!yy_Spnl(G)) { goto l943; }
  {  int yypos944= G->pos, yythunkpos944= G->thunkpos;  if (!yymatchString(G, "ol")) goto l945;  goto l944;
  l945:;	  G->pos= yypos944; G->thunkpos= yythunkpos944;  if (!yymatchString(G, "OL")) goto l943;
  }
  l944:;	  if (!yy_Spnl(G)) { goto l943; }
  l946:;	
  {  int yypos947= G->pos, yythunkpos947= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l947; }  goto l946;
  l947:;	  G->pos= yypos947; G->thunkpos= yythunkpos947;
  }  if (!yymatchChar(G, '>')) goto l943;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenOl", G->buf+G->pos));
  return 1;
  l943:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenOl", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockNoscript(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockNoscript"));  if (!yy_HtmlBlockOpenNoscript(G)) { goto l948; }
  l949:;	
  {  int yypos950= G->pos, yythunkpos950= G->thunkpos;
  {  int yypos951= G->pos, yythunkpos951= G->thunkpos;  if (!yy_HtmlBlockNoscript(G)) { goto l952; }  goto l951;
  l952:;	  G->pos= yypos951; G->thunkpos= yythunkpos951;
  {  int yypos953= G->pos, yythunkpos953= G->thunkpos;  if (!yy_HtmlBlockCloseNoscript(G)) { goto l953; }  goto l950;
  l953:;	  G->pos= yypos953; G->thunkpos= yythunkpos953;
  }  if (!yymatchDot(G)) goto l950;
  }
  l951:;	  goto l949;
  l950:;	  G->pos= yypos950; G->thunkpos= yythunkpos950;
  }  if (!yy_HtmlBlockCloseNoscript(G)) { goto l948; }
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockNoscript", G->buf+G->pos));
  return 1;
  l948:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockNoscript", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseNoscript(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseNoscript"));  if (!yymatchChar(G, '<')) goto l954;  if (!yy_Spnl(G)) { goto l954; }  if (!yymatchChar(G, '/')) goto l954;
  {  int yypos955= G->pos, yythunkpos955= G->thunkpos;  if (!yymatchString(G, "noscript")) goto l956;  goto l955;
  l956:;	  G->pos= yypos955; G->thunkpos= yythunkpos955;  if (!yymatchString(G, "NOSCRIPT")) goto l954;
  }
  l955:;	  if (!yy_Spnl(G)) { goto l954; }  if (!yymatchChar(G, '>')) goto l954;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseNoscript", G->buf+G->pos));
  return 1;
  l954:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseNoscript", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenNoscript(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenNoscript"));  if (!yymatchChar(G, '<')) goto l957;  if (!yy_Spnl(G)) { goto l957; }
  {  int yypos958= G->pos, yythunkpos958= G->thunkpos;  if (!yymatchString(G, "noscript")) goto l959;  goto l958;
  l959:;	  G->pos= yypos958; G->thunkpos= yythunkpos958;  if (!yymatchString(G, "NOSCRIPT")) goto l957;
  }
  l958:;	  if (!yy_Spnl(G)) { goto l957; }
  l960:;	
  {  int yypos961= G->pos, yythunkpos961= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l961; }  goto l960;
  l961:;	  G->pos= yypos961; G->thunkpos= yythunkpos961;
  }  if (!yymatchChar(G, '>')) goto l957;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenNoscript", G->buf+G->pos));
  return 1;
  l957:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenNoscript", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockNoframes(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockNoframes"));  if (!yy_HtmlBlockOpenNoframes(G)) { goto l962; }
  l963:;	
  {  int yypos964= G->pos, yythunkpos964= G->thunkpos;
  {  int yypos965= G->pos, yythunkpos965= G->thunkpos;  if (!yy_HtmlBlockNoframes(G)) { goto l966; }  goto l965;
  l966:;	  G->pos= yypos965; G->thunkpos= yythunkpos965;
  {  int yypos967= G->pos, yythunkpos967= G->thunkpos;  if (!yy_HtmlBlockCloseNoframes(G)) { goto l967; }  goto l964;
  l967:;	  G->pos= yypos967; G->thunkpos= yythunkpos967;
  }  if (!yymatchDot(G)) goto l964;
  }
  l965:;	  goto l963;
  l964:;	  G->pos= yypos964; G->thunkpos= yythunkpos964;
  }  if (!yy_HtmlBlockCloseNoframes(G)) { goto l962; }
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockNoframes", G->buf+G->pos));
  return 1;
  l962:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockNoframes", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseNoframes(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseNoframes"));  if (!yymatchChar(G, '<')) goto l968;  if (!yy_Spnl(G)) { goto l968; }  if (!yymatchChar(G, '/')) goto l968;
  {  int yypos969= G->pos, yythunkpos969= G->thunkpos;  if (!yymatchString(G, "noframes")) goto l970;  goto l969;
  l970:;	  G->pos= yypos969; G->thunkpos= yythunkpos969;  if (!yymatchString(G, "NOFRAMES")) goto l968;
  }
  l969:;	  if (!yy_Spnl(G)) { goto l968; }  if (!yymatchChar(G, '>')) goto l968;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseNoframes", G->buf+G->pos));
  return 1;
  l968:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseNoframes", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenNoframes(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenNoframes"));  if (!yymatchChar(G, '<')) goto l971;  if (!yy_Spnl(G)) { goto l971; }
  {  int yypos972= G->pos, yythunkpos972= G->thunkpos;  if (!yymatchString(G, "noframes")) goto l973;  goto l972;
  l973:;	  G->pos= yypos972; G->thunkpos= yythunkpos972;  if (!yymatchString(G, "NOFRAMES")) goto l971;
  }
  l972:;	  if (!yy_Spnl(G)) { goto l971; }
  l974:;	
  {  int yypos975= G->pos, yythunkpos975= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l975; }  goto l974;
  l975:;	  G->pos= yypos975; G->thunkpos= yythunkpos975;
  }  if (!yymatchChar(G, '>')) goto l971;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenNoframes", G->buf+G->pos));
  return 1;
  l971:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenNoframes", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockMenu(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockMenu"));  if (!yy_HtmlBlockOpenMenu(G)) { goto l976; }
  l977:;	
  {  int yypos978= G->pos, yythunkpos978= G->thunkpos;
  {  int yypos979= G->pos, yythunkpos979= G->thunkpos;  if (!yy_HtmlBlockMenu(G)) { goto l980; }  goto l979;
  l980:;	  G->pos= yypos979; G->thunkpos= yythunkpos979;
  {  int yypos981= G->pos, yythunkpos981= G->thunkpos;  if (!yy_HtmlBlockCloseMenu(G)) { goto l981; }  goto l978;
  l981:;	  G->pos= yypos981; G->thunkpos= yythunkpos981;
  }  if (!yymatchDot(G)) goto l978;
  }
  l979:;	  goto l977;
  l978:;	  G->pos= yypos978; G->thunkpos= yythunkpos978;
  }  if (!yy_HtmlBlockCloseMenu(G)) { goto l976; }
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockMenu", G->buf+G->pos));
  return 1;
  l976:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockMenu", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseMenu(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseMenu"));  if (!yymatchChar(G, '<')) goto l982;  if (!yy_Spnl(G)) { goto l982; }  if (!yymatchChar(G, '/')) goto l982;
  {  int yypos983= G->pos, yythunkpos983= G->thunkpos;  if (!yymatchString(G, "menu")) goto l984;  goto l983;
  l984:;	  G->pos= yypos983; G->thunkpos= yythunkpos983;  if (!yymatchString(G, "MENU")) goto l982;
  }
  l983:;	  if (!yy_Spnl(G)) { goto l982; }  if (!yymatchChar(G, '>')) goto l982;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseMenu", G->buf+G->pos));
  return 1;
  l982:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseMenu", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenMenu(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenMenu"));  if (!yymatchChar(G, '<')) goto l985;  if (!yy_Spnl(G)) { goto l985; }
  {  int yypos986= G->pos, yythunkpos986= G->thunkpos;  if (!yymatchString(G, "menu")) goto l987;  goto l986;
  l987:;	  G->pos= yypos986; G->thunkpos= yythunkpos986;  if (!yymatchString(G, "MENU")) goto l985;
  }
  l986:;	  if (!yy_Spnl(G)) { goto l985; }
  l988:;	
  {  int yypos989= G->pos, yythunkpos989= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l989; }  goto l988;
  l989:;	  G->pos= yypos989; G->thunkpos= yythunkpos989;
  }  if (!yymatchChar(G, '>')) goto l985;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenMenu", G->buf+G->pos));
  return 1;
  l985:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenMenu", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockH6(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "HtmlBlockH6"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l990;  if (!yy_LocMarker(G)) { goto l990; }  yyDo(G, yySet, -1, 0);  if (!yy_HtmlBlockOpenH6(G)) { goto l990; }
  l991:;	
  {  int yypos992= G->pos, yythunkpos992= G->thunkpos;
  {  int yypos993= G->pos, yythunkpos993= G->thunkpos;  if (!yy_HtmlBlockH6(G)) { goto l994; }  goto l993;
  l994:;	  G->pos= yypos993; G->thunkpos= yythunkpos993;
  {  int yypos995= G->pos, yythunkpos995= G->thunkpos;  if (!yy_HtmlBlockCloseH6(G)) { goto l995; }  goto l992;
  l995:;	  G->pos= yypos995; G->thunkpos= yythunkpos995;
  }  if (!yymatchDot(G)) goto l992;
  }
  l993:;	  goto l991;
  l992:;	  G->pos= yypos992; G->thunkpos= yythunkpos992;
  }  if (!yy_HtmlBlockCloseH6(G)) { goto l990; }  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l990;  yyDo(G, yy_1_HtmlBlockH6, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockH6", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l990:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockH6", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseH6(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseH6"));  if (!yymatchChar(G, '<')) goto l996;  if (!yy_Spnl(G)) { goto l996; }  if (!yymatchChar(G, '/')) goto l996;
  {  int yypos997= G->pos, yythunkpos997= G->thunkpos;  if (!yymatchString(G, "h6")) goto l998;  goto l997;
  l998:;	  G->pos= yypos997; G->thunkpos= yythunkpos997;  if (!yymatchString(G, "H6")) goto l996;
  }
  l997:;	  if (!yy_Spnl(G)) { goto l996; }  if (!yymatchChar(G, '>')) goto l996;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseH6", G->buf+G->pos));
  return 1;
  l996:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseH6", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenH6(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenH6"));  if (!yymatchChar(G, '<')) goto l999;  if (!yy_Spnl(G)) { goto l999; }
  {  int yypos1000= G->pos, yythunkpos1000= G->thunkpos;  if (!yymatchString(G, "h6")) goto l1001;  goto l1000;
  l1001:;	  G->pos= yypos1000; G->thunkpos= yythunkpos1000;  if (!yymatchString(G, "H6")) goto l999;
  }
  l1000:;	  if (!yy_Spnl(G)) { goto l999; }
  l1002:;	
  {  int yypos1003= G->pos, yythunkpos1003= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l1003; }  goto l1002;
  l1003:;	  G->pos= yypos1003; G->thunkpos= yythunkpos1003;
  }  if (!yymatchChar(G, '>')) goto l999;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenH6", G->buf+G->pos));
  return 1;
  l999:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenH6", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockH5(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "HtmlBlockH5"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l1004;  if (!yy_LocMarker(G)) { goto l1004; }  yyDo(G, yySet, -1, 0);  if (!yy_HtmlBlockOpenH5(G)) { goto l1004; }
  l1005:;	
  {  int yypos1006= G->pos, yythunkpos1006= G->thunkpos;
  {  int yypos1007= G->pos, yythunkpos1007= G->thunkpos;  if (!yy_HtmlBlockH5(G)) { goto l1008; }  goto l1007;
  l1008:;	  G->pos= yypos1007; G->thunkpos= yythunkpos1007;
  {  int yypos1009= G->pos, yythunkpos1009= G->thunkpos;  if (!yy_HtmlBlockCloseH5(G)) { goto l1009; }  goto l1006;
  l1009:;	  G->pos= yypos1009; G->thunkpos= yythunkpos1009;
  }  if (!yymatchDot(G)) goto l1006;
  }
  l1007:;	  goto l1005;
  l1006:;	  G->pos= yypos1006; G->thunkpos= yythunkpos1006;
  }  if (!yy_HtmlBlockCloseH5(G)) { goto l1004; }  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l1004;  yyDo(G, yy_1_HtmlBlockH5, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockH5", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l1004:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockH5", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseH5(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseH5"));  if (!yymatchChar(G, '<')) goto l1010;  if (!yy_Spnl(G)) { goto l1010; }  if (!yymatchChar(G, '/')) goto l1010;
  {  int yypos1011= G->pos, yythunkpos1011= G->thunkpos;  if (!yymatchString(G, "h5")) goto l1012;  goto l1011;
  l1012:;	  G->pos= yypos1011; G->thunkpos= yythunkpos1011;  if (!yymatchString(G, "H5")) goto l1010;
  }
  l1011:;	  if (!yy_Spnl(G)) { goto l1010; }  if (!yymatchChar(G, '>')) goto l1010;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseH5", G->buf+G->pos));
  return 1;
  l1010:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseH5", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenH5(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenH5"));  if (!yymatchChar(G, '<')) goto l1013;  if (!yy_Spnl(G)) { goto l1013; }
  {  int yypos1014= G->pos, yythunkpos1014= G->thunkpos;  if (!yymatchString(G, "h5")) goto l1015;  goto l1014;
  l1015:;	  G->pos= yypos1014; G->thunkpos= yythunkpos1014;  if (!yymatchString(G, "H5")) goto l1013;
  }
  l1014:;	  if (!yy_Spnl(G)) { goto l1013; }
  l1016:;	
  {  int yypos1017= G->pos, yythunkpos1017= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l1017; }  goto l1016;
  l1017:;	  G->pos= yypos1017; G->thunkpos= yythunkpos1017;
  }  if (!yymatchChar(G, '>')) goto l1013;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenH5", G->buf+G->pos));
  return 1;
  l1013:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenH5", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockH4(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "HtmlBlockH4"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l1018;  if (!yy_LocMarker(G)) { goto l1018; }  yyDo(G, yySet, -1, 0);  if (!yy_HtmlBlockOpenH4(G)) { goto l1018; }
  l1019:;	
  {  int yypos1020= G->pos, yythunkpos1020= G->thunkpos;
  {  int yypos1021= G->pos, yythunkpos1021= G->thunkpos;  if (!yy_HtmlBlockH4(G)) { goto l1022; }  goto l1021;
  l1022:;	  G->pos= yypos1021; G->thunkpos= yythunkpos1021;
  {  int yypos1023= G->pos, yythunkpos1023= G->thunkpos;  if (!yy_HtmlBlockCloseH4(G)) { goto l1023; }  goto l1020;
  l1023:;	  G->pos= yypos1023; G->thunkpos= yythunkpos1023;
  }  if (!yymatchDot(G)) goto l1020;
  }
  l1021:;	  goto l1019;
  l1020:;	  G->pos= yypos1020; G->thunkpos= yythunkpos1020;
  }  if (!yy_HtmlBlockCloseH4(G)) { goto l1018; }  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l1018;  yyDo(G, yy_1_HtmlBlockH4, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockH4", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l1018:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockH4", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseH4(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseH4"));  if (!yymatchChar(G, '<')) goto l1024;  if (!yy_Spnl(G)) { goto l1024; }  if (!yymatchChar(G, '/')) goto l1024;
  {  int yypos1025= G->pos, yythunkpos1025= G->thunkpos;  if (!yymatchString(G, "h4")) goto l1026;  goto l1025;
  l1026:;	  G->pos= yypos1025; G->thunkpos= yythunkpos1025;  if (!yymatchString(G, "H4")) goto l1024;
  }
  l1025:;	  if (!yy_Spnl(G)) { goto l1024; }  if (!yymatchChar(G, '>')) goto l1024;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseH4", G->buf+G->pos));
  return 1;
  l1024:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseH4", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenH4(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenH4"));  if (!yymatchChar(G, '<')) goto l1027;  if (!yy_Spnl(G)) { goto l1027; }
  {  int yypos1028= G->pos, yythunkpos1028= G->thunkpos;  if (!yymatchString(G, "h4")) goto l1029;  goto l1028;
  l1029:;	  G->pos= yypos1028; G->thunkpos= yythunkpos1028;  if (!yymatchString(G, "H4")) goto l1027;
  }
  l1028:;	  if (!yy_Spnl(G)) { goto l1027; }
  l1030:;	
  {  int yypos1031= G->pos, yythunkpos1031= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l1031; }  goto l1030;
  l1031:;	  G->pos= yypos1031; G->thunkpos= yythunkpos1031;
  }  if (!yymatchChar(G, '>')) goto l1027;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenH4", G->buf+G->pos));
  return 1;
  l1027:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenH4", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockH3(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "HtmlBlockH3"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l1032;  if (!yy_LocMarker(G)) { goto l1032; }  yyDo(G, yySet, -1, 0);  if (!yy_HtmlBlockOpenH3(G)) { goto l1032; }
  l1033:;	
  {  int yypos1034= G->pos, yythunkpos1034= G->thunkpos;
  {  int yypos1035= G->pos, yythunkpos1035= G->thunkpos;  if (!yy_HtmlBlockH3(G)) { goto l1036; }  goto l1035;
  l1036:;	  G->pos= yypos1035; G->thunkpos= yythunkpos1035;
  {  int yypos1037= G->pos, yythunkpos1037= G->thunkpos;  if (!yy_HtmlBlockCloseH3(G)) { goto l1037; }  goto l1034;
  l1037:;	  G->pos= yypos1037; G->thunkpos= yythunkpos1037;
  }  if (!yymatchDot(G)) goto l1034;
  }
  l1035:;	  goto l1033;
  l1034:;	  G->pos= yypos1034; G->thunkpos= yythunkpos1034;
  }  if (!yy_HtmlBlockCloseH3(G)) { goto l1032; }  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l1032;  yyDo(G, yy_1_HtmlBlockH3, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockH3", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l1032:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockH3", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseH3(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseH3"));  if (!yymatchChar(G, '<')) goto l1038;  if (!yy_Spnl(G)) { goto l1038; }  if (!yymatchChar(G, '/')) goto l1038;
  {  int yypos1039= G->pos, yythunkpos1039= G->thunkpos;  if (!yymatchString(G, "h3")) goto l1040;  goto l1039;
  l1040:;	  G->pos= yypos1039; G->thunkpos= yythunkpos1039;  if (!yymatchString(G, "H3")) goto l1038;
  }
  l1039:;	  if (!yy_Spnl(G)) { goto l1038; }  if (!yymatchChar(G, '>')) goto l1038;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseH3", G->buf+G->pos));
  return 1;
  l1038:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseH3", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenH3(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenH3"));  if (!yymatchChar(G, '<')) goto l1041;  if (!yy_Spnl(G)) { goto l1041; }
  {  int yypos1042= G->pos, yythunkpos1042= G->thunkpos;  if (!yymatchString(G, "h3")) goto l1043;  goto l1042;
  l1043:;	  G->pos= yypos1042; G->thunkpos= yythunkpos1042;  if (!yymatchString(G, "H3")) goto l1041;
  }
  l1042:;	  if (!yy_Spnl(G)) { goto l1041; }
  l1044:;	
  {  int yypos1045= G->pos, yythunkpos1045= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l1045; }  goto l1044;
  l1045:;	  G->pos= yypos1045; G->thunkpos= yythunkpos1045;
  }  if (!yymatchChar(G, '>')) goto l1041;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenH3", G->buf+G->pos));
  return 1;
  l1041:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenH3", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockH2(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "HtmlBlockH2"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l1046;  if (!yy_LocMarker(G)) { goto l1046; }  yyDo(G, yySet, -1, 0);  if (!yy_HtmlBlockOpenH2(G)) { goto l1046; }
  l1047:;	
  {  int yypos1048= G->pos, yythunkpos1048= G->thunkpos;
  {  int yypos1049= G->pos, yythunkpos1049= G->thunkpos;  if (!yy_HtmlBlockH2(G)) { goto l1050; }  goto l1049;
  l1050:;	  G->pos= yypos1049; G->thunkpos= yythunkpos1049;
  {  int yypos1051= G->pos, yythunkpos1051= G->thunkpos;  if (!yy_HtmlBlockCloseH2(G)) { goto l1051; }  goto l1048;
  l1051:;	  G->pos= yypos1051; G->thunkpos= yythunkpos1051;
  }  if (!yymatchDot(G)) goto l1048;
  }
  l1049:;	  goto l1047;
  l1048:;	  G->pos= yypos1048; G->thunkpos= yythunkpos1048;
  }  if (!yy_HtmlBlockCloseH2(G)) { goto l1046; }  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l1046;  yyDo(G, yy_1_HtmlBlockH2, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockH2", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l1046:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockH2", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseH2(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseH2"));  if (!yymatchChar(G, '<')) goto l1052;  if (!yy_Spnl(G)) { goto l1052; }  if (!yymatchChar(G, '/')) goto l1052;
  {  int yypos1053= G->pos, yythunkpos1053= G->thunkpos;  if (!yymatchString(G, "h2")) goto l1054;  goto l1053;
  l1054:;	  G->pos= yypos1053; G->thunkpos= yythunkpos1053;  if (!yymatchString(G, "H2")) goto l1052;
  }
  l1053:;	  if (!yy_Spnl(G)) { goto l1052; }  if (!yymatchChar(G, '>')) goto l1052;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseH2", G->buf+G->pos));
  return 1;
  l1052:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseH2", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenH2(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenH2"));  if (!yymatchChar(G, '<')) goto l1055;  if (!yy_Spnl(G)) { goto l1055; }
  {  int yypos1056= G->pos, yythunkpos1056= G->thunkpos;  if (!yymatchString(G, "h2")) goto l1057;  goto l1056;
  l1057:;	  G->pos= yypos1056; G->thunkpos= yythunkpos1056;  if (!yymatchString(G, "H2")) goto l1055;
  }
  l1056:;	  if (!yy_Spnl(G)) { goto l1055; }
  l1058:;	
  {  int yypos1059= G->pos, yythunkpos1059= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l1059; }  goto l1058;
  l1059:;	  G->pos= yypos1059; G->thunkpos= yythunkpos1059;
  }  if (!yymatchChar(G, '>')) goto l1055;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenH2", G->buf+G->pos));
  return 1;
  l1055:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenH2", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockH1(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "HtmlBlockH1"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l1060;  if (!yy_LocMarker(G)) { goto l1060; }  yyDo(G, yySet, -1, 0);  if (!yy_HtmlBlockOpenH1(G)) { goto l1060; }
  l1061:;	
  {  int yypos1062= G->pos, yythunkpos1062= G->thunkpos;
  {  int yypos1063= G->pos, yythunkpos1063= G->thunkpos;  if (!yy_HtmlBlockH1(G)) { goto l1064; }  goto l1063;
  l1064:;	  G->pos= yypos1063; G->thunkpos= yythunkpos1063;
  {  int yypos1065= G->pos, yythunkpos1065= G->thunkpos;  if (!yy_HtmlBlockCloseH1(G)) { goto l1065; }  goto l1062;
  l1065:;	  G->pos= yypos1065; G->thunkpos= yythunkpos1065;
  }  if (!yymatchDot(G)) goto l1062;
  }
  l1063:;	  goto l1061;
  l1062:;	  G->pos= yypos1062; G->thunkpos= yythunkpos1062;
  }  if (!yy_HtmlBlockCloseH1(G)) { goto l1060; }  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l1060;  yyDo(G, yy_1_HtmlBlockH1, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockH1", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l1060:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockH1", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseH1(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseH1"));  if (!yymatchChar(G, '<')) goto l1066;  if (!yy_Spnl(G)) { goto l1066; }  if (!yymatchChar(G, '/')) goto l1066;
  {  int yypos1067= G->pos, yythunkpos1067= G->thunkpos;  if (!yymatchString(G, "h1")) goto l1068;  goto l1067;
  l1068:;	  G->pos= yypos1067; G->thunkpos= yythunkpos1067;  if (!yymatchString(G, "H1")) goto l1066;
  }
  l1067:;	  if (!yy_Spnl(G)) { goto l1066; }  if (!yymatchChar(G, '>')) goto l1066;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseH1", G->buf+G->pos));
  return 1;
  l1066:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseH1", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenH1(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenH1"));  if (!yymatchChar(G, '<')) goto l1069;  if (!yy_Spnl(G)) { goto l1069; }
  {  int yypos1070= G->pos, yythunkpos1070= G->thunkpos;  if (!yymatchString(G, "h1")) goto l1071;  goto l1070;
  l1071:;	  G->pos= yypos1070; G->thunkpos= yythunkpos1070;  if (!yymatchString(G, "H1")) goto l1069;
  }
  l1070:;	  if (!yy_Spnl(G)) { goto l1069; }
  l1072:;	
  {  int yypos1073= G->pos, yythunkpos1073= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l1073; }  goto l1072;
  l1073:;	  G->pos= yypos1073; G->thunkpos= yythunkpos1073;
  }  if (!yymatchChar(G, '>')) goto l1069;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenH1", G->buf+G->pos));
  return 1;
  l1069:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenH1", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockForm(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockForm"));  if (!yy_HtmlBlockOpenForm(G)) { goto l1074; }
  l1075:;	
  {  int yypos1076= G->pos, yythunkpos1076= G->thunkpos;
  {  int yypos1077= G->pos, yythunkpos1077= G->thunkpos;  if (!yy_HtmlBlockForm(G)) { goto l1078; }  goto l1077;
  l1078:;	  G->pos= yypos1077; G->thunkpos= yythunkpos1077;
  {  int yypos1079= G->pos, yythunkpos1079= G->thunkpos;  if (!yy_HtmlBlockCloseForm(G)) { goto l1079; }  goto l1076;
  l1079:;	  G->pos= yypos1079; G->thunkpos= yythunkpos1079;
  }  if (!yymatchDot(G)) goto l1076;
  }
  l1077:;	  goto l1075;
  l1076:;	  G->pos= yypos1076; G->thunkpos= yythunkpos1076;
  }  if (!yy_HtmlBlockCloseForm(G)) { goto l1074; }
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockForm", G->buf+G->pos));
  return 1;
  l1074:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockForm", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseForm(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseForm"));  if (!yymatchChar(G, '<')) goto l1080;  if (!yy_Spnl(G)) { goto l1080; }  if (!yymatchChar(G, '/')) goto l1080;
  {  int yypos1081= G->pos, yythunkpos1081= G->thunkpos;  if (!yymatchString(G, "form")) goto l1082;  goto l1081;
  l1082:;	  G->pos= yypos1081; G->thunkpos= yythunkpos1081;  if (!yymatchString(G, "FORM")) goto l1080;
  }
  l1081:;	  if (!yy_Spnl(G)) { goto l1080; }  if (!yymatchChar(G, '>')) goto l1080;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseForm", G->buf+G->pos));
  return 1;
  l1080:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseForm", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenForm(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenForm"));  if (!yymatchChar(G, '<')) goto l1083;  if (!yy_Spnl(G)) { goto l1083; }
  {  int yypos1084= G->pos, yythunkpos1084= G->thunkpos;  if (!yymatchString(G, "form")) goto l1085;  goto l1084;
  l1085:;	  G->pos= yypos1084; G->thunkpos= yythunkpos1084;  if (!yymatchString(G, "FORM")) goto l1083;
  }
  l1084:;	  if (!yy_Spnl(G)) { goto l1083; }
  l1086:;	
  {  int yypos1087= G->pos, yythunkpos1087= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l1087; }  goto l1086;
  l1087:;	  G->pos= yypos1087; G->thunkpos= yythunkpos1087;
  }  if (!yymatchChar(G, '>')) goto l1083;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenForm", G->buf+G->pos));
  return 1;
  l1083:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenForm", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockFieldset(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockFieldset"));  if (!yy_HtmlBlockOpenFieldset(G)) { goto l1088; }
  l1089:;	
  {  int yypos1090= G->pos, yythunkpos1090= G->thunkpos;
  {  int yypos1091= G->pos, yythunkpos1091= G->thunkpos;  if (!yy_HtmlBlockFieldset(G)) { goto l1092; }  goto l1091;
  l1092:;	  G->pos= yypos1091; G->thunkpos= yythunkpos1091;
  {  int yypos1093= G->pos, yythunkpos1093= G->thunkpos;  if (!yy_HtmlBlockCloseFieldset(G)) { goto l1093; }  goto l1090;
  l1093:;	  G->pos= yypos1093; G->thunkpos= yythunkpos1093;
  }  if (!yymatchDot(G)) goto l1090;
  }
  l1091:;	  goto l1089;
  l1090:;	  G->pos= yypos1090; G->thunkpos= yythunkpos1090;
  }  if (!yy_HtmlBlockCloseFieldset(G)) { goto l1088; }
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockFieldset", G->buf+G->pos));
  return 1;
  l1088:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockFieldset", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseFieldset(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseFieldset"));  if (!yymatchChar(G, '<')) goto l1094;  if (!yy_Spnl(G)) { goto l1094; }  if (!yymatchChar(G, '/')) goto l1094;
  {  int yypos1095= G->pos, yythunkpos1095= G->thunkpos;  if (!yymatchString(G, "fieldset")) goto l1096;  goto l1095;
  l1096:;	  G->pos= yypos1095; G->thunkpos= yythunkpos1095;  if (!yymatchString(G, "FIELDSET")) goto l1094;
  }
  l1095:;	  if (!yy_Spnl(G)) { goto l1094; }  if (!yymatchChar(G, '>')) goto l1094;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseFieldset", G->buf+G->pos));
  return 1;
  l1094:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseFieldset", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenFieldset(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenFieldset"));  if (!yymatchChar(G, '<')) goto l1097;  if (!yy_Spnl(G)) { goto l1097; }
  {  int yypos1098= G->pos, yythunkpos1098= G->thunkpos;  if (!yymatchString(G, "fieldset")) goto l1099;  goto l1098;
  l1099:;	  G->pos= yypos1098; G->thunkpos= yythunkpos1098;  if (!yymatchString(G, "FIELDSET")) goto l1097;
  }
  l1098:;	  if (!yy_Spnl(G)) { goto l1097; }
  l1100:;	
  {  int yypos1101= G->pos, yythunkpos1101= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l1101; }  goto l1100;
  l1101:;	  G->pos= yypos1101; G->thunkpos= yythunkpos1101;
  }  if (!yymatchChar(G, '>')) goto l1097;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenFieldset", G->buf+G->pos));
  return 1;
  l1097:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenFieldset", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockDl(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockDl"));  if (!yy_HtmlBlockOpenDl(G)) { goto l1102; }
  l1103:;	
  {  int yypos1104= G->pos, yythunkpos1104= G->thunkpos;
  {  int yypos1105= G->pos, yythunkpos1105= G->thunkpos;  if (!yy_HtmlBlockDl(G)) { goto l1106; }  goto l1105;
  l1106:;	  G->pos= yypos1105; G->thunkpos= yythunkpos1105;
  {  int yypos1107= G->pos, yythunkpos1107= G->thunkpos;  if (!yy_HtmlBlockCloseDl(G)) { goto l1107; }  goto l1104;
  l1107:;	  G->pos= yypos1107; G->thunkpos= yythunkpos1107;
  }  if (!yymatchDot(G)) goto l1104;
  }
  l1105:;	  goto l1103;
  l1104:;	  G->pos= yypos1104; G->thunkpos= yythunkpos1104;
  }  if (!yy_HtmlBlockCloseDl(G)) { goto l1102; }
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockDl", G->buf+G->pos));
  return 1;
  l1102:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockDl", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseDl(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseDl"));  if (!yymatchChar(G, '<')) goto l1108;  if (!yy_Spnl(G)) { goto l1108; }  if (!yymatchChar(G, '/')) goto l1108;
  {  int yypos1109= G->pos, yythunkpos1109= G->thunkpos;  if (!yymatchString(G, "dl")) goto l1110;  goto l1109;
  l1110:;	  G->pos= yypos1109; G->thunkpos= yythunkpos1109;  if (!yymatchString(G, "DL")) goto l1108;
  }
  l1109:;	  if (!yy_Spnl(G)) { goto l1108; }  if (!yymatchChar(G, '>')) goto l1108;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseDl", G->buf+G->pos));
  return 1;
  l1108:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseDl", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenDl(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenDl"));  if (!yymatchChar(G, '<')) goto l1111;  if (!yy_Spnl(G)) { goto l1111; }
  {  int yypos1112= G->pos, yythunkpos1112= G->thunkpos;  if (!yymatchString(G, "dl")) goto l1113;  goto l1112;
  l1113:;	  G->pos= yypos1112; G->thunkpos= yythunkpos1112;  if (!yymatchString(G, "DL")) goto l1111;
  }
  l1112:;	  if (!yy_Spnl(G)) { goto l1111; }
  l1114:;	
  {  int yypos1115= G->pos, yythunkpos1115= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l1115; }  goto l1114;
  l1115:;	  G->pos= yypos1115; G->thunkpos= yythunkpos1115;
  }  if (!yymatchChar(G, '>')) goto l1111;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenDl", G->buf+G->pos));
  return 1;
  l1111:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenDl", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockDiv(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockDiv"));  if (!yy_HtmlBlockOpenDiv(G)) { goto l1116; }
  l1117:;	
  {  int yypos1118= G->pos, yythunkpos1118= G->thunkpos;
  {  int yypos1119= G->pos, yythunkpos1119= G->thunkpos;  if (!yy_HtmlBlockDiv(G)) { goto l1120; }  goto l1119;
  l1120:;	  G->pos= yypos1119; G->thunkpos= yythunkpos1119;
  {  int yypos1121= G->pos, yythunkpos1121= G->thunkpos;  if (!yy_HtmlBlockCloseDiv(G)) { goto l1121; }  goto l1118;
  l1121:;	  G->pos= yypos1121; G->thunkpos= yythunkpos1121;
  }  if (!yymatchDot(G)) goto l1118;
  }
  l1119:;	  goto l1117;
  l1118:;	  G->pos= yypos1118; G->thunkpos= yythunkpos1118;
  }  if (!yy_HtmlBlockCloseDiv(G)) { goto l1116; }
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockDiv", G->buf+G->pos));
  return 1;
  l1116:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockDiv", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseDiv(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseDiv"));  if (!yymatchChar(G, '<')) goto l1122;  if (!yy_Spnl(G)) { goto l1122; }  if (!yymatchChar(G, '/')) goto l1122;
  {  int yypos1123= G->pos, yythunkpos1123= G->thunkpos;  if (!yymatchString(G, "div")) goto l1124;  goto l1123;
  l1124:;	  G->pos= yypos1123; G->thunkpos= yythunkpos1123;  if (!yymatchString(G, "DIV")) goto l1122;
  }
  l1123:;	  if (!yy_Spnl(G)) { goto l1122; }  if (!yymatchChar(G, '>')) goto l1122;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseDiv", G->buf+G->pos));
  return 1;
  l1122:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseDiv", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenDiv(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenDiv"));  if (!yymatchChar(G, '<')) goto l1125;  if (!yy_Spnl(G)) { goto l1125; }
  {  int yypos1126= G->pos, yythunkpos1126= G->thunkpos;  if (!yymatchString(G, "div")) goto l1127;  goto l1126;
  l1127:;	  G->pos= yypos1126; G->thunkpos= yythunkpos1126;  if (!yymatchString(G, "DIV")) goto l1125;
  }
  l1126:;	  if (!yy_Spnl(G)) { goto l1125; }
  l1128:;	
  {  int yypos1129= G->pos, yythunkpos1129= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l1129; }  goto l1128;
  l1129:;	  G->pos= yypos1129; G->thunkpos= yythunkpos1129;
  }  if (!yymatchChar(G, '>')) goto l1125;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenDiv", G->buf+G->pos));
  return 1;
  l1125:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenDiv", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockDir(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockDir"));  if (!yy_HtmlBlockOpenDir(G)) { goto l1130; }
  l1131:;	
  {  int yypos1132= G->pos, yythunkpos1132= G->thunkpos;
  {  int yypos1133= G->pos, yythunkpos1133= G->thunkpos;  if (!yy_HtmlBlockDir(G)) { goto l1134; }  goto l1133;
  l1134:;	  G->pos= yypos1133; G->thunkpos= yythunkpos1133;
  {  int yypos1135= G->pos, yythunkpos1135= G->thunkpos;  if (!yy_HtmlBlockCloseDir(G)) { goto l1135; }  goto l1132;
  l1135:;	  G->pos= yypos1135; G->thunkpos= yythunkpos1135;
  }  if (!yymatchDot(G)) goto l1132;
  }
  l1133:;	  goto l1131;
  l1132:;	  G->pos= yypos1132; G->thunkpos= yythunkpos1132;
  }  if (!yy_HtmlBlockCloseDir(G)) { goto l1130; }
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockDir", G->buf+G->pos));
  return 1;
  l1130:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockDir", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseDir(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseDir"));  if (!yymatchChar(G, '<')) goto l1136;  if (!yy_Spnl(G)) { goto l1136; }  if (!yymatchChar(G, '/')) goto l1136;
  {  int yypos1137= G->pos, yythunkpos1137= G->thunkpos;  if (!yymatchString(G, "dir")) goto l1138;  goto l1137;
  l1138:;	  G->pos= yypos1137; G->thunkpos= yythunkpos1137;  if (!yymatchString(G, "DIR")) goto l1136;
  }
  l1137:;	  if (!yy_Spnl(G)) { goto l1136; }  if (!yymatchChar(G, '>')) goto l1136;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseDir", G->buf+G->pos));
  return 1;
  l1136:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseDir", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenDir(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenDir"));  if (!yymatchChar(G, '<')) goto l1139;  if (!yy_Spnl(G)) { goto l1139; }
  {  int yypos1140= G->pos, yythunkpos1140= G->thunkpos;  if (!yymatchString(G, "dir")) goto l1141;  goto l1140;
  l1141:;	  G->pos= yypos1140; G->thunkpos= yythunkpos1140;  if (!yymatchString(G, "DIR")) goto l1139;
  }
  l1140:;	  if (!yy_Spnl(G)) { goto l1139; }
  l1142:;	
  {  int yypos1143= G->pos, yythunkpos1143= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l1143; }  goto l1142;
  l1143:;	  G->pos= yypos1143; G->thunkpos= yythunkpos1143;
  }  if (!yymatchChar(G, '>')) goto l1139;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenDir", G->buf+G->pos));
  return 1;
  l1139:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenDir", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCenter(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCenter"));  if (!yy_HtmlBlockOpenCenter(G)) { goto l1144; }
  l1145:;	
  {  int yypos1146= G->pos, yythunkpos1146= G->thunkpos;
  {  int yypos1147= G->pos, yythunkpos1147= G->thunkpos;  if (!yy_HtmlBlockCenter(G)) { goto l1148; }  goto l1147;
  l1148:;	  G->pos= yypos1147; G->thunkpos= yythunkpos1147;
  {  int yypos1149= G->pos, yythunkpos1149= G->thunkpos;  if (!yy_HtmlBlockCloseCenter(G)) { goto l1149; }  goto l1146;
  l1149:;	  G->pos= yypos1149; G->thunkpos= yythunkpos1149;
  }  if (!yymatchDot(G)) goto l1146;
  }
  l1147:;	  goto l1145;
  l1146:;	  G->pos= yypos1146; G->thunkpos= yythunkpos1146;
  }  if (!yy_HtmlBlockCloseCenter(G)) { goto l1144; }
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCenter", G->buf+G->pos));
  return 1;
  l1144:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCenter", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseCenter(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseCenter"));  if (!yymatchChar(G, '<')) goto l1150;  if (!yy_Spnl(G)) { goto l1150; }  if (!yymatchChar(G, '/')) goto l1150;
  {  int yypos1151= G->pos, yythunkpos1151= G->thunkpos;  if (!yymatchString(G, "center")) goto l1152;  goto l1151;
  l1152:;	  G->pos= yypos1151; G->thunkpos= yythunkpos1151;  if (!yymatchString(G, "CENTER")) goto l1150;
  }
  l1151:;	  if (!yy_Spnl(G)) { goto l1150; }  if (!yymatchChar(G, '>')) goto l1150;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseCenter", G->buf+G->pos));
  return 1;
  l1150:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseCenter", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenCenter(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenCenter"));  if (!yymatchChar(G, '<')) goto l1153;  if (!yy_Spnl(G)) { goto l1153; }
  {  int yypos1154= G->pos, yythunkpos1154= G->thunkpos;  if (!yymatchString(G, "center")) goto l1155;  goto l1154;
  l1155:;	  G->pos= yypos1154; G->thunkpos= yythunkpos1154;  if (!yymatchString(G, "CENTER")) goto l1153;
  }
  l1154:;	  if (!yy_Spnl(G)) { goto l1153; }
  l1156:;	
  {  int yypos1157= G->pos, yythunkpos1157= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l1157; }  goto l1156;
  l1157:;	  G->pos= yypos1157; G->thunkpos= yythunkpos1157;
  }  if (!yymatchChar(G, '>')) goto l1153;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenCenter", G->buf+G->pos));
  return 1;
  l1153:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenCenter", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockBlockquote(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockBlockquote"));  if (!yy_HtmlBlockOpenBlockquote(G)) { goto l1158; }
  l1159:;	
  {  int yypos1160= G->pos, yythunkpos1160= G->thunkpos;
  {  int yypos1161= G->pos, yythunkpos1161= G->thunkpos;  if (!yy_HtmlBlockBlockquote(G)) { goto l1162; }  goto l1161;
  l1162:;	  G->pos= yypos1161; G->thunkpos= yythunkpos1161;
  {  int yypos1163= G->pos, yythunkpos1163= G->thunkpos;  if (!yy_HtmlBlockCloseBlockquote(G)) { goto l1163; }  goto l1160;
  l1163:;	  G->pos= yypos1163; G->thunkpos= yythunkpos1163;
  }  if (!yymatchDot(G)) goto l1160;
  }
  l1161:;	  goto l1159;
  l1160:;	  G->pos= yypos1160; G->thunkpos= yythunkpos1160;
  }  if (!yy_HtmlBlockCloseBlockquote(G)) { goto l1158; }
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockBlockquote", G->buf+G->pos));
  return 1;
  l1158:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockBlockquote", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseBlockquote(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseBlockquote"));  if (!yymatchChar(G, '<')) goto l1164;  if (!yy_Spnl(G)) { goto l1164; }  if (!yymatchChar(G, '/')) goto l1164;
  {  int yypos1165= G->pos, yythunkpos1165= G->thunkpos;  if (!yymatchString(G, "blockquote")) goto l1166;  goto l1165;
  l1166:;	  G->pos= yypos1165; G->thunkpos= yythunkpos1165;  if (!yymatchString(G, "BLOCKQUOTE")) goto l1164;
  }
  l1165:;	  if (!yy_Spnl(G)) { goto l1164; }  if (!yymatchChar(G, '>')) goto l1164;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseBlockquote", G->buf+G->pos));
  return 1;
  l1164:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseBlockquote", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenBlockquote(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenBlockquote"));  if (!yymatchChar(G, '<')) goto l1167;  if (!yy_Spnl(G)) { goto l1167; }
  {  int yypos1168= G->pos, yythunkpos1168= G->thunkpos;  if (!yymatchString(G, "blockquote")) goto l1169;  goto l1168;
  l1169:;	  G->pos= yypos1168; G->thunkpos= yythunkpos1168;  if (!yymatchString(G, "BLOCKQUOTE")) goto l1167;
  }
  l1168:;	  if (!yy_Spnl(G)) { goto l1167; }
  l1170:;	
  {  int yypos1171= G->pos, yythunkpos1171= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l1171; }  goto l1170;
  l1171:;	  G->pos= yypos1171; G->thunkpos= yythunkpos1171;
  }  if (!yymatchChar(G, '>')) goto l1167;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenBlockquote", G->buf+G->pos));
  return 1;
  l1167:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenBlockquote", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockAddress(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockAddress"));  if (!yy_HtmlBlockOpenAddress(G)) { goto l1172; }
  l1173:;	
  {  int yypos1174= G->pos, yythunkpos1174= G->thunkpos;
  {  int yypos1175= G->pos, yythunkpos1175= G->thunkpos;  if (!yy_HtmlBlockAddress(G)) { goto l1176; }  goto l1175;
  l1176:;	  G->pos= yypos1175; G->thunkpos= yythunkpos1175;
  {  int yypos1177= G->pos, yythunkpos1177= G->thunkpos;  if (!yy_HtmlBlockCloseAddress(G)) { goto l1177; }  goto l1174;
  l1177:;	  G->pos= yypos1177; G->thunkpos= yythunkpos1177;
  }  if (!yymatchDot(G)) goto l1174;
  }
  l1175:;	  goto l1173;
  l1174:;	  G->pos= yypos1174; G->thunkpos= yythunkpos1174;
  }  if (!yy_HtmlBlockCloseAddress(G)) { goto l1172; }
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockAddress", G->buf+G->pos));
  return 1;
  l1172:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockAddress", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseAddress(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseAddress"));  if (!yymatchChar(G, '<')) goto l1178;  if (!yy_Spnl(G)) { goto l1178; }  if (!yymatchChar(G, '/')) goto l1178;
  {  int yypos1179= G->pos, yythunkpos1179= G->thunkpos;  if (!yymatchString(G, "address")) goto l1180;  goto l1179;
  l1180:;	  G->pos= yypos1179; G->thunkpos= yythunkpos1179;  if (!yymatchString(G, "ADDRESS")) goto l1178;
  }
  l1179:;	  if (!yy_Spnl(G)) { goto l1178; }  if (!yymatchChar(G, '>')) goto l1178;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseAddress", G->buf+G->pos));
  return 1;
  l1178:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseAddress", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlAttribute(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlAttribute"));
  {  int yypos1184= G->pos, yythunkpos1184= G->thunkpos;  if (!yy_AlphanumericAscii(G)) { goto l1185; }  goto l1184;
  l1185:;	  G->pos= yypos1184; G->thunkpos= yythunkpos1184;  if (!yymatchChar(G, '-')) goto l1181;
  }
  l1184:;	
  l1182:;	
  {  int yypos1183= G->pos, yythunkpos1183= G->thunkpos;
  {  int yypos1186= G->pos, yythunkpos1186= G->thunkpos;  if (!yy_AlphanumericAscii(G)) { goto l1187; }  goto l1186;
  l1187:;	  G->pos= yypos1186; G->thunkpos= yythunkpos1186;  if (!yymatchChar(G, '-')) goto l1183;
  }
  l1186:;	  goto l1182;
  l1183:;	  G->pos= yypos1183; G->thunkpos= yythunkpos1183;
  }  if (!yy_Spnl(G)) { goto l1181; }
  {  int yypos1188= G->pos, yythunkpos1188= G->thunkpos;  if (!yymatchChar(G, '=')) goto l1188;  if (!yy_Spnl(G)) { goto l1188; }
  {  int yypos1190= G->pos, yythunkpos1190= G->thunkpos;  if (!yy_Quoted(G)) { goto l1191; }  goto l1190;
  l1191:;	  G->pos= yypos1190; G->thunkpos= yythunkpos1190;
  {  int yypos1194= G->pos, yythunkpos1194= G->thunkpos;  if (!yymatchChar(G, '>')) goto l1194;  goto l1188;
  l1194:;	  G->pos= yypos1194; G->thunkpos= yythunkpos1194;
  }  if (!yy_Nonspacechar(G)) { goto l1188; }
  l1192:;	
  {  int yypos1193= G->pos, yythunkpos1193= G->thunkpos;
  {  int yypos1195= G->pos, yythunkpos1195= G->thunkpos;  if (!yymatchChar(G, '>')) goto l1195;  goto l1193;
  l1195:;	  G->pos= yypos1195; G->thunkpos= yythunkpos1195;
  }  if (!yy_Nonspacechar(G)) { goto l1193; }  goto l1192;
  l1193:;	  G->pos= yypos1193; G->thunkpos= yythunkpos1193;
  }
  }
  l1190:;	  goto l1189;
  l1188:;	  G->pos= yypos1188; G->thunkpos= yythunkpos1188;
  }
  l1189:;	  if (!yy_Spnl(G)) { goto l1181; }
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlAttribute", G->buf+G->pos));
  return 1;
  l1181:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlAttribute", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Spnl(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Spnl"));  if (!yy_Sp(G)) { goto l1196; }
  {  int yypos1197= G->pos, yythunkpos1197= G->thunkpos;  if (!yy_Newline(G)) { goto l1197; }  if (!yy_Sp(G)) { goto l1197; }  goto l1198;
  l1197:;	  G->pos= yypos1197; G->thunkpos= yythunkpos1197;
  }
  l1198:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "Spnl", G->buf+G->pos));
  return 1;
  l1196:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Spnl", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenAddress(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenAddress"));  if (!yymatchChar(G, '<')) goto l1199;  if (!yy_Spnl(G)) { goto l1199; }
  {  int yypos1200= G->pos, yythunkpos1200= G->thunkpos;  if (!yymatchString(G, "address")) goto l1201;  goto l1200;
  l1201:;	  G->pos= yypos1200; G->thunkpos= yythunkpos1200;  if (!yymatchString(G, "ADDRESS")) goto l1199;
  }
  l1200:;	  if (!yy_Spnl(G)) { goto l1199; }
  l1202:;	
  {  int yypos1203= G->pos, yythunkpos1203= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l1203; }  goto l1202;
  l1203:;	  G->pos= yypos1203; G->thunkpos= yythunkpos1203;
  }  if (!yymatchChar(G, '>')) goto l1199;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenAddress", G->buf+G->pos));
  return 1;
  l1199:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenAddress", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_OptionallyIndentedLine(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "OptionallyIndentedLine"));
  {  int yypos1205= G->pos, yythunkpos1205= G->thunkpos;  if (!yy_Indent(G)) { goto l1205; }  goto l1206;
  l1205:;	  G->pos= yypos1205; G->thunkpos= yythunkpos1205;
  }
  l1206:;	  if (!yy_Line(G)) { goto l1204; }
  yyprintf((stderr, "  ok   %s @ %s\n", "OptionallyIndentedLine", G->buf+G->pos));
  return 1;
  l1204:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "OptionallyIndentedLine", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Indent(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Indent"));
  {  int yypos1208= G->pos, yythunkpos1208= G->thunkpos;  if (!yymatchChar(G, '\t')) goto l1209;  goto l1208;
  l1209:;	  G->pos= yypos1208; G->thunkpos= yythunkpos1208;  if (!yymatchString(G, "    ")) goto l1207;
  }
  l1208:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "Indent", G->buf+G->pos));
  return 1;
  l1207:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Indent", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_ListBlockLine(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "ListBlockLine"));
  {  int yypos1211= G->pos, yythunkpos1211= G->thunkpos;  if (!yy_BlankLine(G)) { goto l1211; }  goto l1210;
  l1211:;	  G->pos= yypos1211; G->thunkpos= yythunkpos1211;
  }
  {  int yypos1212= G->pos, yythunkpos1212= G->thunkpos;
  {  int yypos1213= G->pos, yythunkpos1213= G->thunkpos;  if (!yy_Indent(G)) { goto l1213; }  goto l1214;
  l1213:;	  G->pos= yypos1213; G->thunkpos= yythunkpos1213;
  }
  l1214:;	
  {  int yypos1215= G->pos, yythunkpos1215= G->thunkpos;  if (!yy_Bullet(G)) { goto l1216; }  goto l1215;
  l1216:;	  G->pos= yypos1215; G->thunkpos= yythunkpos1215;  if (!yy_Enumerator(G)) { goto l1212; }
  }
  l1215:;	  goto l1210;
  l1212:;	  G->pos= yypos1212; G->thunkpos= yythunkpos1212;
  }
  {  int yypos1217= G->pos, yythunkpos1217= G->thunkpos;  if (!yy_HorizontalRule(G)) { goto l1217; }  goto l1210;
  l1217:;	  G->pos= yypos1217; G->thunkpos= yythunkpos1217;
  }  if (!yy_OptionallyIndentedLine(G)) { goto l1210; }
  yyprintf((stderr, "  ok   %s @ %s\n", "ListBlockLine", G->buf+G->pos));
  return 1;
  l1210:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "ListBlockLine", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_ListContinuationBlock(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "ListContinuationBlock"));  if (!yy_StartList(G)) { goto l1218; }  yyDo(G, yySet, -1, 0);  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l1218;
  l1219:;	
  {  int yypos1220= G->pos, yythunkpos1220= G->thunkpos;  if (!yy_BlankLine(G)) { goto l1220; }  goto l1219;
  l1220:;	  G->pos= yypos1220; G->thunkpos= yythunkpos1220;
  }  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l1218;  yyDo(G, yy_1_ListContinuationBlock, G->begin, G->end);  if (!yy_Indent(G)) { goto l1218; }  if (!yy_ListBlock(G)) { goto l1218; }  yyDo(G, yy_2_ListContinuationBlock, G->begin, G->end);
  l1221:;	
  {  int yypos1222= G->pos, yythunkpos1222= G->thunkpos;  if (!yy_Indent(G)) { goto l1222; }  if (!yy_ListBlock(G)) { goto l1222; }  yyDo(G, yy_2_ListContinuationBlock, G->begin, G->end);  goto l1221;
  l1222:;	  G->pos= yypos1222; G->thunkpos= yythunkpos1222;
  }  yyDo(G, yy_3_ListContinuationBlock, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "ListContinuationBlock", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l1218:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "ListContinuationBlock", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_ListBlock(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "ListBlock"));  if (!yy_StartList(G)) { goto l1223; }  yyDo(G, yySet, -1, 0);
  {  int yypos1224= G->pos, yythunkpos1224= G->thunkpos;  if (!yy_BlankLine(G)) { goto l1224; }  goto l1223;
  l1224:;	  G->pos= yypos1224; G->thunkpos= yythunkpos1224;
  }  if (!yy_Line(G)) { goto l1223; }  yyDo(G, yy_1_ListBlock, G->begin, G->end);
  l1225:;	
  {  int yypos1226= G->pos, yythunkpos1226= G->thunkpos;  if (!yy_ListBlockLine(G)) { goto l1226; }  yyDo(G, yy_2_ListBlock, G->begin, G->end);  goto l1225;
  l1226:;	  G->pos= yypos1226; G->thunkpos= yythunkpos1226;
  }  yyDo(G, yy_3_ListBlock, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "ListBlock", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l1223:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "ListBlock", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_ListItem(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "ListItem"));
  {  int yypos1228= G->pos, yythunkpos1228= G->thunkpos;  if (!yy_Bullet(G)) { goto l1229; }  goto l1228;
  l1229:;	  G->pos= yypos1228; G->thunkpos= yythunkpos1228;  if (!yy_Enumerator(G)) { goto l1227; }
  }
  l1228:;	  if (!yy_StartList(G)) { goto l1227; }  yyDo(G, yySet, -1, 0);  if (!yy_ListBlock(G)) { goto l1227; }  yyDo(G, yy_1_ListItem, G->begin, G->end);
  l1230:;	
  {  int yypos1231= G->pos, yythunkpos1231= G->thunkpos;  if (!yy_ListContinuationBlock(G)) { goto l1231; }  yyDo(G, yy_2_ListItem, G->begin, G->end);  goto l1230;
  l1231:;	  G->pos= yypos1231; G->thunkpos= yythunkpos1231;
  }  yyDo(G, yy_3_ListItem, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "ListItem", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l1227:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "ListItem", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Enumerator(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Enumerator"));  if (!yy_NonindentSpace(G)) { goto l1232; }  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l1232;  if (!yymatchClass(G, (unsigned char *)"\000\000\000\000\000\000\377\003\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l1232;
  l1233:;	
  {  int yypos1234= G->pos, yythunkpos1234= G->thunkpos;  if (!yymatchClass(G, (unsigned char *)"\000\000\000\000\000\000\377\003\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l1234;  goto l1233;
  l1234:;	  G->pos= yypos1234; G->thunkpos= yythunkpos1234;
  }  if (!yymatchChar(G, '.')) goto l1232;  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l1232;  if (!yy_Spacechar(G)) { goto l1232; }
  l1235:;	
  {  int yypos1236= G->pos, yythunkpos1236= G->thunkpos;  if (!yy_Spacechar(G)) { goto l1236; }  goto l1235;
  l1236:;	  G->pos= yypos1236; G->thunkpos= yythunkpos1236;
  }  yyDo(G, yy_1_Enumerator, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "Enumerator", G->buf+G->pos));
  return 1;
  l1232:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Enumerator", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_ListItemTight(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "ListItemTight"));
  {  int yypos1238= G->pos, yythunkpos1238= G->thunkpos;  if (!yy_Bullet(G)) { goto l1239; }  goto l1238;
  l1239:;	  G->pos= yypos1238; G->thunkpos= yythunkpos1238;  if (!yy_Enumerator(G)) { goto l1237; }
  }
  l1238:;	  if (!yy_StartList(G)) { goto l1237; }  yyDo(G, yySet, -1, 0);  if (!yy_ListBlock(G)) { goto l1237; }  yyDo(G, yy_1_ListItemTight, G->begin, G->end);
  l1240:;	
  {  int yypos1241= G->pos, yythunkpos1241= G->thunkpos;
  {  int yypos1242= G->pos, yythunkpos1242= G->thunkpos;  if (!yy_BlankLine(G)) { goto l1242; }  goto l1241;
  l1242:;	  G->pos= yypos1242; G->thunkpos= yythunkpos1242;
  }  if (!yy_ListContinuationBlock(G)) { goto l1241; }  yyDo(G, yy_2_ListItemTight, G->begin, G->end);  goto l1240;
  l1241:;	  G->pos= yypos1241; G->thunkpos= yythunkpos1241;
  }
  {  int yypos1243= G->pos, yythunkpos1243= G->thunkpos;  if (!yy_ListContinuationBlock(G)) { goto l1243; }  goto l1237;
  l1243:;	  G->pos= yypos1243; G->thunkpos= yythunkpos1243;
  }  yyDo(G, yy_3_ListItemTight, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "ListItemTight", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l1237:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "ListItemTight", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_ListLoose(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 2, 0);
  yyprintf((stderr, "%s\n", "ListLoose"));  if (!yy_StartList(G)) { goto l1244; }  yyDo(G, yySet, -2, 0);  if (!yy_ListItem(G)) { goto l1244; }  yyDo(G, yySet, -1, 0);
  l1247:;	
  {  int yypos1248= G->pos, yythunkpos1248= G->thunkpos;  if (!yy_BlankLine(G)) { goto l1248; }  goto l1247;
  l1248:;	  G->pos= yypos1248; G->thunkpos= yythunkpos1248;
  }  yyDo(G, yy_1_ListLoose, G->begin, G->end);
  l1245:;	
  {  int yypos1246= G->pos, yythunkpos1246= G->thunkpos;  if (!yy_ListItem(G)) { goto l1246; }  yyDo(G, yySet, -1, 0);
  l1249:;	
  {  int yypos1250= G->pos, yythunkpos1250= G->thunkpos;  if (!yy_BlankLine(G)) { goto l1250; }  goto l1249;
  l1250:;	  G->pos= yypos1250; G->thunkpos= yythunkpos1250;
  }  yyDo(G, yy_1_ListLoose, G->begin, G->end);  goto l1245;
  l1246:;	  G->pos= yypos1246; G->thunkpos= yythunkpos1246;
  }  yyDo(G, yy_2_ListLoose, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "ListLoose", G->buf+G->pos));  yyDo(G, yyPop, 2, 0);
  return 1;
  l1244:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "ListLoose", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_ListTight(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "ListTight"));  if (!yy_StartList(G)) { goto l1251; }  yyDo(G, yySet, -1, 0);  if (!yy_ListItemTight(G)) { goto l1251; }  yyDo(G, yy_1_ListTight, G->begin, G->end);
  l1252:;	
  {  int yypos1253= G->pos, yythunkpos1253= G->thunkpos;  if (!yy_ListItemTight(G)) { goto l1253; }  yyDo(G, yy_1_ListTight, G->begin, G->end);  goto l1252;
  l1253:;	  G->pos= yypos1253; G->thunkpos= yythunkpos1253;
  }
  l1254:;	
  {  int yypos1255= G->pos, yythunkpos1255= G->thunkpos;  if (!yy_BlankLine(G)) { goto l1255; }  goto l1254;
  l1255:;	  G->pos= yypos1255; G->thunkpos= yythunkpos1255;
  }
  {  int yypos1256= G->pos, yythunkpos1256= G->thunkpos;
  {  int yypos1257= G->pos, yythunkpos1257= G->thunkpos;  if (!yy_Bullet(G)) { goto l1258; }  goto l1257;
  l1258:;	  G->pos= yypos1257; G->thunkpos= yythunkpos1257;  if (!yy_Enumerator(G)) { goto l1256; }
  }
  l1257:;	  goto l1251;
  l1256:;	  G->pos= yypos1256; G->thunkpos= yythunkpos1256;
  }  yyDo(G, yy_2_ListTight, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "ListTight", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l1251:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "ListTight", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Spacechar(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Spacechar"));
  {  int yypos1260= G->pos, yythunkpos1260= G->thunkpos;  if (!yymatchChar(G, ' ')) goto l1261;  goto l1260;
  l1261:;	  G->pos= yypos1260; G->thunkpos= yythunkpos1260;  if (!yymatchChar(G, '\t')) goto l1259;
  }
  l1260:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "Spacechar", G->buf+G->pos));
  return 1;
  l1259:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Spacechar", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Bullet(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Bullet"));
  {  int yypos1263= G->pos, yythunkpos1263= G->thunkpos;  if (!yy_HorizontalRule(G)) { goto l1263; }  goto l1262;
  l1263:;	  G->pos= yypos1263; G->thunkpos= yythunkpos1263;
  }  if (!yy_NonindentSpace(G)) { goto l1262; }  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l1262;
  {  int yypos1264= G->pos, yythunkpos1264= G->thunkpos;  if (!yymatchChar(G, '+')) goto l1265;  goto l1264;
  l1265:;	  G->pos= yypos1264; G->thunkpos= yythunkpos1264;  if (!yymatchChar(G, '*')) goto l1266;  goto l1264;
  l1266:;	  G->pos= yypos1264; G->thunkpos= yythunkpos1264;  if (!yymatchChar(G, '-')) goto l1262;
  }
  l1264:;	  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l1262;  if (!yy_Spacechar(G)) { goto l1262; }
  l1267:;	
  {  int yypos1268= G->pos, yythunkpos1268= G->thunkpos;  if (!yy_Spacechar(G)) { goto l1268; }  goto l1267;
  l1268:;	  G->pos= yypos1268; G->thunkpos= yythunkpos1268;
  }  yyDo(G, yy_1_Bullet, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "Bullet", G->buf+G->pos));
  return 1;
  l1262:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Bullet", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_VerbatimChunk(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "VerbatimChunk"));
  l1270:;	
  {  int yypos1271= G->pos, yythunkpos1271= G->thunkpos;  if (!yy_BlankLine(G)) { goto l1271; }  goto l1270;
  l1271:;	  G->pos= yypos1271; G->thunkpos= yythunkpos1271;
  }  if (!yy_NonblankIndentedLine(G)) { goto l1269; }
  l1272:;	
  {  int yypos1273= G->pos, yythunkpos1273= G->thunkpos;  if (!yy_NonblankIndentedLine(G)) { goto l1273; }  goto l1272;
  l1273:;	  G->pos= yypos1273; G->thunkpos= yythunkpos1273;
  }
  yyprintf((stderr, "  ok   %s @ %s\n", "VerbatimChunk", G->buf+G->pos));
  return 1;
  l1269:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "VerbatimChunk", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_IndentedLine(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "IndentedLine"));  if (!yy_Indent(G)) { goto l1274; }  if (!yy_Line(G)) { goto l1274; }
  yyprintf((stderr, "  ok   %s @ %s\n", "IndentedLine", G->buf+G->pos));
  return 1;
  l1274:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "IndentedLine", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_NonblankIndentedLine(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "NonblankIndentedLine"));
  {  int yypos1276= G->pos, yythunkpos1276= G->thunkpos;  if (!yy_BlankLine(G)) { goto l1276; }  goto l1275;
  l1276:;	  G->pos= yypos1276; G->thunkpos= yythunkpos1276;
  }  if (!yy_IndentedLine(G)) { goto l1275; }
  yyprintf((stderr, "  ok   %s @ %s\n", "NonblankIndentedLine", G->buf+G->pos));
  return 1;
  l1275:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "NonblankIndentedLine", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Line(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Line"));  if (!yy_RawLine(G)) { goto l1277; }  yyDo(G, yy_1_Line, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "Line", G->buf+G->pos));
  return 1;
  l1277:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Line", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_StartList(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "StartList"));
  {  int yypos1279= G->pos, yythunkpos1279= G->thunkpos;  if (!yymatchDot(G)) goto l1278;  G->pos= yypos1279; G->thunkpos= yythunkpos1279;
  }  yyDo(G, yy_1_StartList, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "StartList", G->buf+G->pos));
  return 1;
  l1278:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "StartList", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_BlockQuoteRaw(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "BlockQuoteRaw"));  if (!yy_StartList(G)) { goto l1280; }  yyDo(G, yySet, -1, 0);  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l1280;  if (!yymatchChar(G, '>')) goto l1280;
  {  int yypos1283= G->pos, yythunkpos1283= G->thunkpos;  if (!yymatchChar(G, ' ')) goto l1283;  goto l1284;
  l1283:;	  G->pos= yypos1283; G->thunkpos= yythunkpos1283;
  }
  l1284:;	  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l1280;  yyDo(G, yy_1_BlockQuoteRaw, G->begin, G->end);  if (!yy_Line(G)) { goto l1280; }  yyDo(G, yy_2_BlockQuoteRaw, G->begin, G->end);
  l1285:;	
  {  int yypos1286= G->pos, yythunkpos1286= G->thunkpos;
  {  int yypos1287= G->pos, yythunkpos1287= G->thunkpos;  if (!yymatchChar(G, '>')) goto l1287;  goto l1286;
  l1287:;	  G->pos= yypos1287; G->thunkpos= yythunkpos1287;
  }
  {  int yypos1288= G->pos, yythunkpos1288= G->thunkpos;  if (!yy_BlankLine(G)) { goto l1288; }  goto l1286;
  l1288:;	  G->pos= yypos1288; G->thunkpos= yythunkpos1288;
  }  if (!yy_Line(G)) { goto l1286; }  yyDo(G, yy_3_BlockQuoteRaw, G->begin, G->end);  goto l1285;
  l1286:;	  G->pos= yypos1286; G->thunkpos= yythunkpos1286;
  }
  l1289:;	
  {  int yypos1290= G->pos, yythunkpos1290= G->thunkpos;  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l1290;  if (!yy_BlankLine(G)) { goto l1290; }  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l1290;  yyDo(G, yy_4_BlockQuoteRaw, G->begin, G->end);  goto l1289;
  l1290:;	  G->pos= yypos1290; G->thunkpos= yythunkpos1290;
  }
  l1281:;	
  {  int yypos1282= G->pos, yythunkpos1282= G->thunkpos;  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l1282;  if (!yymatchChar(G, '>')) goto l1282;
  {  int yypos1291= G->pos, yythunkpos1291= G->thunkpos;  if (!yymatchChar(G, ' ')) goto l1291;  goto l1292;
  l1291:;	  G->pos= yypos1291; G->thunkpos= yythunkpos1291;
  }
  l1292:;	  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l1282;  yyDo(G, yy_1_BlockQuoteRaw, G->begin, G->end);  if (!yy_Line(G)) { goto l1282; }  yyDo(G, yy_2_BlockQuoteRaw, G->begin, G->end);
  l1293:;	
  {  int yypos1294= G->pos, yythunkpos1294= G->thunkpos;
  {  int yypos1295= G->pos, yythunkpos1295= G->thunkpos;  if (!yymatchChar(G, '>')) goto l1295;  goto l1294;
  l1295:;	  G->pos= yypos1295; G->thunkpos= yythunkpos1295;
  }
  {  int yypos1296= G->pos, yythunkpos1296= G->thunkpos;  if (!yy_BlankLine(G)) { goto l1296; }  goto l1294;
  l1296:;	  G->pos= yypos1296; G->thunkpos= yythunkpos1296;
  }  if (!yy_Line(G)) { goto l1294; }  yyDo(G, yy_3_BlockQuoteRaw, G->begin, G->end);  goto l1293;
  l1294:;	  G->pos= yypos1294; G->thunkpos= yythunkpos1294;
  }
  l1297:;	
  {  int yypos1298= G->pos, yythunkpos1298= G->thunkpos;  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l1298;  if (!yy_BlankLine(G)) { goto l1298; }  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l1298;  yyDo(G, yy_4_BlockQuoteRaw, G->begin, G->end);  goto l1297;
  l1298:;	  G->pos= yypos1298; G->thunkpos= yythunkpos1298;
  }  goto l1281;
  l1282:;	  G->pos= yypos1282; G->thunkpos= yythunkpos1282;
  }  yyDo(G, yy_5_BlockQuoteRaw, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "BlockQuoteRaw", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l1280:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "BlockQuoteRaw", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Endline(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Endline"));
  {  int yypos1300= G->pos, yythunkpos1300= G->thunkpos;  if (!yy_LineBreak(G)) { goto l1301; }  goto l1300;
  l1301:;	  G->pos= yypos1300; G->thunkpos= yythunkpos1300;  if (!yy_TerminalEndline(G)) { goto l1302; }  goto l1300;
  l1302:;	  G->pos= yypos1300; G->thunkpos= yythunkpos1300;  if (!yy_NormalEndline(G)) { goto l1299; }
  }
  l1300:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "Endline", G->buf+G->pos));
  return 1;
  l1299:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Endline", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_RawLine(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "RawLine"));
  {  int yypos1304= G->pos, yythunkpos1304= G->thunkpos;  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l1305;
  l1306:;	
  {  int yypos1307= G->pos, yythunkpos1307= G->thunkpos;
  {  int yypos1308= G->pos, yythunkpos1308= G->thunkpos;  if (!yymatchChar(G, '\r')) goto l1308;  goto l1307;
  l1308:;	  G->pos= yypos1308; G->thunkpos= yythunkpos1308;
  }
  {  int yypos1309= G->pos, yythunkpos1309= G->thunkpos;  if (!yymatchChar(G, '\n')) goto l1309;  goto l1307;
  l1309:;	  G->pos= yypos1309; G->thunkpos= yythunkpos1309;
  }  if (!yymatchDot(G)) goto l1307;  goto l1306;
  l1307:;	  G->pos= yypos1307; G->thunkpos= yythunkpos1307;
  }  if (!yy_Newline(G)) { goto l1305; }  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l1305;  goto l1304;
  l1305:;	  G->pos= yypos1304; G->thunkpos= yythunkpos1304;  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l1303;  if (!yymatchDot(G)) goto l1303;
  l1310:;	
  {  int yypos1311= G->pos, yythunkpos1311= G->thunkpos;  if (!yymatchDot(G)) goto l1311;  goto l1310;
  l1311:;	  G->pos= yypos1311; G->thunkpos= yythunkpos1311;
  }  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l1303;  if (!yy_Eof(G)) { goto l1303; }
  }
  l1304:;	  yyDo(G, yy_1_RawLine, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "RawLine", G->buf+G->pos));
  return 1;
  l1303:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "RawLine", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_SetextBottom2(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "SetextBottom2"));  if (!yymatchString(G, "---")) goto l1312;
  l1313:;	
  {  int yypos1314= G->pos, yythunkpos1314= G->thunkpos;  if (!yymatchChar(G, '-')) goto l1314;  goto l1313;
  l1314:;	  G->pos= yypos1314; G->thunkpos= yythunkpos1314;
  }  if (!yy_Newline(G)) { goto l1312; }
  yyprintf((stderr, "  ok   %s @ %s\n", "SetextBottom2", G->buf+G->pos));
  return 1;
  l1312:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "SetextBottom2", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_SetextBottom1(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "SetextBottom1"));  if (!yymatchString(G, "===")) goto l1315;
  l1316:;	
  {  int yypos1317= G->pos, yythunkpos1317= G->thunkpos;  if (!yymatchChar(G, '=')) goto l1317;  goto l1316;
  l1317:;	  G->pos= yypos1317; G->thunkpos= yythunkpos1317;
  }  if (!yy_Newline(G)) { goto l1315; }
  yyprintf((stderr, "  ok   %s @ %s\n", "SetextBottom1", G->buf+G->pos));
  return 1;
  l1315:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "SetextBottom1", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_SetextHeading2(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "SetextHeading2"));
  {  int yypos1319= G->pos, yythunkpos1319= G->thunkpos;  if (!yy_RawLine(G)) { goto l1318; }  if (!yy_SetextBottom2(G)) { goto l1318; }  G->pos= yypos1319; G->thunkpos= yythunkpos1319;
  }  if (!yy_LocMarker(G)) { goto l1318; }  yyDo(G, yySet, -1, 0);  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l1318;
  {  int yypos1322= G->pos, yythunkpos1322= G->thunkpos;  if (!yy_Endline(G)) { goto l1322; }  goto l1318;
  l1322:;	  G->pos= yypos1322; G->thunkpos= yythunkpos1322;
  }  if (!yy_Inline(G)) { goto l1318; }
  l1320:;	
  {  int yypos1321= G->pos, yythunkpos1321= G->thunkpos;
  {  int yypos1323= G->pos, yythunkpos1323= G->thunkpos;  if (!yy_Endline(G)) { goto l1323; }  goto l1321;
  l1323:;	  G->pos= yypos1323; G->thunkpos= yythunkpos1323;
  }  if (!yy_Inline(G)) { goto l1321; }  goto l1320;
  l1321:;	  G->pos= yypos1321; G->thunkpos= yythunkpos1321;
  }
  {  int yypos1324= G->pos, yythunkpos1324= G->thunkpos;  if (!yy_Sp(G)) { goto l1324; }  goto l1325;
  l1324:;	  G->pos= yypos1324; G->thunkpos= yythunkpos1324;
  }
  l1325:;	  if (!yy_Newline(G)) { goto l1318; }  if (!yy_SetextBottom2(G)) { goto l1318; }  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l1318;  yyDo(G, yy_1_SetextHeading2, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "SetextHeading2", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l1318:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "SetextHeading2", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_SetextHeading1(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "SetextHeading1"));
  {  int yypos1327= G->pos, yythunkpos1327= G->thunkpos;  if (!yy_RawLine(G)) { goto l1326; }  if (!yy_SetextBottom1(G)) { goto l1326; }  G->pos= yypos1327; G->thunkpos= yythunkpos1327;
  }  if (!yy_LocMarker(G)) { goto l1326; }  yyDo(G, yySet, -1, 0);  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l1326;
  {  int yypos1330= G->pos, yythunkpos1330= G->thunkpos;  if (!yy_Endline(G)) { goto l1330; }  goto l1326;
  l1330:;	  G->pos= yypos1330; G->thunkpos= yythunkpos1330;
  }  if (!yy_Inline(G)) { goto l1326; }
  l1328:;	
  {  int yypos1329= G->pos, yythunkpos1329= G->thunkpos;
  {  int yypos1331= G->pos, yythunkpos1331= G->thunkpos;  if (!yy_Endline(G)) { goto l1331; }  goto l1329;
  l1331:;	  G->pos= yypos1331; G->thunkpos= yythunkpos1331;
  }  if (!yy_Inline(G)) { goto l1329; }  goto l1328;
  l1329:;	  G->pos= yypos1329; G->thunkpos= yythunkpos1329;
  }
  {  int yypos1332= G->pos, yythunkpos1332= G->thunkpos;  if (!yy_Sp(G)) { goto l1332; }  goto l1333;
  l1332:;	  G->pos= yypos1332; G->thunkpos= yythunkpos1332;
  }
  l1333:;	  if (!yy_Newline(G)) { goto l1326; }  if (!yy_SetextBottom1(G)) { goto l1326; }  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l1326;  yyDo(G, yy_1_SetextHeading1, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "SetextHeading1", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l1326:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "SetextHeading1", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_SetextHeading(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "SetextHeading"));
  {  int yypos1335= G->pos, yythunkpos1335= G->thunkpos;  if (!yy_SetextHeading1(G)) { goto l1336; }  goto l1335;
  l1336:;	  G->pos= yypos1335; G->thunkpos= yythunkpos1335;  if (!yy_SetextHeading2(G)) { goto l1334; }
  }
  l1335:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "SetextHeading", G->buf+G->pos));
  return 1;
  l1334:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "SetextHeading", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_AtxHeading(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "AtxHeading"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l1337;  if (!yy_AtxStart(G)) { goto l1337; }  yyDo(G, yySet, -1, 0);
  {  int yypos1338= G->pos, yythunkpos1338= G->thunkpos;  if (!yy_Sp(G)) { goto l1338; }  goto l1339;
  l1338:;	  G->pos= yypos1338; G->thunkpos= yythunkpos1338;
  }
  l1339:;	  if (!yy_AtxInline(G)) { goto l1337; }
  l1340:;	
  {  int yypos1341= G->pos, yythunkpos1341= G->thunkpos;  if (!yy_AtxInline(G)) { goto l1341; }  goto l1340;
  l1341:;	  G->pos= yypos1341; G->thunkpos= yythunkpos1341;
  }
  {  int yypos1342= G->pos, yythunkpos1342= G->thunkpos;
  {  int yypos1344= G->pos, yythunkpos1344= G->thunkpos;  if (!yy_Sp(G)) { goto l1344; }  goto l1345;
  l1344:;	  G->pos= yypos1344; G->thunkpos= yythunkpos1344;
  }
  l1345:;	
  l1346:;	
  {  int yypos1347= G->pos, yythunkpos1347= G->thunkpos;  if (!yymatchChar(G, '#')) goto l1347;  goto l1346;
  l1347:;	  G->pos= yypos1347; G->thunkpos= yythunkpos1347;
  }  if (!yy_Sp(G)) { goto l1342; }  goto l1343;
  l1342:;	  G->pos= yypos1342; G->thunkpos= yythunkpos1342;
  }
  l1343:;	  if (!yy_Newline(G)) { goto l1337; }  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l1337;  yyDo(G, yy_1_AtxHeading, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "AtxHeading", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l1337:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "AtxHeading", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_AtxStart(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "AtxStart"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l1348;
  {  int yypos1349= G->pos, yythunkpos1349= G->thunkpos;  if (!yymatchString(G, "######")) goto l1350;  goto l1349;
  l1350:;	  G->pos= yypos1349; G->thunkpos= yythunkpos1349;  if (!yymatchString(G, "#####")) goto l1351;  goto l1349;
  l1351:;	  G->pos= yypos1349; G->thunkpos= yythunkpos1349;  if (!yymatchString(G, "####")) goto l1352;  goto l1349;
  l1352:;	  G->pos= yypos1349; G->thunkpos= yythunkpos1349;  if (!yymatchString(G, "###")) goto l1353;  goto l1349;
  l1353:;	  G->pos= yypos1349; G->thunkpos= yythunkpos1349;  if (!yymatchString(G, "##")) goto l1354;  goto l1349;
  l1354:;	  G->pos= yypos1349; G->thunkpos= yythunkpos1349;  if (!yymatchChar(G, '#')) goto l1348;
  }
  l1349:;	  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l1348;  yyDo(G, yy_1_AtxStart, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "AtxStart", G->buf+G->pos));
  return 1;
  l1348:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "AtxStart", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Inline(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Inline"));
  {  int yypos1356= G->pos, yythunkpos1356= G->thunkpos;  if (!yy_Str(G)) { goto l1357; }  goto l1356;
  l1357:;	  G->pos= yypos1356; G->thunkpos= yythunkpos1356;  if (!yy_Endline(G)) { goto l1358; }  goto l1356;
  l1358:;	  G->pos= yypos1356; G->thunkpos= yythunkpos1356;  if (!yy_UlOrStarLine(G)) { goto l1359; }  goto l1356;
  l1359:;	  G->pos= yypos1356; G->thunkpos= yythunkpos1356;  if (!yy_Space(G)) { goto l1360; }  goto l1356;
  l1360:;	  G->pos= yypos1356; G->thunkpos= yythunkpos1356;  if (!yy_Strong(G)) { goto l1361; }  goto l1356;
  l1361:;	  G->pos= yypos1356; G->thunkpos= yythunkpos1356;  if (!yy_Emph(G)) { goto l1362; }  goto l1356;
  l1362:;	  G->pos= yypos1356; G->thunkpos= yythunkpos1356;  if (!yy_Image(G)) { goto l1363; }  goto l1356;
  l1363:;	  G->pos= yypos1356; G->thunkpos= yythunkpos1356;  if (!yy_Link(G)) { goto l1364; }  goto l1356;
  l1364:;	  G->pos= yypos1356; G->thunkpos= yythunkpos1356;  if (!yy_NoteReference(G)) { goto l1365; }  goto l1356;
  l1365:;	  G->pos= yypos1356; G->thunkpos= yythunkpos1356;  if (!yy_InlineNote(G)) { goto l1366; }  goto l1356;
  l1366:;	  G->pos= yypos1356; G->thunkpos= yythunkpos1356;  if (!yy_Code(G)) { goto l1367; }  goto l1356;
  l1367:;	  G->pos= yypos1356; G->thunkpos= yythunkpos1356;  if (!yy_RawHtml(G)) { goto l1368; }  goto l1356;
  l1368:;	  G->pos= yypos1356; G->thunkpos= yythunkpos1356;  if (!yy_Entity(G)) { goto l1369; }  goto l1356;
  l1369:;	  G->pos= yypos1356; G->thunkpos= yythunkpos1356;  if (!yy_EscapedChar(G)) { goto l1370; }  goto l1356;
  l1370:;	  G->pos= yypos1356; G->thunkpos= yythunkpos1356;  if (!yy_Symbol(G)) { goto l1355; }
  }
  l1356:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "Inline", G->buf+G->pos));
  return 1;
  l1355:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Inline", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Sp(GREG *G)
{
  yyprintf((stderr, "%s\n", "Sp"));
  l1372:;	
  {  int yypos1373= G->pos, yythunkpos1373= G->thunkpos;  if (!yy_Spacechar(G)) { goto l1373; }  goto l1372;
  l1373:;	  G->pos= yypos1373; G->thunkpos= yythunkpos1373;
  }
  yyprintf((stderr, "  ok   %s @ %s\n", "Sp", G->buf+G->pos));
  return 1;
}
YY_RULE(int) yy_Newline(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Newline"));
  {  int yypos1375= G->pos, yythunkpos1375= G->thunkpos;  if (!yymatchChar(G, '\n')) goto l1376;  goto l1375;
  l1376:;	  G->pos= yypos1375; G->thunkpos= yythunkpos1375;  if (!yymatchChar(G, '\r')) goto l1374;
  {  int yypos1377= G->pos, yythunkpos1377= G->thunkpos;  if (!yymatchChar(G, '\n')) goto l1377;  goto l1378;
  l1377:;	  G->pos= yypos1377; G->thunkpos= yythunkpos1377;
  }
  l1378:;	
  }
  l1375:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "Newline", G->buf+G->pos));
  return 1;
  l1374:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Newline", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_AtxInline(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "AtxInline"));
  {  int yypos1380= G->pos, yythunkpos1380= G->thunkpos;  if (!yy_Newline(G)) { goto l1380; }  goto l1379;
  l1380:;	  G->pos= yypos1380; G->thunkpos= yythunkpos1380;
  }
  {  int yypos1381= G->pos, yythunkpos1381= G->thunkpos;
  {  int yypos1382= G->pos, yythunkpos1382= G->thunkpos;  if (!yy_Sp(G)) { goto l1382; }  goto l1383;
  l1382:;	  G->pos= yypos1382; G->thunkpos= yythunkpos1382;
  }
  l1383:;	
  l1384:;	
  {  int yypos1385= G->pos, yythunkpos1385= G->thunkpos;  if (!yymatchChar(G, '#')) goto l1385;  goto l1384;
  l1385:;	  G->pos= yypos1385; G->thunkpos= yythunkpos1385;
  }  if (!yy_Sp(G)) { goto l1381; }  if (!yy_Newline(G)) { goto l1381; }  goto l1379;
  l1381:;	  G->pos= yypos1381; G->thunkpos= yythunkpos1381;
  }  if (!yy_Inline(G)) { goto l1379; }
  yyprintf((stderr, "  ok   %s @ %s\n", "AtxInline", G->buf+G->pos));
  return 1;
  l1379:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "AtxInline", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Inlines(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Inlines"));
  {  int yypos1389= G->pos, yythunkpos1389= G->thunkpos;
  {  int yypos1391= G->pos, yythunkpos1391= G->thunkpos;  if (!yy_Endline(G)) { goto l1391; }  goto l1390;
  l1391:;	  G->pos= yypos1391; G->thunkpos= yythunkpos1391;
  }  if (!yy_Inline(G)) { goto l1390; }  goto l1389;
  l1390:;	  G->pos= yypos1389; G->thunkpos= yythunkpos1389;  if (!yy_Endline(G)) { goto l1386; }
  {  int yypos1392= G->pos, yythunkpos1392= G->thunkpos;  if (!yy_Inline(G)) { goto l1386; }  G->pos= yypos1392; G->thunkpos= yythunkpos1392;
  }
  }
  l1389:;	
  l1387:;	
  {  int yypos1388= G->pos, yythunkpos1388= G->thunkpos;
  {  int yypos1393= G->pos, yythunkpos1393= G->thunkpos;
  {  int yypos1395= G->pos, yythunkpos1395= G->thunkpos;  if (!yy_Endline(G)) { goto l1395; }  goto l1394;
  l1395:;	  G->pos= yypos1395; G->thunkpos= yythunkpos1395;
  }  if (!yy_Inline(G)) { goto l1394; }  goto l1393;
  l1394:;	  G->pos= yypos1393; G->thunkpos= yythunkpos1393;  if (!yy_Endline(G)) { goto l1388; }
  {  int yypos1396= G->pos, yythunkpos1396= G->thunkpos;  if (!yy_Inline(G)) { goto l1388; }  G->pos= yypos1396; G->thunkpos= yythunkpos1396;
  }
  }
  l1393:;	  goto l1387;
  l1388:;	  G->pos= yypos1388; G->thunkpos= yythunkpos1388;
  }
  {  int yypos1397= G->pos, yythunkpos1397= G->thunkpos;  if (!yy_Endline(G)) { goto l1397; }  goto l1398;
  l1397:;	  G->pos= yypos1397; G->thunkpos= yythunkpos1397;
  }
  l1398:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "Inlines", G->buf+G->pos));
  return 1;
  l1386:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Inlines", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_NonindentSpace(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "NonindentSpace"));
  {  int yypos1400= G->pos, yythunkpos1400= G->thunkpos;  if (!yymatchString(G, "   ")) goto l1401;  goto l1400;
  l1401:;	  G->pos= yypos1400; G->thunkpos= yythunkpos1400;  if (!yymatchString(G, "  ")) goto l1402;  goto l1400;
  l1402:;	  G->pos= yypos1400; G->thunkpos= yythunkpos1400;  if (!yymatchChar(G, ' ')) goto l1403;  goto l1400;
  l1403:;	  G->pos= yypos1400; G->thunkpos= yythunkpos1400;  if (!yymatchString(G, "")) goto l1399;
  }
  l1400:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "NonindentSpace", G->buf+G->pos));
  return 1;
  l1399:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "NonindentSpace", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Plain(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Plain"));  if (!yy_Inlines(G)) { goto l1404; }
  yyprintf((stderr, "  ok   %s @ %s\n", "Plain", G->buf+G->pos));
  return 1;
  l1404:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Plain", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Para(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Para"));  if (!yy_NonindentSpace(G)) { goto l1405; }  if (!yy_Inlines(G)) { goto l1405; }  if (!yy_BlankLine(G)) { goto l1405; }
  l1406:;	
  {  int yypos1407= G->pos, yythunkpos1407= G->thunkpos;  if (!yy_BlankLine(G)) { goto l1407; }  goto l1406;
  l1407:;	  G->pos= yypos1407; G->thunkpos= yythunkpos1407;
  }
  yyprintf((stderr, "  ok   %s @ %s\n", "Para", G->buf+G->pos));
  return 1;
  l1405:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Para", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_StyleBlock(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "StyleBlock"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l1408;  if (!yy_LocMarker(G)) { goto l1408; }  yyDo(G, yySet, -1, 0);  if (!yy_InStyleTags(G)) { goto l1408; }  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l1408;
  l1409:;	
  {  int yypos1410= G->pos, yythunkpos1410= G->thunkpos;  if (!yy_BlankLine(G)) { goto l1410; }  goto l1409;
  l1410:;	  G->pos= yypos1410; G->thunkpos= yythunkpos1410;
  }  yyDo(G, yy_1_StyleBlock, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "StyleBlock", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l1408:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "StyleBlock", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlock(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "HtmlBlock"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l1411;  if (!yy_LocMarker(G)) { goto l1411; }  yyDo(G, yySet, -1, 0);
  {  int yypos1412= G->pos, yythunkpos1412= G->thunkpos;  if (!yy_HtmlBlockInTags(G)) { goto l1413; }  goto l1412;
  l1413:;	  G->pos= yypos1412; G->thunkpos= yythunkpos1412;  if (!yy_HtmlComment(G)) { goto l1414; }  goto l1412;
  l1414:;	  G->pos= yypos1412; G->thunkpos= yythunkpos1412;  if (!yy_HtmlBlockSelfClosing(G)) { goto l1411; }
  }
  l1412:;	  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l1411;  if (!yy_BlankLine(G)) { goto l1411; }
  l1415:;	
  {  int yypos1416= G->pos, yythunkpos1416= G->thunkpos;  if (!yy_BlankLine(G)) { goto l1416; }  goto l1415;
  l1416:;	  G->pos= yypos1416; G->thunkpos= yythunkpos1416;
  }  yyDo(G, yy_1_HtmlBlock, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlock", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l1411:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlock", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_BulletList(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "BulletList"));
  {  int yypos1418= G->pos, yythunkpos1418= G->thunkpos;  if (!yy_Bullet(G)) { goto l1417; }  G->pos= yypos1418; G->thunkpos= yythunkpos1418;
  }
  {  int yypos1419= G->pos, yythunkpos1419= G->thunkpos;  if (!yy_ListTight(G)) { goto l1420; }  goto l1419;
  l1420:;	  G->pos= yypos1419; G->thunkpos= yythunkpos1419;  if (!yy_ListLoose(G)) { goto l1417; }
  }
  l1419:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "BulletList", G->buf+G->pos));
  return 1;
  l1417:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "BulletList", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_OrderedList(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "OrderedList"));
  {  int yypos1422= G->pos, yythunkpos1422= G->thunkpos;  if (!yy_Enumerator(G)) { goto l1421; }  G->pos= yypos1422; G->thunkpos= yythunkpos1422;
  }
  {  int yypos1423= G->pos, yythunkpos1423= G->thunkpos;  if (!yy_ListTight(G)) { goto l1424; }  goto l1423;
  l1424:;	  G->pos= yypos1423; G->thunkpos= yythunkpos1423;  if (!yy_ListLoose(G)) { goto l1421; }
  }
  l1423:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "OrderedList", G->buf+G->pos));
  return 1;
  l1421:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "OrderedList", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Heading(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Heading"));
  {  int yypos1426= G->pos, yythunkpos1426= G->thunkpos;  if (!yy_SetextHeading(G)) { goto l1427; }  goto l1426;
  l1427:;	  G->pos= yypos1426; G->thunkpos= yythunkpos1426;  if (!yy_AtxHeading(G)) { goto l1425; }
  }
  l1426:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "Heading", G->buf+G->pos));
  return 1;
  l1425:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Heading", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HorizontalRule(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HorizontalRule"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l1428;  if (!yy_NonindentSpace(G)) { goto l1428; }
  {  int yypos1429= G->pos, yythunkpos1429= G->thunkpos;  if (!yymatchChar(G, '*')) goto l1430;  if (!yy_Sp(G)) { goto l1430; }  if (!yymatchChar(G, '*')) goto l1430;  if (!yy_Sp(G)) { goto l1430; }  if (!yymatchChar(G, '*')) goto l1430;
  l1431:;	
  {  int yypos1432= G->pos, yythunkpos1432= G->thunkpos;  if (!yy_Sp(G)) { goto l1432; }  if (!yymatchChar(G, '*')) goto l1432;  goto l1431;
  l1432:;	  G->pos= yypos1432; G->thunkpos= yythunkpos1432;
  }  goto l1429;
  l1430:;	  G->pos= yypos1429; G->thunkpos= yythunkpos1429;  if (!yymatchChar(G, '-')) goto l1433;  if (!yy_Sp(G)) { goto l1433; }  if (!yymatchChar(G, '-')) goto l1433;  if (!yy_Sp(G)) { goto l1433; }  if (!yymatchChar(G, '-')) goto l1433;
  l1434:;	
  {  int yypos1435= G->pos, yythunkpos1435= G->thunkpos;  if (!yy_Sp(G)) { goto l1435; }  if (!yymatchChar(G, '-')) goto l1435;  goto l1434;
  l1435:;	  G->pos= yypos1435; G->thunkpos= yythunkpos1435;
  }  goto l1429;
  l1433:;	  G->pos= yypos1429; G->thunkpos= yythunkpos1429;  if (!yymatchChar(G, '_')) goto l1428;  if (!yy_Sp(G)) { goto l1428; }  if (!yymatchChar(G, '_')) goto l1428;  if (!yy_Sp(G)) { goto l1428; }  if (!yymatchChar(G, '_')) goto l1428;
  l1436:;	
  {  int yypos1437= G->pos, yythunkpos1437= G->thunkpos;  if (!yy_Sp(G)) { goto l1437; }  if (!yymatchChar(G, '_')) goto l1437;  goto l1436;
  l1437:;	  G->pos= yypos1437; G->thunkpos= yythunkpos1437;
  }
  }
  l1429:;	  if (!yy_Sp(G)) { goto l1428; }  if (!yy_Newline(G)) { goto l1428; }  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l1428;  if (!yy_BlankLine(G)) { goto l1428; }
  l1438:;	
  {  int yypos1439= G->pos, yythunkpos1439= G->thunkpos;  if (!yy_BlankLine(G)) { goto l1439; }  goto l1438;
  l1439:;	  G->pos= yypos1439; G->thunkpos= yythunkpos1439;
  }  yyDo(G, yy_1_HorizontalRule, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "HorizontalRule", G->buf+G->pos));
  return 1;
  l1428:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HorizontalRule", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Reference(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 3, 0);
  yyprintf((stderr, "%s\n", "Reference"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l1440;  if (!yy_LocMarker(G)) { goto l1440; }  yyDo(G, yySet, -3, 0);  if (!yy_NonindentSpace(G)) { goto l1440; }
  {  int yypos1441= G->pos, yythunkpos1441= G->thunkpos;  if (!yymatchString(G, "[]")) goto l1441;  goto l1440;
  l1441:;	  G->pos= yypos1441; G->thunkpos= yythunkpos1441;
  }  if (!yy_Label(G)) { goto l1440; }  yyDo(G, yySet, -2, 0);  if (!yymatchChar(G, ':')) goto l1440;  if (!yy_Spnl(G)) { goto l1440; }  if (!yy_RefSrc(G)) { goto l1440; }  yyDo(G, yySet, -1, 0);  if (!yy_RefTitle(G)) { goto l1440; }  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l1440;  if (!yy_BlankLine(G)) { goto l1440; }
  l1442:;	
  {  int yypos1443= G->pos, yythunkpos1443= G->thunkpos;  if (!yy_BlankLine(G)) { goto l1443; }  goto l1442;
  l1443:;	  G->pos= yypos1443; G->thunkpos= yythunkpos1443;
  }  yyDo(G, yy_1_Reference, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "Reference", G->buf+G->pos));  yyDo(G, yyPop, 3, 0);
  return 1;
  l1440:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Reference", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Note(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Note"));  yyText(G, G->begin, G->end);  if (!( EXT(pmh_EXT_NOTES) )) goto l1444;  if (!yy_NonindentSpace(G)) { goto l1444; }  if (!yy_RawNoteReference(G)) { goto l1444; }  if (!yymatchChar(G, ':')) goto l1444;  if (!yy_Sp(G)) { goto l1444; }  if (!yy_RawNoteBlock(G)) { goto l1444; }
  l1445:;	
  {  int yypos1446= G->pos, yythunkpos1446= G->thunkpos;
  {  int yypos1447= G->pos, yythunkpos1447= G->thunkpos;  if (!yy_Indent(G)) { goto l1446; }  G->pos= yypos1447; G->thunkpos= yythunkpos1447;
  }  if (!yy_RawNoteBlock(G)) { goto l1446; }  goto l1445;
  l1446:;	  G->pos= yypos1446; G->thunkpos= yythunkpos1446;
  }
  yyprintf((stderr, "  ok   %s @ %s\n", "Note", G->buf+G->pos));
  return 1;
  l1444:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Note", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Verbatim(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "Verbatim"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l1448;  if (!yy_LocMarker(G)) { goto l1448; }  yyDo(G, yySet, -1, 0);  if (!yy_VerbatimChunk(G)) { goto l1448; }
  l1449:;	
  {  int yypos1450= G->pos, yythunkpos1450= G->thunkpos;  if (!yy_VerbatimChunk(G)) { goto l1450; }  goto l1449;
  l1450:;	  G->pos= yypos1450; G->thunkpos= yythunkpos1450;
  }  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l1448;  yyDo(G, yy_1_Verbatim, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "Verbatim", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l1448:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Verbatim", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_BlockQuote(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "BlockQuote"));  if (!yy_BlockQuoteRaw(G)) { goto l1451; }  yyDo(G, yySet, -1, 0);  yyDo(G, yy_1_BlockQuote, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "BlockQuote", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l1451:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "BlockQuote", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_BlankLine(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "BlankLine"));  if (!yy_Sp(G)) { goto l1452; }  if (!yy_Newline(G)) { goto l1452; }
  yyprintf((stderr, "  ok   %s @ %s\n", "BlankLine", G->buf+G->pos));
  return 1;
  l1452:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "BlankLine", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_LocMarker(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "LocMarker"));
  {  int yypos1454= G->pos, yythunkpos1454= G->thunkpos;  if (!yymatchDot(G)) goto l1453;  G->pos= yypos1454; G->thunkpos= yythunkpos1454;
  }  yyDo(G, yy_1_LocMarker, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "LocMarker", G->buf+G->pos));
  return 1;
  l1453:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "LocMarker", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Block(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Block"));
  l1456:;	
  {  int yypos1457= G->pos, yythunkpos1457= G->thunkpos;  if (!yy_BlankLine(G)) { goto l1457; }  goto l1456;
  l1457:;	  G->pos= yypos1457; G->thunkpos= yythunkpos1457;
  }
  {  int yypos1458= G->pos, yythunkpos1458= G->thunkpos;  if (!yy_BlockQuote(G)) { goto l1459; }  goto l1458;
  l1459:;	  G->pos= yypos1458; G->thunkpos= yythunkpos1458;  if (!yy_Verbatim(G)) { goto l1460; }  goto l1458;
  l1460:;	  G->pos= yypos1458; G->thunkpos= yythunkpos1458;  if (!yy_Note(G)) { goto l1461; }  goto l1458;
  l1461:;	  G->pos= yypos1458; G->thunkpos= yythunkpos1458;  if (!yy_Reference(G)) { goto l1462; }  goto l1458;
  l1462:;	  G->pos= yypos1458; G->thunkpos= yythunkpos1458;  if (!yy_HorizontalRule(G)) { goto l1463; }  goto l1458;
  l1463:;	  G->pos= yypos1458; G->thunkpos= yythunkpos1458;  if (!yy_Heading(G)) { goto l1464; }  goto l1458;
  l1464:;	  G->pos= yypos1458; G->thunkpos= yythunkpos1458;  if (!yy_OrderedList(G)) { goto l1465; }  goto l1458;
  l1465:;	  G->pos= yypos1458; G->thunkpos= yythunkpos1458;  if (!yy_BulletList(G)) { goto l1466; }  goto l1458;
  l1466:;	  G->pos= yypos1458; G->thunkpos= yythunkpos1458;  if (!yy_HtmlBlock(G)) { goto l1467; }  goto l1458;
  l1467:;	  G->pos= yypos1458; G->thunkpos= yythunkpos1458;  if (!yy_StyleBlock(G)) { goto l1468; }  goto l1458;
  l1468:;	  G->pos= yypos1458; G->thunkpos= yythunkpos1458;  if (!yy_Para(G)) { goto l1469; }  goto l1458;
  l1469:;	  G->pos= yypos1458; G->thunkpos= yythunkpos1458;  if (!yy_Plain(G)) { goto l1455; }
  }
  l1458:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "Block", G->buf+G->pos));
  return 1;
  l1455:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Block", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Doc(GREG *G)
{
  yyprintf((stderr, "%s\n", "Doc"));
  l1471:;	
  {  int yypos1472= G->pos, yythunkpos1472= G->thunkpos;  if (!yy_Block(G)) { goto l1472; }  goto l1471;
  l1472:;	  G->pos= yypos1472; G->thunkpos= yythunkpos1472;
  }
  yyprintf((stderr, "  ok   %s @ %s\n", "Doc", G->buf+G->pos));
  return 1;
}

#ifndef YY_PART

typedef int (*yyrule)(GREG *G);

YY_PARSE(int) YY_NAME(parse_from)(GREG *G, yyrule yystart)
{
  int yyok;
  if (!G->buflen)
    {
      G->buflen= YY_BUFFER_START_SIZE;
      G->buf= (char *)YY_ALLOC(G->buflen, G->data);
      G->textlen= YY_BUFFER_START_SIZE;
      G->text= (char *)YY_ALLOC(G->textlen, G->data);
      G->thunkslen= YY_STACK_SIZE;
      G->thunks= (yythunk *)YY_ALLOC(sizeof(yythunk) * G->thunkslen, G->data);
      G->valslen= YY_STACK_SIZE;
      G->vals= (YYSTYPE*)YY_ALLOC(sizeof(YYSTYPE) * G->valslen, G->data);
      G->begin= G->end= G->pos= G->limit= G->thunkpos= 0;
    }
  G->pos = 0;
  G->begin= G->end= G->pos;
  G->thunkpos= 0;
  G->val= G->vals;
  yyok= yystart(G);
  if (yyok) yyDone(G);
  yyCommit(G);
  return yyok;
  (void)yyrefill(NULL);
  (void)yymatchDot(NULL);
  (void)yymatchChar(NULL, 0);
  (void)yymatchString(NULL, NULL);
  (void)yymatchClass(NULL, NULL);
  (void)yyDo(NULL, NULL, 0, 0);
  (void)yyText(NULL, 0, 0);
  (void)yyDone(NULL);
  (void)yyCommit(NULL);
  (void)yyAccept(NULL, 0);
  (void)yyPush(NULL, NULL, 0, NULL, NULL);
  (void)yyPop(NULL, NULL, 0, NULL, NULL);
  (void)yySet(NULL, NULL, 0, NULL, NULL);
}

YY_PARSE(int) YY_NAME(parse)(GREG *G)
{
  return YY_NAME(parse_from)(G, yy_Doc);
}

YY_PARSE(GREG *) YY_NAME(parse_new)(YY_XTYPE data)
{
  GREG *G = (GREG *)YY_CALLOC(1, sizeof(GREG), G->data);
  G->data = data;
  return G;
}

YY_PARSE(void) YY_NAME(parse_free)(GREG *G)
{
  YY_FREE(G->buf);
  YY_FREE(G->text);
  YY_FREE(G->thunks);
  YY_FREE(G->vals);
  YY_FREE(G);
}

#endif




/* PEG Markdown Highlight
 * Copyright 2011-2012 Ali Rantakari -- http://hasseg.org
 * Licensed under the GPL2+ and MIT licenses (see LICENSE for more info).
 * 
 * pmh_parser_foot.c
 * 
 * Code to be appended to the end of the parser code generated from the
 * PEG grammar.
 */


static void _parse(parser_data *p_data, yyrule start_rule)
{
    GREG *g = YY_NAME(parse_new)(p_data);
    if (start_rule == NULL)
        YY_NAME(parse)(g);
    else
        YY_NAME(parse_from)(g, start_rule);
    YY_NAME(parse_free)(g);
    
    pmh_PRINTF("\n\n");
}

static void parse_markdown(parser_data *p_data)
{
    pmh_PRINTF("\nPARSING DOCUMENT: ");
    
    _parse(p_data, NULL);
}

static void parse_references(parser_data *p_data)
{
    pmh_PRINTF("\nPARSING REFERENCES: ");
    
    p_data->parsing_only_references = true;
    _parse(p_data, yy_References);
    p_data->parsing_only_references = false;
    
    p_data->references = p_data->head_elems[pmh_REFERENCE];
    p_data->head_elems[pmh_REFERENCE] = NULL;
}

